@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24114 $
+ * $Revision: 24504 $
  * $Author: rjongbloed $
- * $Date: 2010-03-11 20:29:31 -0600 (Thu, 11 Mar 2010) $
+ * $Date: 2010-06-17 17:49:01 -0500 (Thu, 17 Jun 2010) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGIN_H
@@ -230,7 +230,7 @@ enum PluginCodec_OptionMerge {
   PluginCodec_NotEqualMerge,
   PluginCodec_AlwaysMerge,
   PluginCodec_CustomMerge,
-  PluginCodec_NumOptionMerge,
+  PluginCodec_IntersectionMerge,
 
   PluginCodec_AndMerge = PluginCodec_MinMerge,
   PluginCodec_OrMerge  = PluginCodec_MaxMerge
