@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21720 $
- * $Author: rjongbloed $
- * $Date: 2008-12-09 01:40:33 +0000 (Tue, 09 Dec 2008) $
+ * $Revision: 23525 $
+ * $Author: csoutheren $
+ * $Date: 2009-09-24 01:02:24 -0500 (Thu, 24 Sep 2009) $
  */
 
 #ifndef OPAL_H323_H323EP_H
@@ -49,6 +49,7 @@
 #include <h323/h323con.h>
 #include <h323/h323caps.h>
 #include <h323/h235auth.h>
+#include <asn/h225.h>
 
 #if OPAL_H460
 #include <h460/h4601.h>
@@ -140,7 +141,7 @@ class H323EndPoint : public OpalRTPEndPoint
 
        The default behaviour is pure.
      */
-    virtual PBoolean MakeConnection(
+    virtual PSafePtr<OpalConnection> MakeConnection(
       OpalCall & call,                  ///<  Owner of connection
       const PString & party,            ///<  Remote party to call
       void * userData  = NULL,          ///<  Arbitrary data to pass to connection
@@ -168,6 +169,19 @@ class H323EndPoint : public OpalRTPEndPoint
     virtual void SetH221NonStandardInfo(
       H225_H221NonStandard & info
     ) const;
+
+    /**Set the Gateway supported protocol default always H.323
+      */
+    virtual bool SetGatewaySupportedProtocol(
+      H225_ArrayOf_SupportedProtocols & protocols
+    ) const;
+
+    /**Set the gateway prefixes 
+       Override this to set the acceptable prefixes to the gatekeeper
+      */
+    virtual bool OnSetGatewayPrefixes(
+      PStringList & prefixes
+    ) const;
   //@}
 
 
@@ -278,6 +292,9 @@ class H323EndPoint : public OpalRTPEndPoint
        If the endpoint is already registered with a gatekeeper that meets
        the same criteria then the gatekeeper is not changed, otherwise it is
        deleted (with unregistration) and new one created and registered to.
+
+       Note that a gatekeeper address of "*" is treated like an empty string
+       resulting in gatekeeper discovery.
      */
     PBoolean UseGatekeeper(
       const PString & address = PString::Empty(),     ///<  Address of gatekeeper to use.
@@ -503,7 +520,8 @@ class H323EndPoint : public OpalRTPEndPoint
     PBoolean ParsePartyName(
       const PString & party,          ///<  Party name string.
       PString & alias,                ///<  Parsed alias name
-      H323TransportAddress & address  ///<  Parsed transport address
+      H323TransportAddress & address, ///<  Parsed transport address
+      OpalConnection::StringOptions * stringOptions = NULL ///< String options parsed from party name
     );
 
     /**Find a connection that uses the specified token.
@@ -532,34 +550,36 @@ class H323EndPoint : public OpalRTPEndPoint
                                    H323SignalPDU & setupPDU);
 
     /**Adjust call proceeding PDU being sent. This function is called from
-    the OnReceivedSignalSetup() function before it sends the Call
-    Proceeding PDU. It gives an opportunity for an application to alter
-    the request before transmission to the other endpoint. If this function
-    returns PFalse then the Call Proceeding PDU is not sent at all.
+       the OnReceivedSignalSetup() function before it sends the Call
+       Proceeding PDU. It gives an opportunity for an application to alter
+       the request before transmission to the other endpoint. If this function
+       returns PFalse then the Call Proceeding PDU is not sent at all.
 
-    The default behaviour simply returns PTrue.
-    @param connection the connection associated to the call proceeding
-    @param callProceedingPDU the call processding to modify
-    @return if false, do no send the connect pdu  
+       The default behaviour simply returns PTrue.
+       @param connection the connection associated to the call proceeding
+       @param callProceedingPDU the call processding to modify
+       @return if false, do no send the connect pdu  
      */
-    virtual PBoolean OnSendCallProceeding(H323Connection & connection,
-                                      H323SignalPDU & callProceedingPDU
-                                     );
-    
+    virtual PBoolean OnSendCallProceeding(
+      H323Connection & connection,
+      H323SignalPDU & callProceedingPDU
+    );
+
     /**Adjust call connect PDU being sent. This function is called from
-    the H323Connection::SetConnected function before it sends the connect  PDU. 
-    It gives an opportunity for an application to alter
-    the request before transmission to the other endpoint. If this function
-    returns PFalse then the connect PDU is not sent at all.
+       the H323Connection::SetConnected function before it sends the connect  PDU. 
+       It gives an opportunity for an application to alter
+       the request before transmission to the other endpoint. If this function
+       returns PFalse then the connect PDU is not sent at all.
 
-    The default behaviour simply returns PTrue.
-    @param connection the connection associated to the connect
-    @param connectPDU the connect to modify
-    @return if false, do no send the connect pdu  
+       The default behaviour simply returns PTrue.
+       @param connection the connection associated to the connect
+       @param connectPDU the connect to modify
+       @return if false, do no send the connect pdu  
      */
-    virtual PBoolean OnSendConnect(H323Connection & connection,
-                               H323SignalPDU & connectPDU
-                              );
+    virtual PBoolean OnSendConnect(
+      H323Connection & connection,
+      H323SignalPDU & connectPDU
+    );
     
     /**Call back for incoming call.
        This function is called from the OnReceivedSignalSetup() function
@@ -585,8 +605,8 @@ class H323EndPoint : public OpalRTPEndPoint
        The default behaviour simply returns PTrue.
       */
     virtual PBoolean OnOutgoingCall(
-        H323Connection & conn, 
-        const H323SignalPDU & connectPDU
+      H323Connection & conn, 
+      const H323SignalPDU & connectPDU
     );
 
     /**Handle a connection transfer.
@@ -653,19 +673,20 @@ class H323EndPoint : public OpalRTPEndPoint
         can be used by the application to alter the alerting Pdu
         @return if PFalse, then the alerting is not sent
      */
-      
-    virtual PBoolean OnSendAlerting(H323Connection & connection,  ///< onnection that was established
-                                H323SignalPDU & alerting,     ///< Alerting PDU to modify
-                                const PString & calleeName,   ///< Name of endpoint being alerted.
-                                PBoolean withMedia                ///< Open media with alerting
-                               );
-        
+    virtual PBoolean OnSendAlerting(
+      H323Connection & connection,  ///< onnection that was established
+      H323SignalPDU & alerting,     ///< Alerting PDU to modify
+      const PString & calleeName,   ///< Name of endpoint being alerted.
+      PBoolean withMedia                ///< Open media with alerting
+    );
+
     /** A call back function when the alerting has been sent, can be used by the application 
         to send the connect as soon as the alerting has been sent.
      */
-      
-    virtual PBoolean OnSentAlerting(H323Connection & connection);
-    
+    virtual PBoolean OnSentAlerting(
+      H323Connection & connection
+    );
+
     /**A call back function when a connection indicates it is to be forwarded.
        An H323 application may handle this call back so it can make
        complicated decisions on if the call forward ius to take place. If it
@@ -1069,6 +1090,18 @@ class H323EndPoint : public OpalRTPEndPoint
      */
     void SetInitialBandwidth(unsigned bandwidth) { initialBandwidth = bandwidth; }
 
+#if OPAL_H239
+    /**Get the default H.239 control capability.
+     */
+    bool GetDefaultH239Control() const { return m_defaultH239Control; }
+
+    /**Set the default H.239 control capability.
+     */
+    void SetDefaultH239Control(
+      bool on   ///< H.239 control capability is to be sent to remote
+    ) { m_defaultH239Control = on; }
+#endif
+
     /**Called when an outgoing PDU requires a feature set
      */
     virtual PBoolean OnSendFeatureSet(unsigned, H225_FeatureSet &);
@@ -1312,7 +1345,7 @@ class H323EndPoint : public OpalRTPEndPoint
 
   protected:
     bool InternalCreateGatekeeper(H323Transport * transport);
-    PBoolean InternalMakeCall(
+    H323Connection * InternalMakeCall(
       OpalCall & call,
       const PString & existingToken,           ///<  Existing connection to be transferred
       const PString & callIdentity,            ///<  Call identity of the secondary call (if it exists)
@@ -1334,12 +1367,16 @@ class H323EndPoint : public OpalRTPEndPoint
     PBoolean        canDisplayAmountString;
     PBoolean        canEnforceDurationLimit;
 #if OPAL_H450
-    unsigned    callIntrusionProtectionLevel;
+    unsigned        callIntrusionProtectionLevel;
 #endif
 
-    TerminalTypes terminalType;
+    TerminalTypes   terminalType;
 
-    PBoolean          clearCallOnRoundTripFail;
+#if OPAL_H239
+    bool            m_defaultH239Control;
+#endif
+
+    PBoolean        clearCallOnRoundTripFail;
 
     // Some more configuration variables, rarely changed.
     PTimeInterval signallingChannelCallTimeout;
