@@ -26,25 +26,33 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22553 $
- * $Author: csoutheren $
- * $Date: 2009-05-07 08:56:06 +0000 (Thu, 07 May 2009) $
+ * $Revision: 23980 $
+ * $Author: rjongbloed $
+ * $Date: 2010-01-24 19:30:23 -0600 (Sun, 24 Jan 2010) $
  */
 
 #ifndef OPAL_OPAL_H
 #define OPAL_OPAL_H
 
+#include <stdlib.h>
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-/** /file opal.h
-    This file contains an API for accessing the OPAL system via "C" language interface.
-    A contrained set of functions are declared which allows the library to be easily
-    "late bound" using Windows LoadLibrary() or Unix dlopen() at run time.
+/**\file opal.h
+   \brief This file contains a simplified API to the OPAL system. It provides a
+    pure "C" language interface as will as a very simple C++ class and a
+    contrained set of functions for "late binding".
 
-    You may look at the sample code in opal/samples/c_api/main.c for an example of how
-    to do late binding.
+    It should be noted that the simplified API, is sill not very simple. There are
+    complexities invoved that cannot be avoided. However, this API does remove
+    some issues with the full API such as multi-threading and advanced C++ concepts.
+
+    The other major feature of this API is the ability to be  easily "late bound"
+    using Windows LoadLibrary() or Unix dlopen() at run time. You may look at the
+    sample code in opal/samples/c_api/main.c for an example of how to do late binding.
 
     Late binding also allows for easier integration of OPAL fucntionality to
     interpreted languages such as Java, Perl etc. Systems like "swig" may be used to
@@ -53,6 +61,10 @@ extern "C" {
     To make the above easier, there are only four functions: OpalInitialise(),
     OpalShutDown(), OpalGetMessage() and OpalSendMessage(). All commands to OPAL and
     indications back from OPAL are done through the latter two functions.
+
+    This API also provides a basic C++ class OpalContext, which may be used for C++
+    programmers that do not wish to learn the large number of classes in the full
+    API. At the cost of minimal extensibility and control.
   */
 
 #ifdef _WIN32
@@ -61,12 +73,14 @@ extern "C" {
   #define OPAL_EXPORT
 #endif
 
+/// Handle to initialised OPAL instance.
 typedef struct OpalHandleStruct * OpalHandle;
 
+
 typedef struct OpalMessage OpalMessage;
 
-
-#define OPAL_C_API_VERSION 18
+/// Current API version
+#define OPAL_C_API_VERSION 21
 
 
 ///////////////////////////////////////
@@ -82,20 +96,21 @@ typedef struct OpalMessage OpalMessage;
     The C string options are space separated tokens indicating various options
     to be enabled, for example the protocols to be available. NULL or an empty
     string will load all available protocols. The current protocol tokens are:
-
+        <code>
         sip sips h323 h323s iax2 pc local pots pstn ivr
-
+        </code>
     The above protocols are in priority order, so if a protocol is not
     explicitly in the address, then the first one of the opposite "category"
     s used. There are two categories, network protocols (sip, h323, iax & pstn)
     and non-network protocols (pc, local, pots & ivr).
 
     Additional options are:
-
-        TraceLevel=1     Level for tracing.
-        TraceAppend      Append to the trace file.
-        TraceFile="name" Set the filename for trace output. Note quotes are
-                         required if spaces are in filename.
+        <table border=0>
+        <tr><td>TraceLevel=1     <td>Level for tracing.
+        <tr><td>TraceAppend      <td>Append to the trace file.
+        <tr><td>TraceFile="name" <td>Set the filename for trace output. Note quotes are
+                                     required if spaces are in filename.
+        </table>
     It should also be noted that there must not be spaces around the '=' sign
     in the above options.
 
@@ -104,6 +119,7 @@ typedef struct OpalMessage OpalMessage;
     the library.
 
     Example:
+      <code>
       OpalHandle hOPAL;
       unsigned   version;
 
@@ -117,7 +133,7 @@ typedef struct OpalMessage OpalMessage;
         fputs("Could not initialise OPAL\n", stderr);
         return false;
       }
-
+      </code>
   */
 OpalHandle OPAL_EXPORT OpalInitialise(unsigned * version, const char * options);
 
@@ -138,7 +154,9 @@ typedef OpalHandle (OPAL_EXPORT *OpalInitialiseFunction)(unsigned * version, con
     must be the handle returned by OpalInitialise().
 
     Example:
+      <code>
       OpalShutDown(hOPAL);
+      </code>
   */
 void OPAL_EXPORT OpalShutDown(OpalHandle opal);
 
@@ -172,6 +190,7 @@ typedef void (OPAL_EXPORT *OpalShutDownFunction)(OpalHandle opal);
     will break from its block and return NULL.
 
     Example:
+      <code>
       OpalMessage * message;
         
       while ((message = OpalGetMessage(hOPAL, timeout)) != NULL) {
@@ -188,6 +207,7 @@ typedef void (OPAL_EXPORT *OpalShutDownFunction)(OpalHandle opal);
         }
         FreeMessageFunction(message);
       }
+      </code>
   */
 OpalMessage * OPAL_EXPORT OpalGetMessage(OpalHandle opal, unsigned timeout);
 
@@ -227,6 +247,7 @@ typedef OpalMessage * (OPAL_EXPORT *OpalGetMessageFunction)(OpalHandle opal, uns
     The returned message must be disposed of by a call to OpalFreeMessage().
 
     Example:
+      <code>
       void SendCommand(OpalMessage * command)
       {
         OpalMessage * response;
@@ -234,13 +255,14 @@ typedef OpalMessage * (OPAL_EXPORT *OpalGetMessageFunction)(OpalHandle opal, uns
           puts("OPAL not initialised.");
         else if (response->m_type != OpalIndCommandError)
           HandleResponse(response);
-        else if (response->m_param.m_commandError == NULL || *response->m_param.m_commandError == '\0')
+        else if (response->m_param.m_commandError == NULL || *response->m_param.m_commandError == '\\0')
           puts("OPAL error.");
         else
           printf("OPAL error: %s\n", response->m_param.m_commandError);
 
         FreeMessageFunction(response);
       }
+      </code>
   */
 OpalMessage * OPAL_EXPORT OpalSendMessage(OpalHandle opal, const OpalMessage * message);
 
@@ -275,14 +297,14 @@ typedef void (OPAL_EXPORT *OpalFreeMessageFunction)(OpalMessage * message);
 
 ///////////////////////////////////////
 
-#define OPAL_PREFIX_H323  "h323"
-#define OPAL_PREFIX_SIP   "sip"
-#define OPAL_PREFIX_IAX2  "iax2"
-#define OPAL_PREFIX_PCSS  "pc"
-#define OPAL_PREFIX_LOCAL "local"
-#define OPAL_PREFIX_POTS  "pots"
-#define OPAL_PREFIX_PSTN  "pstn"
-#define OPAL_PREFIX_IVR   "ivr"
+#define OPAL_PREFIX_H323  "h323"    ///< H.323 Protocol supported string for OpalInitialise()
+#define OPAL_PREFIX_SIP   "sip"     ///< SIP Protocol supported string for OpalInitialise()
+#define OPAL_PREFIX_IAX2  "iax2"    ///< IAX2 Protocol supported string for OpalInitialise()
+#define OPAL_PREFIX_PCSS  "pc"      ///< PC sound system supported string for OpalInitialise()
+#define OPAL_PREFIX_LOCAL "local"   ///< Local endpoint supported string for OpalInitialise()
+#define OPAL_PREFIX_POTS  "pots"    ///< Plain Old Telephone System supported string for OpalInitialise()
+#define OPAL_PREFIX_PSTN  "pstn"    ///< Public Switched Network supported string for OpalInitialise()
+#define OPAL_PREFIX_IVR   "ivr"     ///< Interactive Voice Response supported string for OpalInitialise()
 
 #define OPAL_PREFIX_ALL OPAL_PREFIX_H323  " " \
                         OPAL_PREFIX_SIP   " " \
@@ -377,6 +399,8 @@ typedef enum OpalMessageType {
                                     level error, in which case another host may be tried, and that the
                                     responsibility for finalising the call has moved "upstream". See the
                                     OpalParamSetUpCall structure for more information. */
+  OpalCmdAlerting,              /**<Send an indication to the remote that we are "ringing". The OpalMessage
+                                    m_callToken field indicates which call is alerting.  */
   OpalMessageTypeCount
 } OpalMessageType;
 
@@ -406,6 +430,13 @@ typedef enum OpalEchoCancelMode {
     Returns size of data actually read or written, or -1 if there is an error
     and the media stream should be shut down.
 
+    The "write" function, which is taking data from a remote and providing it
+    to the "C" application for writing, should not be assumed to have a one to
+    one correspondence with RTP packets. The OPAL jiter buffer may insert
+    "silence" data for missing or too late packets. In this case the function
+    is called with the size parameter equal to zero. It is up to the
+    application what it does in that circumstance.
+
     Note that this function will be called in the context of different threads
     so the user must take care of any mutex and synchonisation issues.
  */
@@ -445,7 +476,11 @@ typedef int (*OpalMessageAvailableFunction)(
 
 
 /**Type code the media data call back functions data type.
-   This is used by the OpalCmdSetGeneralParameters command in the OpalParamGeneral structure.
+   This is used by the OpalCmdSetGeneralParameters command in the
+   OpalParamGeneral structure.
+
+   This controls if the whole RTP data frame or just the paylaod part
+   is passed to the read/write function.
   */
 typedef enum OpalMediaDataType {
   OpalMediaDataNoChange,      /**< No change to the media data type. */
@@ -456,10 +491,37 @@ typedef enum OpalMediaDataType {
 } OpalMediaDataType;
 
 
+/**Timing mode for the media data call back functions data type.
+   This is used by the OpalCmdSetGeneralParameters command in the
+   OpalParamGeneral structure.
+
+   This controls if the read/write function is in control of the real time
+   aspects of the media flow. If synchronous then the read/write function
+   is expected to handle the real time "pacing" of the read or written data.
+
+   Note this is important both for reads and writes. For example in
+   synchronous mode you cannot simply read from a file and send, or you will
+   likely overrun the remotes buffers. Similarly for writing to a file, the
+   correct operation of the OPAL jitter buffer is dependent on it not being
+   drained too fast by the "write" function.
+
+   If marked as asynchroous then the OPAL stack itself will take care of the
+   timing and things like read/write to a disk file will work correctly.
+  */
+typedef enum OpalMediaTiming {
+  OpalMediaTimingNoChange,      /**< No change to the media data type. */
+  OpalMediaTimingSynchronous,   /**< Indicate the read/write function is going to handle
+                                     all real time aspects of the media flow. */
+  OpalMediaTimingAsynchronous   /**< Indicate the read/write function does not handle
+                                     the real time aspects of the media flow. */
+} OpalMediaTiming;
+
+
 /**General parameters for the OpalCmdSetGeneralParameters command.
    This is only passed to and returned from the OpalSendMessage() function.
 
    Example:
+      <code>
       OpalMessage   command;
       OpalMessage * response;
 
@@ -468,6 +530,7 @@ typedef enum OpalMediaDataType {
       command.m_param.m_general.m_stunServer = "stun.voxgratia.org";
       command.m_param.m_general.m_mediaMask = "RFC4175*";
       response = OpalSendMessage(hOPAL, &command);
+      </code>
   */
 typedef struct OpalParamGeneral {
   const char * m_audioRecordDevice;   /**< Audio recording device name */
@@ -527,7 +590,9 @@ typedef struct OpalParamGeneral {
                                            units (8000Hz). */
   OpalEchoCancelMode m_echoCancellation; /**< Accoustic Echo Cancellation control. 0=no change, 1=disabled,
                                               2=enabled. */
-  unsigned     m_audioBuffers;        /**< Set the number of hardware sound buffers to use. */
+  unsigned     m_audioBuffers;        /**< Set the number of hardware sound buffers to use.
+                                           Note the largest of m_audioBuffers and m_audioBufferTime/frametime
+                                           will be used. */
   OpalMediaDataFunction m_mediaReadData;   /**< Callback function for reading raw media data. See
                                                 OpalMediaDataFunction for more information. */
   OpalMediaDataFunction m_mediaWriteData;  /**< Callback function for writing raw media data. See
@@ -543,9 +608,23 @@ typedef struct OpalParamGeneral {
                                            list of entries of the form "codec:option=value". Codec is either
                                            a media type (e.g. "Audio" or "Video") or a specific media format,
                                            for example:
+                                             <code>
                                              "G.723.1:Tx Frames Per Packet=2\nH.263:Annex T=0\n"
                                              "Video:Max Rx Frame Width=176\nVideo:Max Rx Frame Height=144"
+                                             </code>
                                            */
+  unsigned     m_audioBufferTime;     /**< Set the hardware sound buffers to use in milliseconds.
+                                           Note the largest of m_audioBuffers and m_audioBufferTime/frametime
+                                           will be used. */
+  unsigned m_manualAlerting;          /**< Indicate that an "alerting" message is automatically (value=1)
+                                           or manually (value=2) sent to the remote on receipt of an
+                                           OpalIndIncomingCall message. If set to manual then it is up
+                                           to the application to send a OpalCmdAlerting message to
+                                           indicate to the remote system that we are "ringing".
+                                           If zero then no change is made. */
+  OpalMediaTiming m_mediaTiming;      /**< Indicate that the media read/write callback function
+                                           handles the real time aspects of the media flow.
+                                           0=no change, 1=synchronous, 2=asynchronous. */
 } OpalParamGeneral;
 
 
@@ -584,6 +663,7 @@ typedef struct OpalProductDescription {
    This is only passed to and returned from the OpalSendMessage() function.
 
    Example:
+      <code>
       OpalMessage   command;
       OpalMessage * response;
 
@@ -593,6 +673,7 @@ typedef struct OpalProductDescription {
       command.m_param.m_protocol.m_displayName = "Robert Jongbloed";
       command.m_param.m_protocol.m_interfaceAddresses = "*";
       response = OpalSendMessage(hOPAL, &command);
+      </code>
   */
 typedef struct OpalParamProtocol {
   const char * m_prefix;              /**< Protocol prefix for parameters, e.g. "h323" or "sip". If this is
@@ -612,13 +693,17 @@ typedef struct OpalParamProtocol {
 } OpalParamProtocol;
 
 
+/// Name of SIP event package for Message Waiting events.
 #define OPAL_MWI_EVENT_PACKAGE             "message-summary"
+
+/// Name of SIP even package fo rmonitoring call status
 #define OPAL_LINE_APPEARANCE_EVENT_PACKAGE "dialog;sla;ma"
 
 /**Registration parameters for the OpalCmdRegistration command.
    This is only passed to and returned from the OpalSendMessage() function.
 
    Example:
+      <code>
       OpalMessage   command;
       OpalMessage * response;
 
@@ -673,6 +758,7 @@ typedef struct OpalParamProtocol {
       command.m_param.m_registrationInfo.m_eventPackage = OPAL_LINE_APPEARANCE_EVENT_PACKAGE;
       command.m_param.m_registrationInfo.m_timeToLive = 0;
       response = OpalSendMessage(hOPAL, &command);
+      </code>
   */
 typedef struct OpalParamRegistration {
   const char * m_protocol;      /**< Protocol prefix for registration. Currently must be "h323" or
@@ -756,6 +842,7 @@ typedef struct OpalStatusRegistration {
    to the data for the call in progress.
 
    Example:
+      <code>
       OpalMessage   command;
       OpalMessage * response;
 
@@ -776,6 +863,7 @@ typedef struct OpalStatusRegistration {
       command.m_param.m_callSetUp.m_callToken = callToken;
       command.m_param.m_callSetUp.m_partyB = "sip:10.0.1.12";
       response = OpalSendMessage(hOPAL, &command);
+      </code>
   */
 typedef struct OpalParamSetUpCall {
   const char * m_partyA;      /**< A-Party for call.
@@ -914,7 +1002,7 @@ typedef struct OpalParamSetUserData {
 } OpalParamSetUserData;
 
 
-/**User input information for the OpalIndUserInput indication.
+/**User input information for the OpalIndUserInput/OpalCmdUserInput indication.
 
    This is may be returned from the OpalGetMessage() function or
    provided to the OpalSendMessage() function.
@@ -927,7 +1015,7 @@ typedef struct OpalStatusUserInput {
                                    Generally zero is passed which means the m_userInput is a
                                    single "string" input. If non-zero then m_userInput must
                                    be a single character. */
-} OpalStatusUserInput;
+} OpalStatusUserInput, OpalParamUserInput;
 
 
 /**Message Waiting information for the OpalIndMessageWaiting indication.
@@ -977,6 +1065,29 @@ typedef struct OpalStatusLineAppearance {
 } OpalStatusLineAppearance;
 
 
+/**Type of mixing for video when recording.
+   This is used by the OpalCmdStartRecording command in the OpalParamRecording structure.
+  */
+typedef enum OpalVideoRecordMixMode {
+  OpalSideBySideLetterbox, /**< Two images side by side with black bars top and bottom.
+                                It is expected that the input frames and output are all
+                                the same aspect ratio, e.g. 4:3. Works well if inputs
+                                are QCIF and output is CIF for example. */
+  OpalSideBySideScaled,    /**< Two images side by side, scaled to fit halves of output
+                                frame. It is expected that the output frame be double
+                                the width of the input data to maintain aspect ratio.
+                                e.g. for CIF inputs, output would be 704x288. */
+  OpalStackedPillarbox,    /**< Two images, one on top of the other with black bars down
+                                the sides. It is expected that the input frames and output
+                                are all the same aspect ratio, e.g. 4:3. Works well if
+                                inputs are QCIF and output is CIF for example. */
+  OpalStackedScaled,       /**< Two images, one on top of the other, scaled to fit halves
+                                of output frame. It is expected that the output frame be
+                                double the height of the input data to maintain aspect
+                                ratio. e.g. for CIF inputs, output would be 352x576. */
+} OpalVideoRecordMixMode;
+
+
 /**Call recording information for the OpalCmdStartRecording command.
   */
 typedef struct OpalParamRecording {
@@ -986,6 +1097,18 @@ typedef struct OpalParamRecording {
   unsigned     m_channels;   /**< Number of channels in WAV file, 1 for mono (default) or 2 for
                                   stereo where incoming & outgoing audio are in individual
                                   channels. */
+  const char * m_audioFormat; /**< Audio recording format. This is generally an OpalMediaFormat
+                                   name which will be used in the recording file. The exact
+                                   values possible is dependent on many factors including the
+                                   specific file type and what codecs are loaded as plug ins. */
+  const char * m_videoFormat; /**< Video recording format. This is generally an OpalMediaFormat
+                                   name which will be used in the recording file. The exact
+                                   values possible is dependent on many factors including the
+                                   specific file type and what codecs are loaded as plug ins. */
+  unsigned     m_videoWidth;  /**< Width of image for recording video. */
+  unsigned     m_videoHeight; /**< Height of image for recording video. */
+  unsigned     m_videoRate;   /**< Frame rate for recording video. */
+  OpalVideoRecordMixMode m_videoMixing; /**< How the two images are saved in video recording. */
 } OpalParamRecording;
 
 
@@ -1060,11 +1183,11 @@ struct OpalMessage {
     OpalParamGeneral         m_general;            ///< Used by OpalCmdSetGeneralParameters
     OpalParamProtocol        m_protocol;           ///< Used by OpalCmdSetProtocolParameters
     OpalParamRegistration    m_registrationInfo;   ///< Used by OpalCmdRegistration
-    OpalStatusRegistration   m_registrationStatus; ///< Used by OpalIndRegistrationStatus
+    OpalStatusRegistration   m_registrationStatus; ///< Used by OpalIndRegistration
     OpalParamSetUpCall       m_callSetUp;          ///< Used by OpalCmdSetUpCall/OpalIndProceeding/OpalIndAlerting/OpalIndEstablished
-    const char *             m_callToken;          ///< Used by OpalCmdAnswerCall/OpalCmdHoldcall/OpalCmdRetreiveCall/OpalCmdStopRecording
+    const char *             m_callToken;          ///< Used by OpalCmdAnswerCall/OpalCmdHoldcall/OpalCmdRetrieveCall/OpalCmdStopRecording/OpalCmdAlerting
     OpalStatusIncomingCall   m_incomingCall;       ///< Used by OpalIndIncomingCall
-    OpalStatusUserInput      m_userInput;          ///< Used by OpalIndUserInput
+    OpalStatusUserInput      m_userInput;          ///< Used by OpalIndUserInput/OpalCmdUserInput
     OpalStatusMessageWaiting m_messageWaiting;     ///< Used by OpalIndMessageWaiting
     OpalStatusLineAppearance m_lineAppearance;     ///< Used by OpalIndLineAppearance
     OpalStatusCallCleared    m_callCleared;        ///< Used by OpalIndCallCleared
@@ -1080,6 +1203,131 @@ struct OpalMessage {
 };
 #endif
 
+#if defined(__cplusplus) || defined(DOC_PLUS_PLUS)
+
+/// Wrapper around the OpalMessage structure
+class OpalMessagePtr
+{
+  public:
+    OpalMessagePtr(OpalMessageType type = OpalIndCommandError);
+    ~OpalMessagePtr();
+
+    OpalMessageType GetType() const;
+    void SetType(OpalMessageType type);
+
+    const char               * GetCallToken() const;          ///< Used by OpalCmdAnswerCall/OpalCmdHoldCall/OpalCmdRetrieveCall/OpalCmdStopRecording/OpalCmdAlerting
+    void                       SetCallToken(const char * token);
+
+    const char               * GetCommandError() const;       ///< Used by OpalIndCommandError
+
+    OpalParamGeneral         * GetGeneralParams() const;      ///< Used by OpalCmdSetGeneralParameters
+    OpalParamProtocol        * GetProtocolParams() const;     ///< Used by OpalCmdSetProtocolParameters
+    OpalParamRegistration    * GetRegistrationInfo() const;   ///< Used by OpalCmdRegistration
+    OpalStatusRegistration   * GetRegistrationStatus() const; ///< Used by OpalIndRegistration
+    OpalParamSetUpCall       * GetCallSetUp() const;          ///< Used by OpalCmdSetUpCall/OpalIndProceeding/OpalIndAlerting/OpalIndEstablished
+    OpalStatusIncomingCall   * GetIncomingCall() const;       ///< Used by OpalIndIncomingCall
+    OpalStatusUserInput      * GetUserInput() const;          ///< Used by OpalIndUserInput/OpalCmdUserInput
+    OpalStatusMessageWaiting * GetMessageWaiting() const;     ///< Used by OpalIndMessageWaiting
+    OpalStatusLineAppearance * GetLineAppearance() const;     ///< Used by OpalIndLineAppearance
+    OpalStatusCallCleared    * GetCallCleared() const;        ///< Used by OpalIndCallCleared
+    OpalParamCallCleared     * GetClearCall() const;          ///< Used by OpalCmdClearCall
+    OpalStatusMediaStream    * GetMediaStream() const;        ///< Used by OpalIndMediaStream/OpalCmdMediaStream
+    OpalParamSetUserData     * GetSetUserData() const;        ///< Used by OpalCmdSetUserData
+    OpalParamRecording       * GetRecording() const;          ///< Used by OpalCmdStartRecording
+
+  protected:
+    OpalMessage * m_message;
+
+  private:
+    OpalMessagePtr(const OpalMessagePtr &) { }
+    void operator=(const OpalMessagePtr &) { }
+
+  friend class OpalContext;
+};
+
+
+#ifdef GetMessage
+#undef GetMessage
+#endif
+#ifdef SendMessage
+#undef SendMessage
+#endif
+
+
+/** This class is a wrapper around the "C" API.
+
+    It may seem odd to have a C++ wrapper around a "C" API which is itself a
+    wrapper around a C++ API, but sometimes a C++ programmer may wish to
+    access the OPAL system via this simplified API instead of the quite
+    complex one in the base OPAL library.
+  */
+class OpalContext
+{
+  public:
+    /// Construct an unintialised OPAL context.
+    OpalContext();
+
+    /// Destroy the OPAL context, calls ShutDown().
+    virtual ~OpalContext();
+
+    /// Calls OpalIntialise() to initialise the OPAL context.
+    /// Returns version of API supported by library, zero if error.
+    unsigned Initialise(
+      const char * options,  ///< List of options to pass to OpalIntialise()
+      unsigned version = OPAL_C_API_VERSION ///< Version expected by application
+    );
+
+    /// Indicate if the OPAL context has been initialised.
+    bool IsInitialised() const { return m_handle != NULL; }
+
+    /// Calls OpalShutDown() to dispose of the OPAL context.
+    void ShutDown();
+
+    /// Calls OpalGetMessage() to get next message from the OPAL context.
+    bool GetMessage(
+      OpalMessagePtr & message,
+      unsigned timeout = 0
+    );
+
+    /// Calls OpalSendMessage() to send a message to the OPAL context.
+    bool SendMessage(
+      const OpalMessagePtr & message,  ///< Message to send to OPAL.
+      OpalMessagePtr & response        ///< Response from OPAL.
+    );
+
+
+    /// Execute OpalSendMessage() using OpalCmdSetUpCall
+    bool SetUpCall(
+      OpalMessagePtr & response,       ///< Response from OPAL context on initiating call.
+      const char * partyB,             ///< Destination address, see OpalCmdSetUpCall.
+      const char * partyA = NULL,      ///< Calling sub-system, see OpalCmdSetUpCall.
+      const char * alertingType = NULL ///< Alerting type code, see OpalCmdSetUpCall.
+    );
+
+    /// Answer a call using OpalCmdAnswerCall via OpalSendMessage()
+    bool AnswerCall(
+      const char * callToken           ///< Call token for call being answered.
+    );
+
+    /// Clear a call using OpalCmdClearCall via OpalSendMessage()
+    bool ClearCall(
+      const char * callToken,          ///< Call token for call being cleared.
+      OpalCallEndReason reason = OpalCallEndedByLocalUser  ///< Code for the call termination, see OpalCmdClearCall.
+    );
+
+    /// Send user input using OpalCmdUserInput via OpalSendMessage()
+    bool SendUserInput(
+      const char * callToken,     ///< Call token for the call, see OpalCmdUserInput.
+      const char * userInput,     ///< User input string, e.g. "#", see OpalCmdUserInput.
+      unsigned     duration = 0   ///< Duration in milliseconds for tone, see OpalCmdUserInput.
+    );
+
+  protected:
+    OpalHandle m_handle;
+};
+
+#endif
+
 #endif // OPAL_OPAL_H
 
 
