@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22096 $
+ * $Revision: 24547 $
  * $Author: rjongbloed $
- * $Date: 2009-02-20 21:25:23 -0600 (Fri, 20 Feb 2009) $
+ * $Date: 2010-06-30 01:36:31 -0500 (Wed, 30 Jun 2010) $
  */
 
 #ifndef OPAL_OPAL_CALL_H
@@ -39,6 +39,7 @@
 #include <opal/buildopts.h>
 
 #include <opal/connection.h>
+#include <opal/recording.h>
 #include <opal/guid.h>
 
 #include <ptlib/safecoll.h>
@@ -115,6 +116,10 @@ class OpalCall : public PSafeObject
       */
     OpalConnection::CallEndReason GetCallEndReason() const { return callEndReason; }
 
+    /**Get the reason for this connection shutting down as text.
+      */
+    PString GetCallEndReasonText() const { return OpalConnection::GetCallEndReasonText(callEndReason); }
+
     /**Set the call clearance reason.
        An application should have no cause to use this function. It is present
        for the H323EndPoint::ClearCall() function to set the clearance reason.
@@ -326,17 +331,40 @@ class OpalCall : public PSafeObject
     bool IsOnHold() const;
 
     /**Transfer connection.
-       If the newAddress indicates a protocol type and it is the same as
-       connection then this simply calls TransferConnection on the
-       connection variable.
+       There are several scenarios for this function.
 
-       If there is a protocol change, e.g. "pc" to "t38", then the connection
-       is completely closed, all links to the second conection in the call (if
-       any) severed and a new connection establshed and new streams started.
+       If 'connection' is not NULL and the protocol type for 'address' is
+       the same, or the protocol type is "*", or the address is a valid and
+       current call token, then this simply calls TransferConnection() on the
+       'connection' variable.
+       
+       e.g. connection="sip:fred@bloggs.com" and address="*:1234", then
+       connection->TransferConnection("sip:1234") is called, which sends a
+       REFER command to the remote. The connection and call are subsequently
+       released in this case.
+
+       Another example, if connection="pc:Speakers" and address="pc:Headset",
+       then the sound device in the 'connection' instance is changed and the
+       call and media is continued unchanged. The connection and call are
+       <b>not</b> released in this case.
+
+       If there is a protocol change, e.g. "pc:" to "t38:", then 'connection'
+       is completely released, all media streams to the second conection in
+       the call (if any) are severed and a new connection established and new
+       media streams started.
+
+       If 'connection' is NULL, it will choose the first connection in the call
+       of the same protocol type. For example, in the previous example above
+       where address="pc:Headset", and the call is from "pc:Speakers" to
+       "sip:fred@bloggs.com", the function would operate the same even if
+       'connection' is NULL.
+
+       If there are no connections of the same protocol type, then nothing is
+       done and false is returned.
       */
     bool Transfer(
-      OpalConnection & connection, ///< Connection to transfer
-      const PString & newAddress   ///< New address to transfer to
+      const PString & address,           ///< New address to transfer to
+      OpalConnection * connection = NULL ///< Connection to transfer
     );
   //@}
 
@@ -369,6 +397,9 @@ class OpalCall : public PSafeObject
       const OpalMediaType & mediaType,          ///<  Media type of channel to open
       unsigned sessionID = 0,                   ///<  Session to start streams on
       const OpalMediaFormat & preselectedFormat = OpalMediaFormat()  ///< Format for source stream to choose from
+#if OPAL_VIDEO
+      , OpalVideoFormat::ContentRole contentRole = OpalVideoFormat::eNoRole ///< Content role for video
+#endif
     );
 
     /**Select media format pair from the source/destination list.
@@ -376,6 +407,7 @@ class OpalCall : public PSafeObject
        Default behavour calls OpalTranscoder::SelectFormats().
       */
     virtual bool SelectMediaFormats(
+      const OpalMediaType & mediaType,        ///< Media type for selection.
       const OpalMediaFormatList & srcFormats, ///<  Names of possible source formats
       const OpalMediaFormatList & dstFormats, ///<  Names of possible destination formats
       const OpalMediaFormatList & allFormats, ///<  Master list of formats for merging options
@@ -395,10 +427,14 @@ class OpalCall : public PSafeObject
       const RTP_Session & session         ///<  Session with statistics
     );
 
+    /**Start the media streams on the connections.
+     */
+    virtual void StartMediaStreams();
+
     /**Close the media streams on the connections.
      */
     virtual void CloseMediaStreams();
-    
+
     /**See if the media can bypass the local host.
      */
     virtual PBoolean IsMediaBypassPossible(
@@ -477,7 +513,7 @@ class OpalCall : public PSafeObject
     const PTime & GetStartTime() const { return startTime; }
   //@}
 
-
+#if OPAL_HAS_MIXER
     /**Start recording a call.
        Current version saves to a WAV file. It may either mix the receive and
        transmit audio stream to a single mono file, or the streams are placed
@@ -485,7 +521,7 @@ class OpalCall : public PSafeObject
       */
     bool StartRecording(
       const PFilePath & filename, ///< File into which to record
-      bool mono = false           ///< Record as mono/stereo
+      const OpalRecordManager::Options & options = false ///< Record mixing options
     );
 
     /**Indicate if recording is currently active on call.
@@ -497,13 +533,37 @@ class OpalCall : public PSafeObject
       */
     void StopRecording();
 
-    /** Call back for having a frame of audio to record.
+    /** Call back on recording started.
       */
-    virtual void OnRecordAudio(const PString & streamId, const RTP_DataFrame & frame);
+    virtual bool OnStartRecording(
+      const PString & streamId,       ///< Unique ID for stream within call
+      const OpalMediaFormat & format  ///< Media format for stream
+    );
 
     /** Call back on recording stopped.
       */
-    virtual void OnStopRecordAudio(const PString & streamId);
+    virtual void OnStopRecording(
+      const PString & streamId       ///< Unique ID for stream within call
+    );
+
+    /** Call back for having a frame of audio to record.
+      */
+    virtual void OnRecordAudio(
+      const PString & streamId,       ///< Unique ID for stream within call
+      const RTP_DataFrame & frame     ///< Media data
+    );
+
+#if OPAL_VIDEO
+    /** Call back for having a frame of video to record.
+      */
+    virtual void OnRecordVideo(
+      const PString & streamId,       ///< Unique ID for stream within call
+      const RTP_DataFrame & frame     ///< Media data
+    );
+#endif
+#endif // OPAL_HAS_MIXER
+
+    void InternalOnClear();
 
   protected:
     void SetPartyNames();
@@ -525,12 +585,14 @@ class OpalCall : public PSafeObject
     bool    isClearing;
 
     OpalConnection::CallEndReason callEndReason;
+    PSyncPoint                  * endCallSyncPoint;
 
     PSafeList<OpalConnection> connectionsActive;
 
-    PSyncPoint * endCallSyncPoint;
+#if OPAL_HAS_MIXER
+    OpalRecordManager * m_recordManager;
+#endif
 
-    
   //use to add the connection to the call's connection list
   friend OpalConnection::OpalConnection(OpalCall &, OpalEndPoint &, const PString &, unsigned int, OpalConnection::StringOptions *);
   //use to remove the connection from the call's connection list
