@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
+ * $Revision: 24031 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Date: 2010-02-05 19:18:59 -0600 (Fri, 05 Feb 2010) $
  */
 
 #ifndef OPAL_OPAL_IVR_H
@@ -41,16 +41,52 @@
 #if OPAL_IVR
 
 #include <opal/opalvxml.h>
-#include <opal/endpoint.h>
+#include <opal/localep.h>
 
 class OpalIVRConnection;
 
 
 /**Interactive Voice Response endpoint.
+   This endpoint type will create an connection that executes a script,
+   interacting with the corresponding connection in the call, playing messages
+   and waiting for input. The other connection is typically a network such
+   as H.323 or SIP but due to the orthoganlity of OPAL could be any connection
+   type.
+
+   The script, set via the OpalIVREndPoint::SetDefaultVXML() function, or
+   presented after the "ivr:" in the URI string used to make the call, may be
+   any of the following:
+
+      VXML script, in which case it MUST start with "<?xml" or "<vxml".
+
+      A file path with the extension .vxml, the VSM script is read from that
+      file.
+
+      A file URI for a .vxml file as above.
+
+      A file URI for a .WAV file, the WAV file is played and the connection is
+      released.
+
+      Or a series of simple separated by ';'. The commands are:
+
+          repeat=n      Repeat next WAV file or Speak command  n times.
+          delay=n       Delay after repeats n milliseconds.
+          voice=name    Set Text To Speech voice to name
+          tone=t        Emit DTMF tone t
+          silence=n     Emit silence for n milliseconds
+          speak=text    Speak the text using the Text To Speech system.
+          speak=$var    Speak the internal variable using the Text To Speech system.
+          file:///x.wav File URI for WAV file to play.
+
+      Variables may be one of:
+          Time
+          Originator-Address
+          Remote-Address
+          Source-IP-Address
  */
-class OpalIVREndPoint : public OpalEndPoint
+class OpalIVREndPoint : public OpalLocalEndPoint
 {
-    PCLASSINFO(OpalIVREndPoint, OpalEndPoint);
+    PCLASSINFO(OpalIVREndPoint, OpalLocalEndPoint);
   public:
   /**@name Construction */
   //@{
@@ -97,7 +133,7 @@ class OpalIVREndPoint : public OpalEndPoint
 
        The default behaviour is pure.
      */
-    virtual PBoolean MakeConnection(
+    virtual PSafePtr<OpalConnection> MakeConnection(
       OpalCall & call,          ///<  Owner of connection
       const PString & party,    ///<  Remote party to call
       void * userData = NULL,   ///<  Arbitrary data to pass to connection
@@ -125,15 +161,11 @@ class OpalIVREndPoint : public OpalEndPoint
       */
     virtual OpalIVRConnection * CreateConnection(
       OpalCall & call,        ///<  Owner of connection
-      const PString & token,  ///<  Call token for new connection
       void * userData,        ///<  Arbitrary data to pass to connection
       const PString & vxml,   ///<  vxml to execute
+      unsigned int options,
       OpalConnection::StringOptions * stringOptions = NULL
     );
-
-    /**Create a unique token for a new conection.
-      */
-    virtual PString CreateConnectionToken();
   //@}
 
   /**@name Options and configuration */
@@ -179,18 +211,20 @@ class OpalIVREndPoint : public OpalEndPoint
   //@}
 
   protected:
-    unsigned            nextTokenNumber;
     PString             defaultVXML;
     OpalMediaFormatList defaultMediaFormats;
     PString             defaultTts;
+
+  private:
+    P_REMOVE_VIRTUAL(OpalIVRConnection *, CreateConnection(OpalCall &,const PString &,void *,const PString &,OpalConnection::StringOptions *),0);
 };
 
 
 /**Interactive Voice Response connection.
  */
-class OpalIVRConnection : public OpalConnection
+class OpalIVRConnection : public OpalLocalConnection
 {
-    PCLASSINFO(OpalIVRConnection, OpalConnection);
+    PCLASSINFO(OpalIVRConnection, OpalLocalConnection);
   public:
   /**@name Construction */
   //@{
@@ -199,9 +233,9 @@ class OpalIVRConnection : public OpalConnection
     OpalIVRConnection(
       OpalCall & call,            ///<  Owner calll for connection
       OpalIVREndPoint & endpoint, ///<  Owner endpoint for connection
-      const PString & token,      ///<  Token for connection
       void * userData,            ///<  Arbitrary data to pass to connection
       const PString & vxml,       ///<  vxml to execute
+      unsigned int options,
       OpalConnection::StringOptions * stringOptions = NULL
     );
 
@@ -226,35 +260,6 @@ class OpalIVRConnection : public OpalConnection
       */
     virtual PString GetLocalPartyURL() const;
 
-    /**Start an outgoing connection.
-       This function will initiate the connection to the remote entity, for
-       example in H.323 it sends a SETUP, in SIP it sends an INVITE etc.
-
-       The default behaviour does.
-      */
-    virtual PBoolean SetUpConnection();
-
-    /**Indicate to remote endpoint an alert is in progress.
-       If this is an incoming connection and the AnswerCallResponse is in a
-       AnswerCallDeferred or AnswerCallPending state, then this function is
-       used to indicate to that endpoint that an alert is in progress. This is
-       usually due to another connection which is in the call (the B party)
-       has received an OnAlerting() indicating that its remote endpoint is
-       "ringing".
-
-       The default behaviour does nothing.
-      */
-    virtual PBoolean SetAlerting(
-      const PString & calleeName,   ///<  Name of endpoint being alerted.
-      PBoolean withMedia                ///<  Open media with alerting
-    );
-
-    /**Indicate to remote endpoint we are connected.
-
-       The default behaviour does nothing.
-      */
-    virtual PBoolean SetConnected();
-
     /**A call back function whenever a connection is "established".
        This indicates that a connection to an endpoint was established. This
        usually occurs after OnConnected() and indicates that the connection
@@ -308,7 +313,7 @@ class OpalIVRConnection : public OpalConnection
       */
     virtual PBoolean StartVXML();
 
-    void OnMediaPatchStop(unsigned, bool);
+    virtual void OnStopMediaPatch(OpalMediaPatch & patch);
 
     PTextToSpeech * SetTextToSpeech(PTextToSpeech * _tts, PBoolean autoDelete = PFalse)
     { return vxmlSession.SetTextToSpeech(_tts, autoDelete); }
@@ -361,8 +366,6 @@ class OpalIVRMediaStream : public OpalRawMediaStream
        Returns PFalse for IVR streams.
       */
     virtual PBoolean IsSynchronous() const;
-
-    PBoolean ReadPacket(RTP_DataFrame & packet);
   //@}
 
   protected:
