@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23772 $
+ * $Revision: 24362 $
  * $Author: rjongbloed $
- * $Date: 2009-11-16 20:39:18 -0600 (Mon, 16 Nov 2009) $
+ * $Date: 2010-05-20 03:24:51 -0500 (Thu, 20 May 2010) $
  */
 
 #ifndef OPAL_OPAL_MEDIASTRM_H
@@ -358,6 +358,8 @@ class OpalMediaStream : public PSafeObject
   //@}
 
   protected:
+    void IncrementTimestamp(PINDEX size);
+
     OpalConnection & connection;
     unsigned         sessionID;
     PString          identifier;
@@ -373,6 +375,10 @@ class OpalMediaStream : public PSafeObject
     OpalMediaPatch * mediaPatch;
     PNotifier        commandNotifier;
 
+    RTP_DataFrame::PayloadTypes m_payloadType;
+    unsigned                    m_frameTime;
+    PINDEX                      m_frameSize;
+
   private:
     P_REMOVE_VIRTUAL_VOID(OnPatchStart());
     P_REMOVE_VIRTUAL_VOID(OnPatchStop());
