@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 23973 $
+ * $Author: rjongbloed $
+ * $Date: 2010-01-24 18:33:51 -0600 (Sun, 24 Jan 2010) $
  */
 
 #ifndef OPAL_OPAL_PATCH_H
@@ -40,13 +40,12 @@
 
 #include <opal/mediastrm.h>
 #include <opal/mediacmd.h>
+#include <codec/ratectl.h>
 
 #include <list>
 
-
 class OpalTranscoder;
 
-
 /**Media stream "patch cord".
    This class is the thread of control that transfers data from one
    "source" OpalMediStream to one or more other "sink" OpalMediStream
@@ -94,6 +93,16 @@ class OpalMediaPatch : public PObject
       */
     virtual void Start();
 
+    /**Indicate the patch has started. Typically called from the beginning
+       of the patch thread.
+
+       Default behaviour make sure jitter buffer is created.
+
+       Returns bool indicating all media streams are asynchronous so the patch
+               should assure no thread startvation
+      */
+    virtual bool OnStartMediaPatch();
+
     /**Close the patch.
       This is an internal function that closes all of the sink streams and
        waits for the the thread to terminate. It is called when the source
@@ -189,7 +198,34 @@ class OpalMediaPatch : public PObject
       PBoolean fromSink                 ///<  Flag for source or sink
     );
 
-    virtual PBoolean PushFrame(RTP_DataFrame & /*frame*/) { return PFalse; };
+    /**Push a frame out to all the sink streams, transcoding as necessary.
+      */
+    virtual PBoolean PushFrame(
+      RTP_DataFrame & frame   ///< Frame to push
+    );
+
+    /**Set bypass patch instance.
+
+       This can be useful for back to back calls that happen to be the same
+       media format and you wish to avoid double decoding and encoding of
+       media. Note this scenario is not the same as two OpalConnections within
+       the same OpalCall, but two completely independent OpalCall where one
+       connection is to be bypassed. For example, two OpalCall instances might
+       have two SIPConnection instances and two OpalMixerConnection instances
+       connected via a single OpalMixerNode. Now while there are ONLY two
+       calls in the node, it is a waste to decode the audio, add to mixer and
+       re-encode it again. In practice this is identical to just bypassing the
+       mixer node completely, until a third party is added, then we need to
+       switch back to normal (non-bypassed) operation.
+
+       Note it is up to the caller to assure the patch instance lifetimes are
+       correct for the bypass.
+
+       @return true if bypass was set, false if conflict with another bypass.
+      */
+    bool SetBypassPatch(
+      OpalMediaPatch * patch
+    );
 
     /**Get the transcoder used within a sink stream
       */
@@ -197,7 +233,7 @@ class OpalMediaPatch : public PObject
     virtual void UnLockSinkTranscoder() const;
 
 #if OPAL_STATISTICS
-    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
+    virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromSink) const;
 #endif
   //@}
 
@@ -219,7 +255,7 @@ class OpalMediaPatch : public PObject
         void SetCommandNotifier(const PNotifier & notifier);
         bool WriteFrame(RTP_DataFrame & sourceFrame);
 #if OPAL_STATISTICS
-        void GetStatistics(OpalMediaStatistics & statistics) const;
+        void GetStatistics(OpalMediaStatistics & statistics, bool fromSource) const;
 #endif
 
         OpalMediaPatch  &  patch;
@@ -232,21 +268,8 @@ class OpalMediaPatch : public PObject
 
 #if OPAL_VIDEO
         void SetRateControlParameters(const OpalMediaFormat & mediaFormat);
-        bool RateControlExceeded(const PTimeInterval & currentTime);
-
-        bool          rcEnabled;
-        unsigned      rcByteRate;
-        unsigned      rcWindowSize;
-        unsigned      rcMaxConsecutiveFramesSkip;
-        unsigned      rcConsecutiveFramesSkipped;
-        PTimeInterval rcLastTime;
-        PINDEX        rcTotalSize;
-        
-        struct FrameInfo {
-          PTimeInterval time;
-          PINDEX        size;
-        };
-        std::list<FrameInfo> frameInfoList;
+        bool RateControlExceeded(bool & forceIFrame);
+        OpalVideoRateController * rateController;
 #endif
     };
     PList<Sink> sinks;
@@ -259,7 +282,15 @@ class OpalMediaPatch : public PObject
         OpalMediaFormat stage;
     };
     PList<Filter> filters;
-        
+
+#if OPAL_VIDEO
+    bool             m_videoDecoder;
+#endif
+    bool             m_bypassActive;
+    OpalMediaPatch * m_bypassToPatch;
+    OpalMediaPatch * m_bypassFromPatch;
+    PSyncPoint       m_bypassEnded;
+
     class Thread : public PThread {
         PCLASSINFO(Thread, PThread);
       public:
@@ -271,6 +302,9 @@ class OpalMediaPatch : public PObject
     Thread * patchThread;
     PMutex patchThreadMutex;
     mutable PReadWriteMutex inUse;
+
+  private:
+    P_REMOVE_VIRTUAL(bool, OnPatchStart(), false);
 };
 
 /**Passive Media Patch
@@ -290,7 +324,6 @@ class OpalPassiveMediaPatch : public OpalMediaPatch
     );
 
     virtual void Start();
-    virtual PBoolean PushFrame(RTP_DataFrame & frame);
 };
 
 
