@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
+ * $Revision: 24150 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Date: 2010-03-23 16:17:19 -0500 (Tue, 23 Mar 2010) $
  */
 
 #ifndef OPAL_OPAL_TRANSCODERS_H
@@ -125,7 +125,7 @@ __inline OpalTranscoderKey MakeOpalTranscoderKey(const char * from, const char *
 }
 
 #define OPAL_REGISTER_TRANSCODER(cls, input, output) \
-  OpalTranscoderFactory::Worker<cls> OpalTranscoder_##cls(MakeOpalTranscoderKey(input, output))
+  PFACTORY_CREATE(OpalTranscoderFactory, cls, MakeOpalTranscoderKey(input, output), false)
 
 
 /**This class embodies the implementation of a specific transcoder instance
@@ -233,10 +233,14 @@ class OpalTranscoder : public OpalMediaFormatPair
        There could be many possible matches between the two lists, so
        preference is given to the order of the destination formats.
 
-       Returns PFalse if there is no registered media transcoder that can be used
-       between the two named formats.
+       At least one of the two selected medai formats (source or destination)
+       must also be of the specified media type ("audio", "video" etc).
+
+       Returns false if there is no registered media transcoder that can be
+               used to get from src to dst for the media type.
       */
     static bool SelectFormats(
+      const OpalMediaType & mediaType,        ///< Media type for selection.
       const OpalMediaFormatList & srcFormats, ///<  Names of possible source formats
       const OpalMediaFormatList & dstFormats, ///<  Names of possible destination formats
       const OpalMediaFormatList & allFormats, ///<  Master list of formats for merging options
@@ -355,9 +359,7 @@ class OpalFramedTranscoder : public OpalTranscoder
       */
     OpalFramedTranscoder(
       const OpalMediaFormat & inputMediaFormat,  ///<  Input media format
-      const OpalMediaFormat & outputMediaFormat, ///<  Output media format
-      PINDEX inputBytesPerFrame,  ///<  Number of bytes in an input frame
-      PINDEX outputBytesPerFrame  ///<  Number of bytes in an output frame
+      const OpalMediaFormat & outputMediaFormat  ///<  Output media format
     );
   //@}
 
@@ -421,9 +423,11 @@ class OpalFramedTranscoder : public OpalTranscoder
   //@}
 
   protected:
-    PINDEX     inputBytesPerFrame;
-    PINDEX     outputBytesPerFrame;
-    PINDEX     maxOutputDataSize;
+    void CalculateSizes();
+
+    PINDEX inputBytesPerFrame;
+    PINDEX outputBytesPerFrame;
+    PINDEX maxOutputDataSize;
 };
 
 
@@ -518,7 +522,7 @@ class OpalEmptyFramedAudioTranscoder : public OpalFramedTranscoder
   PCLASSINFO(OpalEmptyFramedAudioTranscoder, OpalFramedTranscoder);
   public:
     OpalEmptyFramedAudioTranscoder(const char * inFormat, const char * outFormat)
-      : OpalFramedTranscoder(inFormat, outFormat, 100, 100)
+      : OpalFramedTranscoder(inFormat, outFormat)
     {  }
 
     PBoolean ConvertFrame(const BYTE *, PINDEX &, BYTE *, PINDEX &)
