@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21986 $
+ * $Revision: 23901 $
  * $Author: rjongbloed $
- * $Date: 2009-02-04 04:31:07 +0000 (Wed, 04 Feb 2009) $
+ * $Date: 2010-01-05 18:23:31 -0600 (Tue, 05 Jan 2010) $
  */
 
 #ifndef OPAL_RTP_JITTER_H
@@ -52,53 +52,63 @@ class RTP_JitterBufferAnalyser;
    application. The user is required to use a descendant of this class, and
    provide a "OnReadPacket" method, so that network packets can be placed in
    this class instance */
-class OpalJitterBuffer : public PObject
+class OpalJitterBuffer : public PSafeObject
 {
-  PCLASSINFO(OpalJitterBuffer, PObject);
+  PCLASSINFO(OpalJitterBuffer, PSafeObject);
 
   public:
+  /**@name Construction */
+  //@{
     /**Constructor for this jitter buffer. The size of this buffer can be
-       altered later with the SetDelay method */
+       altered later with the SetDelay method
+      */
     OpalJitterBuffer(
       unsigned minJitterDelay, ///<  Minimum delay in RTP timestamp units
       unsigned maxJitterDelay, ///<  Maximum delay in RTP timestamp units
       unsigned timeUnits = 8,  ///<  Time units, usually 8 or 16
-      PINDEX stackSize = 30000 ///<  Stack size for jitter thread
+      PINDEX packetSize = 2048 ///<  Max RTP packet size
     );
     
-    /**Destructor, which closes this down and deletes the internal list of frames */
+    /** Destructor, which closes this down and deletes the internal list of frames
+      */
     virtual ~OpalJitterBuffer();
+  //@}
 
+  /**@name Overrides from PObject */
+  //@{
     /**Report the statistics for this jitter instance */
-    void PrintOn(ostream & strm  ) const;
+    void PrintOn(
+      ostream & strm
+    ) const;
+  //@}
 
-    /**This method is where this OpalJitterBuffer collects data from the
-       outside world.  A descendant class of OpalJitterBuffer will override
-       this method
-
-    @return PTrue on successful read, PFalse on faulty read. */
-    virtual PBoolean OnReadPacket    (
-      RTP_DataFrame & frame,  ///<  Frame read from the RTP session
-      PBoolean loop               ///<  If PTrue, loop as long as data is available, if PFalse, only process once
-    ) = 0;
-
-//    PINDEX GetSize() const { return bufferSize; }
+  /**@name Operations */
+  //@{
     /**Set the maximum delay the jitter buffer will operate to.
       */
     void SetDelay(
       unsigned minJitterDelay, ///<  Minimum delay in RTP timestamp units
-      unsigned maxJitterDelay  ///<  Maximum delay in RTP timestamp units
+      unsigned maxJitterDelay, ///<  Maximum delay in RTP timestamp units
+      PINDEX packetSize = 2048 ///<  Max RTP packet size
     );
 
-    void UseImmediateReduction(PBoolean state) { doJitterReductionImmediately = state; }
+    /** Reset jitter buffer.
+        Jitter buffer is cleared and "restocked" from input data.
+      */
+    void Reset() { m_resetJitterBufferNow = true; }
 
-    /**Read a data frame from the RTP channel.
-       Any control frames received are dispatched to callbacks and are not
-       returned by this function. It will block until a data frame is
-       available or an error occurs.
+    /**Write data frame from the RTP channel.
+      */
+    virtual PBoolean WriteData(
+      const RTP_DataFrame & frame   ///< Frame to feed into jitter buffer
+    );
+
+    /**Read a data frame from the jitter buffer.
+       This function never blocks. If no data is available, an RTP packet
+       with zero payload size is returned.
       */
     virtual PBoolean ReadData(
-      RTP_DataFrame & frame   ///<  Frame read from the RTP session
+      RTP_DataFrame & frame   ///<  Frame to extract from jitter buffer
     );
 
     /**Get current delay for jitter buffer.
@@ -124,115 +134,130 @@ class OpalJitterBuffer : public PObject
     /**Set maximum consecutive marker bits before buffer starts to ignore them.
       */
     void SetMaxConsecutiveMarkerBits(DWORD max) { maxConsecutiveMarkerBits = max; }
-
-    /**Start jitter thread
-      */
-    virtual void Resume();
-
-    PDECLARE_NOTIFIER(PThread, OpalJitterBuffer, JitterThreadMain);
-
-    PBoolean WaitForTermination(const PTimeInterval & t)
-    { 
-      if (jitterThread == NULL) 
-        return PTrue;
-      shuttingDown = true;
-      return jitterThread->WaitForTermination(t); 
-    }
-
-    bool IsEmpty() { return jitterBuffer.size() == 0; }
+  //@}
 
   protected:
-    void Start(unsigned _minJitterTime, unsigned _maxJitterTime);
+    class Entry : public RTP_DataFrame
+    {
+      public:
+        Entry(PINDEX sz) : RTP_DataFrame(0, sz) { }
+        PTimeInterval tick;
+    };
+    OpalJitterBuffer::Entry * GetAvailableEntry();
+    void InternalWriteData(OpalJitterBuffer::Entry * availableEntry);
 
-    PINDEX        bufferSize;
     DWORD         minJitterTime;
     DWORD         maxJitterTime;
+    unsigned      timeUnits;
+    PINDEX        bufferSize;
     DWORD         maxConsecutiveMarkerBits;
 
-    unsigned      timeUnits;
     DWORD         currentJitterTime;
     DWORD         packetsTooLate;
     unsigned      bufferOverruns;
     unsigned      consecutiveBufferOverruns;
     DWORD         consecutiveMarkerBits;
+    bool          markerWarning;
     PTimeInterval consecutiveEarlyPacketStartTime;
     DWORD         lastWriteTimestamp;
     PTimeInterval lastWriteTick;
     DWORD         jitterCalc;
     DWORD         targetJitterTime;
     unsigned      jitterCalcPacketCount;
-    bool          doJitterReductionImmediately;
+    bool          m_resetJitterBufferNow;
 
-    class Entry : public RTP_DataFrame
+    struct FrameQueue : public PList<Entry>
     {
-      public:
-        Entry() : RTP_DataFrame(0, 512) { } // Allocate enough for 250ms of L16 audio, zero payload size
-        PTimeInterval tick;
+      FrameQueue() { DisallowDeleteObjects(); }
+      ~FrameQueue() { AllowDeleteObjects(); }
     };
-
-    class FrameQueue : public std::deque<Entry *>
-    {
-      public:
-        void resize(size_type _Newsize)
-        { 
-          while (size() < (size_t)_Newsize)
-            push_back(new Entry);
-          while (size() > (size_t)_Newsize) {
-            delete front();
-            pop_front();
-          }
-        }
-
-        ~FrameQueue()
-        { resize(0); }
-    };
-
     FrameQueue freeFrames;
     FrameQueue jitterBuffer;
-    inline Entry * GetNewest(bool pop) { Entry * e = jitterBuffer.back(); if (pop) jitterBuffer.pop_back(); return e; }
-    inline Entry * GetOldest(bool pop) { Entry * e = jitterBuffer.front(); if (pop) jitterBuffer.pop_front(); return e; }
+    Entry * GetNewest(bool pop);
+    Entry * GetOldest(bool pop);
 
     Entry * currentFrame;    // storage of current frame
 
     PMutex bufferMutex;
-    bool   shuttingDown;
     bool   preBuffering;
     bool   firstReadData;
 
     RTP_JitterBufferAnalyser * analyser;
-
-    PThread * jitterThread;
-    PINDEX    jitterStackSize;
-
-    PBoolean Init(Entry * & currentReadFrame, PBoolean & markerWarning);
-    PBoolean PreRead(Entry * & currentReadFrame, PBoolean & markerWarning);
-    PBoolean OnRead(Entry * & currentReadFrame, PBoolean & markerWarning, PBoolean loop);
-    void DeInit(Entry * & currentReadFrame, PBoolean & markerWarning);
 };
 
+
+/**A descendant of the OpalJitterBuffer that starts a thread to read
+   from something continuously and feed it into the jitter buffer.
+  */
+class OpalJitterBufferThread : public OpalJitterBuffer
+{
+    PCLASSINFO(OpalJitterBufferThread, OpalJitterBuffer);
+ public:
+    OpalJitterBufferThread(
+      unsigned minJitterDelay, ///<  Minimum delay in RTP timestamp units
+      unsigned maxJitterDelay, ///<  Maximum delay in RTP timestamp units
+      unsigned timeUnits = 8,  ///<  Time units, usually 8 or 16
+      PINDEX packetSize = 2048 ///<  Max RTP packet size
+    );
+    ~OpalJitterBufferThread();
+
+    /**Read a data frame from the jitter buffer.
+       This function never blocks. If no data is available, an RTP packet
+       with zero payload size is returned.
+
+       Override of base class so can terminate caller when shutting down.
+      */
+    virtual PBoolean ReadData(
+      RTP_DataFrame & frame   ///<  Frame to extract from jitter buffer
+    );
+
+    /**This class instance collects data from the outside world in this
+       method.
+
+       @return true on successful read, false on faulty read. */
+    virtual PBoolean OnReadPacket(
+      RTP_DataFrame & frame,  ///<  Frame read from the RTP session
+      PBoolean loop           ///<  If true, loop as long as data is available, if false, only process once
+    ) = 0;
+
+  protected:
+    PDECLARE_NOTIFIER(PThread, OpalJitterBufferThread, JitterThreadMain);
+
+    /// Internal function to be called from derived class constructor
+    void StartThread();
+
+    /// Internal function to be called from derived class destructor
+    void WaitForThreadTermination();
+
+    PThread * m_jitterThread;
+    bool      m_running;
+};
+
+
 /////////////////////////////////////////////////////////////////////////////
 /**A descendant of the OpalJitterBuffer that reads RTP_DataFrame instances
-   from the RTP_Sessions */
-class RTP_JitterBuffer : public OpalJitterBuffer
+   from the RTP_Sessions
+  */
+class RTP_JitterBuffer : public OpalJitterBufferThread
 {
-    PCLASSINFO(RTP_JitterBuffer, OpalJitterBuffer);
-
+    PCLASSINFO(RTP_JitterBuffer, OpalJitterBufferThread);
  public:
     RTP_JitterBuffer(
       RTP_Session & session,   ///<  Associated RTP session tor ead data from
       unsigned minJitterDelay, ///<  Minimum delay in RTP timestamp units
       unsigned maxJitterDelay, ///<  Maximum delay in RTP timestamp units
       unsigned timeUnits = 8,  ///<  Time units, usually 8 or 16
-      PINDEX stackSize = 30000 ///<  Stack size for jitter thread
+      PINDEX packetSize = 2048 ///<  Max RTP packet size
     );
+    ~RTP_JitterBuffer();
 
     /**This class instance collects data from the outside world in this
        method.
 
-    @return PTrue on successful read, PFalse on faulty read. */
+       @return true on successful read, false on faulty read. */
     virtual PBoolean OnReadPacket(
       RTP_DataFrame & frame,  ///<  Frame read from the RTP session
-      PBoolean loop           ///<  If PTrue, loop as long as data is available, if PFalse, only process once
+      PBoolean loop           ///<  If true, loop as long as data is available, if false, only process once
     );
 
  protected:
