@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22893 $
+ * $Revision: 24239 $
  * $Author: rjongbloed $
- * $Date: 2009-06-15 08:37:26 +0000 (Mon, 15 Jun 2009) $
+ * $Date: 2010-04-13 00:04:36 -0500 (Tue, 13 Apr 2010) $
  */
 
 #ifndef OPAL_SIP_HANDLERS_H
@@ -43,8 +43,7 @@
 
 #if OPAL_SIP
 
-#include <ptlib/safecoll.h>
-
+#include <opal/pres_ent.h>
 #include <opal/connection.h>
 #include <sip/sippdu.h>
 
@@ -58,18 +57,15 @@ class SIPHandler : public PSafeObject
 
 protected:
   SIPHandler(
-    SIPEndPoint & ep, 
-    const PString & target,
-    const PString & remote,
-    int expireTime = 0,
-    int offlineExpire = 30,
-    const PTimeInterval & retryMin = PMaxTimeInterval,
-    const PTimeInterval & retryMax = PMaxTimeInterval
+    SIPEndPoint & ep,
+    const SIPParameters & params
   );
 
 public:
   ~SIPHandler();
 
+  virtual Comparison Compare(const PObject & other) const;
+
   virtual bool ShutDown();
 
   enum State {
@@ -79,13 +75,14 @@ public:
     Refreshing,       // The registration is being refreshed
     Restoring,        // The registration is trying to be restored after being offline
     Unsubscribing,    // The unregistration is in process
-    Unsubscribed      // The registrating is inactive
+    Unsubscribed,     // The registrating is inactive
+    NumStates
   };
 
   void SetState (SIPHandler::State s);
 
   inline SIPHandler::State GetState () 
-  { return state; }
+  { return m_state; }
 
   virtual OpalTransport * GetTransport();
 
@@ -114,14 +111,19 @@ public:
 
   virtual SIP_PDU::Methods GetMethod() = 0;
   virtual SIPSubscribe::EventPackage GetEventPackage() const
-  { return PString::Empty(); }
+  { return SIPEventPackage(); }
 
+  virtual void OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response);
+  virtual void OnReceivedIntervalTooBrief(SIPTransaction & transaction, SIP_PDU & response);
+  virtual void OnReceivedTemporarilyUnavailable(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnTransactionFailed(SIPTransaction & transaction);
+
+  virtual void OnFailed(const SIP_PDU & response);
   virtual void OnFailed(SIP_PDU::StatusCodes);
 
-  bool ActivateState(SIPHandler::State state, unsigned msecs = 1000);
+  bool ActivateState(SIPHandler::State state);
   virtual bool SendNotify(const PObject * /*body*/) { return false; }
 
   SIPEndPoint & GetEndPoint() const { return endpoint; }
@@ -132,10 +134,13 @@ public:
   const PString & GetPassword() const     { return m_password; }
   const PString & GetRealm() const        { return m_realm; }
   const SIPURL & GetRemoteAddress() const { return m_remoteAddress; }
+  const SIPURL & GetProxy() const         { return m_proxy; }
+
+  SIPMIMEInfo m_mime;
 
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
-  void CollapseFork(SIPTransaction & transaction);
+  void RetryLater(unsigned after);
   PDECLARE_NOTIFIER(PTimer, SIPHandler, OnExpireTimeout);
   static PBoolean WriteSIPHandler(OpalTransport & transport, void * info);
   bool WriteSIPHandler(OpalTransport & transport);
@@ -157,12 +162,23 @@ protected:
   int                         offlineExpire;
   PString                     body;
   unsigned                    authenticationAttempts;
-  State                       state;
+  State                       m_state;
+  queue<State>                m_stateQueue;
+  bool                        m_receivedResponse;
   PTimer                      expireTimer; 
   PTimeInterval               retryTimeoutMin; 
   PTimeInterval               retryTimeoutMax; 
   SIPURL                      m_proxy;
   OpalProductInfo             m_productInfo;
+
+  // Keep a copy of the keys used for easy removal on destruction
+  typedef std::map<PString, PSafePtr<SIPHandler> > IndexMap;
+  std::pair<IndexMap::iterator, bool> m_byCallID;
+  std::pair<IndexMap::iterator, bool> m_byAorAndPackage;
+  std::pair<IndexMap::iterator, bool> m_byAuthIdAndRealm;
+  std::pair<IndexMap::iterator, bool> m_byAorUserAndRealm;
+
+  friend class SIPHandlersList;
 };
 
 #if PTRACING
@@ -191,6 +207,8 @@ public:
 
   void UpdateParameters(const SIPRegister::Params & params);
 
+  const SIPRegister::Params & GetParams() const { return m_parameters; }
+
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
   void SendStatus(SIP_PDU::StatusCodes code, State state);
@@ -210,7 +228,7 @@ public:
   virtual SIPTransaction * CreateTransaction (OpalTransport &);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
   virtual PBoolean OnReceivedNOTIFY(SIP_PDU & response);
-  virtual void OnFailed(SIP_PDU::StatusCodes r);
+  virtual void OnFailed(const SIP_PDU & response);
   virtual SIP_PDU::Methods GetMethod ()
     { return SIP_PDU::Method_SUBSCRIBE; }
   virtual SIPEventPackage GetEventPackage() const
@@ -220,9 +238,12 @@ public:
 
   virtual bool IsDuplicateCSeq(unsigned sequenceNumber) { return m_dialog.IsDuplicateCSeq(sequenceNumber); }
 
+  const SIPSubscribe::Params & GetParams() const { return m_parameters; }
+
 protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
   void SendStatus(SIP_PDU::StatusCodes code, State state);
+  bool DispatchNOTIFY(SIP_PDU & request, SIP_PDU & response);
 
   SIPSubscribe::Params     m_parameters;
   SIPDialogContext         m_dialog;
@@ -287,12 +308,10 @@ public:
     { return SIP_PDU::Method_PUBLISH; }
   virtual SIPEventPackage GetEventPackage() const
     { return m_parameters.m_eventPackage; }
-  virtual void SetBody(const PString & body);
 
 private:
   SIPSubscribe::Params m_parameters;
   PString              m_sipETag;
-  bool                 m_stateChanged;
 };
 
 
@@ -301,10 +320,8 @@ class SIPMessageHandler : public SIPHandler
   PCLASSINFO(SIPMessageHandler, SIPHandler);
 public:
   SIPMessageHandler(SIPEndPoint & ep, 
-                    const PString & to,
-                    const PString & body,
-                    const PString & remoteContact,
-                    const PString & callId);
+                    const SIPMessage::Params & params,
+                    const PString & body);
   ~SIPMessageHandler();
 
   virtual SIPTransaction * CreateTransaction (OpalTransport &);
@@ -313,8 +330,13 @@ public:
   virtual void OnFailed (SIP_PDU::StatusCodes);
   virtual void SetBody(const PString & b);
 
+  PURL m_localAddress;
+  PString m_id;
+
 private:
+  SIPMessage::Params m_parameters;
   virtual void OnExpireTimeout(PTimer &, INT);
+  PString m_body;
 };
 
 
@@ -322,8 +344,7 @@ class SIPPingHandler : public SIPHandler
 {
   PCLASSINFO(SIPPingHandler, SIPHandler);
 public:
-  SIPPingHandler(SIPEndPoint & ep, 
-                 const PString & to);
+  SIPPingHandler(SIPEndPoint & ep, const PURL & to);
   virtual SIPTransaction * CreateTransaction (OpalTransport &);
   virtual SIP_PDU::Methods GetMethod ()
     { return SIP_PDU::Method_MESSAGE; }
@@ -338,15 +359,17 @@ class SIPHandlersList
   public:
     /** Append a new handler to the list
       */
-    void Append(SIPHandler * handler)
-      { m_handlersList.Append(handler); }
+    void Append(SIPHandler * handler);
 
     /** Remove a handler from the list.
         Handler is not immediately deleted but marked for deletion later by
         DeleteObjectsToBeRemoved() when all references are done with the handler.
       */
-    void Remove(SIPHandler * handler)
-      { m_handlersList.Remove(handler); }
+    void Remove(SIPHandler * handler);
+
+    /** Update indexes for handler in the list
+      */
+    void Update(SIPHandler * handler);
 
     /** Clean up lists of handler.
       */
@@ -386,8 +409,8 @@ class SIPHandlersList
      * or 6001@seconix.com when registering 6001@seconix.com to
      * sip.seconix.com
      */
-    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PString & url, SIP_PDU::Methods meth, PSafetyMode m);
-    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PString & url, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode m);
+    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PURL & url, SIP_PDU::Methods meth, PSafetyMode m);
+    PSafePtr<SIPHandler> FindSIPHandlerByUrl(const PURL & url, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode m);
 
     /**
      * Find the SIPHandler object with the specified registration host.
@@ -397,30 +420,52 @@ class SIPHandlersList
     PSafePtr <SIPHandler> FindSIPHandlerByDomain(const PString & name, SIP_PDU::Methods meth, PSafetyMode m);
 
   protected:
+    void RemoveIndexes(SIPHandler * handler);
+
+    PMutex m_extraMutex;
     PSafeList<SIPHandler> m_handlersList;
+
+    typedef SIPHandler::IndexMap IndexMap;
+    PSafePtr<SIPHandler> FindBy(IndexMap & by, const PString & key, PSafetyMode m);
+
+    IndexMap m_byCallID;
+    IndexMap m_byAorAndPackage;
+    IndexMap m_byAuthIdAndRealm;
+    IndexMap m_byAorUserAndRealm;
 };
 
 
 /** Information for SIP "presence" event package notification messages.
   */
-struct SIPPresenceInfo
+class SIPPresenceInfo : public OpalPresenceInfo
 {
-  enum BasicStates {
-    Unknown,
-    Open,
-    Closed
-  };
+public:
+  SIPPresenceInfo(
+    const PString & personId = PString::Empty(),
+    State state = Unchanged
+  );
 
-  SIPPresenceInfo() : m_basic(Unknown) { }
+  // basic presence defined by RFC 3863
+  PString m_tupleId;
+  PString m_contact;
 
-  PString     m_address;
-  PString     m_entity;
-  BasicStates m_basic;
-  PString     m_note;
-  PString     m_contact;
-  PString     m_to;
+  // presence extensions defined by RFC 4480
+  PStringArray m_activities;  // list of activities, seperated by newline
 
-  PString AsString() const;
+  // presence agent
+  PString m_presenceAgent;
+
+  PString AsXML() const;
+
+  void PrintOn(ostream & strm) const;
+  friend ostream & operator<<(ostream & strm, const SIPPresenceInfo & info) { info.PrintOn(strm); return strm; }
+
+  static State FromSIPActivityString(const PString & str);
+
+  static bool AsSIPActivityString(State state, PString & str);
+  bool AsSIPActivityString(PString & str) const;
+
+  PString m_personId;
 };
 
 
