@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24146 $
+ * $Revision: 24536 $
  * $Author: rjongbloed $
- * $Date: 2010-03-22 21:58:12 -0500 (Mon, 22 Mar 2010) $
+ * $Date: 2010-06-23 22:15:38 -0500 (Wed, 23 Jun 2010) $
  */
 
 #ifndef OPAL_SIP_SIPCON_H
@@ -502,7 +502,7 @@ class SIPConnection : public OpalRTPConnection
       unsigned sessionID,
       OpalRTPSessionManager & rtpSessions,
       SDPSessionDescription & sdpOut,
-      bool isReINVITE
+      bool offerOpenMediaStreamOnly = false
     );
     virtual bool AnswerSDPMediaDescription(
       const SDPSessionDescription & sdpIn,
