@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23493 $
+ * $Revision: 24198 $
  * $Author: rjongbloed $
- * $Date: 2009-09-19 02:00:38 +0000 (Sat, 19 Sep 2009) $
+ * $Date: 2010-04-06 20:26:52 -0500 (Tue, 06 Apr 2010) $
  */
 
 #ifndef OPAL_SIP_SIPPDU_H
@@ -42,6 +42,7 @@
 
 #include <ptclib/mime.h>
 #include <ptclib/url.h>
+#include <ptclib/http.h>
 #include <sip/sdp.h>
 #include <opal/rtpconn.h>
 
@@ -53,6 +54,8 @@ class OpalProductInfo;
 class SIPEndPoint;
 class SIPConnection;
 class SIP_PDU;
+class SIPSubscribeHandler;
+class SIPDialogContext;
 
 
 /////////////////////////////////////////////////////////////////////////
@@ -279,9 +282,8 @@ class SIPMIMEInfo : public PMIMEInfo
     void SetCallID(const PString & v);
 
     PString GetContact() const;
-    bool GetContacts(std::list<SIPURL> & contacts) const;
+    bool GetContacts(std::set<SIPURL> & contacts) const;
     void SetContact(const PString & v);
-    void SetContact(const SIPURL & url);
 
     PString GetSubject() const;
     void SetSubject(const PString & v);
@@ -343,7 +345,7 @@ class SIPMIMEInfo : public PMIMEInfo
     PString GetEvent() const;
     void SetEvent(const PString & v);
     
-    PCaselessString GetSubscriptionState() const;
+    PCaselessString GetSubscriptionState(PStringToString & info) const;
     void SetSubscriptionState(const PString & v);
     
     PString GetUserAgent() const;
@@ -370,6 +372,9 @@ class SIPMIMEInfo : public PMIMEInfo
     void GetAlertInfo(PString & info, int & appearance);
     void SetAlertInfo(const PString & info, int appearance);
 
+    PString GetAllowEvents() const;
+    void SetAllowEvents(const PString & v);
+
     /** return the value of a header field parameter, empty if none
      */
     PString GetFieldParameter(
@@ -424,99 +429,22 @@ class SIPMIMEInfo : public PMIMEInfo
 /////////////////////////////////////////////////////////////////////////
 // SIPAuthentication
 
-class SIPAuthentication : public PObject
+typedef PHTTPClientAuthentication SIPAuthentication;
+
+class SIPAuthenticator : public PHTTPClientAuthentication::AuthObject
 {
-  PCLASSINFO(SIPAuthentication, PObject);
   public:
-    SIPAuthentication();
+    SIPAuthenticator(SIP_PDU & pdu);
+    virtual PMIMEInfo & GetMIME();
+    virtual PString GetURI();
+    virtual PString GetEntityBody();
+    virtual PString GetMethod();
 
-    virtual Comparison Compare(
-      const PObject & other
-    ) const;
-
-    virtual PBoolean Parse(
-      const PString & auth,
-      PBoolean proxy
-    ) = 0;
-
-    virtual PBoolean Authorise(
-      SIP_PDU & pdu
-    ) const =  0;
-
-    virtual PBoolean IsProxy() const               { return isProxy; }
-
-    virtual PString GetUsername() const   { return username; }
-    virtual PString GetPassword() const   { return password; }
-    virtual PString GetAuthRealm() const  { return PString::Empty(); }
-
-    virtual void SetUsername(const PString & user) { username = user; }
-    virtual void SetPassword(const PString & pass) { password = pass; }
-    virtual void SetAuthRealm(const PString &)     { }
-
-    PString GetAuthParam(const PString & auth, const char * name) const;
-    PString AsHex(PMessageDigest5::Code & digest) const;
-    PString AsHex(const PBYTEArray & data) const;
-
-    static SIPAuthentication * ParseAuthenticationRequired(bool isProxy,
-                                                const PString & line,
-                                                      PString & errorMsg);
-
-  protected:
-    PBoolean  isProxy;
-
-    PString   username;
-    PString   password;
+  protected:  
+    SIP_PDU & m_pdu;
 };
 
-typedef PFactory<SIPAuthentication> SIPAuthenticationFactory;
 
-/////////////////////////////////////////////////////////////////////////
-
-class SIPDigestAuthentication : public SIPAuthentication
-{
-  PCLASSINFO(SIPDigestAuthentication, SIPAuthentication);
-  public:
-    SIPDigestAuthentication();
-
-    SIPDigestAuthentication & operator =(
-      const SIPDigestAuthentication & auth
-    );
-
-    virtual Comparison Compare(
-      const PObject & other
-    ) const;
-
-    virtual PBoolean Parse(
-      const PString & auth,
-      PBoolean proxy
-    );
-
-    virtual PBoolean Authorise(
-      SIP_PDU & pdu
-    ) const;
-
-    virtual PString GetAuthRealm() const         { return authRealm; }
-    virtual void SetAuthRealm(const PString & r) { authRealm = r; }
-
-    enum Algorithm {
-      Algorithm_MD5,
-      NumAlgorithms
-    };
-    const PString & GetNonce() const       { return nonce; }
-    Algorithm GetAlgorithm() const         { return algorithm; }
-    const PString & GetOpaque() const      { return opaque; }
-
-  protected:
-    PString   authRealm;
-    PString   nonce;
-    Algorithm algorithm;
-    PString   opaque;
-
-    PBoolean qopAuth;
-    PBoolean qopAuthInt;
-    PString cnonce;
-    mutable PAtomicInteger nonceCount;
-};
 
 /////////////////////////////////////////////////////////////////////////
 // SIP_PDU
@@ -623,25 +551,8 @@ class SIP_PDU : public PSafeObject
       MaxSize = 65535
     };
 
-    SIP_PDU();
-
-    /** Construct a Request message
-     */
     SIP_PDU(
-      Methods method,
-      const SIPURL & dest,
-      const PString & to,
-      const PString & from,
-      const PString & callID,
-      unsigned cseq,
-      const OpalTransportAddress & via
-    );
-    /** Construct a Request message for requests in a dialog
-     */
-    SIP_PDU(
-      Methods method,
-      SIPConnection & connection,
-      const OpalTransport & transport
+      Methods method = SIP_PDU::NumMethods
     );
 
     /** Construct a Response message
@@ -654,6 +565,7 @@ class SIP_PDU : public PSafeObject
       const char * extra = NULL,
       const SDPSessionDescription * sdp = NULL
     );
+
     SIP_PDU(const SIP_PDU &);
     SIP_PDU & operator=(const SIP_PDU &);
     ~SIP_PDU();
@@ -662,20 +574,19 @@ class SIP_PDU : public PSafeObject
       ostream & strm
     ) const;
 
-    void Construct(
-      Methods method
-    );
-    void Construct(
-      Methods method,
+    void InitialiseHeaders(
       const SIPURL & dest,
-      const PString & to,
-      const PString & from,
+      const SIPURL & to,
+      const SIPURL & from,
       const PString & callID,
       unsigned cseq,
-      const OpalTransportAddress & via
+      const PString & via
     );
-    void Construct(
-      Methods method,
+    void InitialiseHeaders(
+      SIPDialogContext & dialog,
+      const PString & via = PString::Empty()
+    );
+    void InitialiseHeaders(
       SIPConnection & connection,
       const OpalTransport & transport
     );
@@ -694,7 +605,13 @@ class SIP_PDU : public PSafeObject
     /**Update the VIA field following RFC3261, 18.2.1 and RFC3581.
       */
     void AdjustVia(OpalTransport & transport);
-    
+
+    PString CreateVia(
+      SIPEndPoint & endpoint,
+      const OpalTransport & transport,
+      SIPConnection * connection = NULL
+    );
+
     /**Read PDU from the specified transport.
       */
     PBoolean Read(
@@ -731,33 +648,37 @@ class SIP_PDU : public PSafeObject
 
     PString GetTransactionID() const;
 
-    Methods GetMethod() const                { return method; }
-    StatusCodes GetStatusCode () const       { return statusCode; }
-    const SIPURL & GetURI() const            { return uri; }
-    unsigned GetVersionMajor() const         { return versionMajor; }
-    unsigned GetVersionMinor() const         { return versionMinor; }
-    const PString & GetEntityBody() const    { return entityBody; }
-          PString & GetEntityBody()          { return entityBody; }
-    const PString & GetInfo() const          { return info; }
-    const SIPMIMEInfo & GetMIME() const      { return mime; }
-          SIPMIMEInfo & GetMIME()            { return mime; }
-    void SetURI(const SIPURL & newuri)       { uri = newuri; }
+    Methods GetMethod() const                { return m_method; }
+    StatusCodes GetStatusCode () const       { return m_statusCode; }
+    void SetStatusCode (StatusCodes c)       { m_statusCode = c; }
+    const SIPURL & GetURI() const            { return m_uri; }
+    unsigned GetVersionMajor() const         { return m_versionMajor; }
+    unsigned GetVersionMinor() const         { return m_versionMinor; }
+    const PString & GetEntityBody() const    { return m_entityBody; }
+    void SetEntityBody(const PString & body) { m_entityBody = body; }
+    const PString & GetInfo() const          { return m_info; }
+    void SetInfo(const PString & info)       { m_info = info; }
+    const SIPMIMEInfo & GetMIME() const      { return m_mime; }
+          SIPMIMEInfo & GetMIME()            { return m_mime; }
+    void SetURI(const SIPURL & newuri)       { m_uri = newuri; }
     SDPSessionDescription * GetSDP();
     void SetSDP(SDPSessionDescription * sdp);
 
   protected:
-    Methods     method;                 // Request type, ==NumMethods for Response
-    StatusCodes statusCode;
-    SIPURL      uri;                    // display name & URI, no tag
-    unsigned    versionMajor;
-    unsigned    versionMinor;
-    PString     info;
-    SIPMIMEInfo mime;
-    PString     entityBody;
+    Methods     m_method;                 // Request type, ==NumMethods for Response
+    StatusCodes m_statusCode;
+    SIPURL      m_uri;                    // display name & URI, no tag
+    unsigned    m_versionMajor;
+    unsigned    m_versionMinor;
+    PString     m_info;
+    SIPMIMEInfo m_mime;
+    PString     m_entityBody;
 
     SDPSessionDescription * m_SDP;
 
-    mutable PString transactionID;
+    mutable PString m_transactionID;
+
+    bool m_usePeerTransportAddress;
 };
 
 
@@ -779,6 +700,11 @@ class SIPDialogContext
   public:
     SIPDialogContext();
 
+    PString AsString() const;
+    bool FromString(
+      const PString & str
+    );
+
     const PString & GetCallID() const { return m_callId; }
     void SetCallID(const PString & id) { m_callId = id; }
 
@@ -802,14 +728,24 @@ class SIPDialogContext
 
     const PStringList & GetRouteSet() const { return m_routeSet; }
     void SetRouteSet(const PStringList & routes) { m_routeSet = routes; }
-    void UpdateRouteSet(const SIPURL & proxy);
+
+    const SIPURL & GetProxy() const { return m_proxy; }
+    void SetProxy(const SIPURL & proxy, bool addToRouteSet);
 
     void Update(const SIP_PDU & response);
 
     unsigned GetNextCSeq(unsigned inc = 1) { return m_lastSentCSeq += inc; }
     bool IsDuplicateCSeq(unsigned sequenceNumber);
 
-    bool IsEstablished() const { return !m_remoteTag.IsEmpty(); }
+    bool IsEstablished() const
+    {
+      return !m_callId.IsEmpty() &&
+             !m_requestURI.IsEmpty() &&
+             !m_localTag.IsEmpty() &&
+             !m_remoteTag.IsEmpty();
+    }
+
+    bool UsePeerTransportAddress() const { return m_usePeerTransportAddress; }
 
   protected:
     PString     m_callId;
@@ -821,9 +757,46 @@ class SIPDialogContext
     PStringList m_routeSet;
     unsigned    m_lastSentCSeq;
     unsigned    m_lastReceivedCSeq;
+    bool        m_usePeerTransportAddress;
+    SIPURL      m_proxy;
 };
 
 
+/////////////////////////////////////////////////////////////////////////
+
+struct SIPParameters
+{
+  SIPParameters(
+    const PString & aor = PString::Empty(),
+    const PString & remote = PString::Empty()
+  );
+
+  void Normalise(
+    const PString & defaultUser,
+    const PTimeInterval & defaultExpire
+  );
+
+  PCaselessString m_remoteAddress;
+  PCaselessString m_localAddress;
+  PCaselessString m_proxyAddress;
+  PCaselessString m_addressOfRecord;
+  PCaselessString m_contactAddress;
+  PString         m_authID;
+  PString         m_password;
+  PString         m_realm;
+  unsigned        m_expire;
+  unsigned        m_restoreTime;
+  PTimeInterval   m_minRetryTime;
+  PTimeInterval   m_maxRetryTime;
+  void          * m_userData;
+};
+
+
+#if PTRACING
+ostream & operator<<(ostream & strm, const SIPParameters & params);
+#endif
+
+
 /////////////////////////////////////////////////////////////////////////
 // SIPTransaction
 
@@ -842,54 +815,46 @@ class SIPTransaction : public SIP_PDU
     PCLASSINFO(SIPTransaction, SIP_PDU);
   public:
     SIPTransaction(
+      Methods method,
       SIPEndPoint   & endpoint,
-      OpalTransport & transport,
-      const PTimeInterval & minRetryTime = PMaxTimeInterval, 
-      const PTimeInterval & maxRetryTime = PMaxTimeInterval
+      OpalTransport & transport
     );
     /** Construct a transaction for requests in a dialog.
      *  The transport is used to determine the local address
      */
     SIPTransaction(
-      SIPConnection & connection,
-      OpalTransport & transport,
-      Methods method = NumMethods
+      Methods method,
+      SIPConnection & connection
     );
     ~SIPTransaction();
 
     PBoolean Start();
-    bool IsTrying()     const { return state == Trying; }
-    bool IsProceeding() const { return state == Proceeding; }
-    bool IsInProgress() const { return state == Trying || state == Proceeding; }
-    bool IsFailed()     const { return state > Terminated_Success; }
-    bool IsCompleted()  const { return state >= Completed; }
-    bool IsCanceled()   const { return state == Cancelling || state == Terminated_Cancelled || state == Terminated_Aborted; }
-    bool IsTerminated() const { return state >= Terminated_Success; }
+    bool IsTrying()     const { return m_state == Trying; }
+    bool IsProceeding() const { return m_state == Proceeding; }
+    bool IsInProgress() const { return m_state == Trying || m_state == Proceeding; }
+    bool IsFailed()     const { return m_state > Terminated_Success; }
+    bool IsCompleted()  const { return m_state >= Completed; }
+    bool IsCanceled()   const { return m_state == Cancelling || m_state == Terminated_Cancelled || m_state == Terminated_Aborted; }
+    bool IsTerminated() const { return m_state >= Terminated_Success; }
 
-    void WaitForCompletion();
+    void WaitForTermination();
     PBoolean Cancel();
     void Abort();
 
     virtual PBoolean OnReceivedResponse(SIP_PDU & response);
     virtual PBoolean OnCompleted(SIP_PDU & response);
 
-    OpalTransport & GetTransport() const  { return transport; }
-    SIPConnection * GetConnection() const { return connection; }
-    PString         GetInterface() const { return m_localInterface; }
+    OpalTransport & GetTransport()  const { return m_transport; }
+    SIPConnection * GetConnection() const { return m_connection; }
+    PString         GetInterface()  const { return m_localInterface; }
 
     static PString GenerateCallID();
-    
+
   protected:
-    void Construct(
-      const PTimeInterval & minRetryTime = PMaxTimeInterval,
-      const PTimeInterval & maxRetryTime = PMaxTimeInterval
-    );
-    void Construct(
-      Methods method, 
-      SIPDialogContext & dialog
-    );
     bool SendPDU(SIP_PDU & pdu);
     bool ResendCANCEL();
+    void SetParameters(const SIPParameters & params);
+    void SetContact(const SIPURL & uri);
 
     PDECLARE_NOTIFIER(PTimer, SIPTransaction, OnRetry);
     PDECLARE_NOTIFIER(PTimer, SIPTransaction, OnTimeout);
@@ -910,22 +875,27 @@ class SIPTransaction : public SIP_PDU
     };
     virtual void SetTerminated(States newState);
 
-    SIPEndPoint           & endpoint;
-    OpalTransport         & transport;
-    PSafePtr<SIPConnection> connection;
-    PTimeInterval           retryTimeoutMin; 
-    PTimeInterval           retryTimeoutMax; 
+    SIPEndPoint           & m_endpoint;
+    OpalTransport         & m_transport;
+    PSafePtr<SIPConnection> m_connection;
+    PTimeInterval           m_retryTimeoutMin; 
+    PTimeInterval           m_retryTimeoutMax; 
+
+    States     m_state;
+    unsigned   m_retry;
+    PTimer     m_retryTimer;
+    PTimer     m_completionTimer;
+    PSyncPoint m_terminated;
 
-    States     state;
-    unsigned   retry;
-    PTimer     retryTimer;
-    PTimer     completionTimer;
-    PSyncPoint completed;
     PString              m_localInterface;
     OpalTransportAddress m_remoteAddress;
 };
 
 
+#define OPAL_PROXY_PARAM    "OPAL-proxy"
+#define OPAL_LOCAL_ID_PARAM "OPAL-local-id"
+
+
 /////////////////////////////////////////////////////////////////////////
 // SIPInvite
 
@@ -940,17 +910,16 @@ class SIPInvite : public SIPTransaction
   public:
     SIPInvite(
       SIPConnection & connection,
-      OpalTransport & transport,
       const OpalRTPSessionManager & sm
     );
 
     virtual PBoolean OnReceivedResponse(SIP_PDU & response);
 
-    const OpalRTPSessionManager & GetSessionManager() const { return rtpSessions; }
-          OpalRTPSessionManager & GetSessionManager()       { return rtpSessions; }
+    const OpalRTPSessionManager & GetSessionManager() const { return m_rtpSessions; }
+          OpalRTPSessionManager & GetSessionManager()       { return m_rtpSessions; }
 
   protected:
-    OpalRTPSessionManager rtpSessions;
+    OpalRTPSessionManager m_rtpSessions;
 };
 
 
@@ -975,27 +944,25 @@ class SIPRegister : public SIPTransaction
     };
 
     /// Registrar parameters
-    struct Params {
-      Params();
+    struct Params : public SIPParameters {
+      Params()
+        : m_registrarAddress(m_remoteAddress)
+        , m_compatibility(SIPRegister::e_FullyCompliant)
+      { }
 
-      PString       m_addressOfRecord;
-      PString       m_registrarAddress;
-      PString       m_contactAddress;
-      PString       m_authID;
-      PString       m_password;
-      PString       m_realm;
-      unsigned      m_expire;
-      unsigned      m_restoreTime;
-      PTimeInterval m_minRetryTime;
-      PTimeInterval m_maxRetryTime;
-      void        * m_userData;
+      Params(const Params & param)
+        : SIPParameters(param)
+        , m_registrarAddress(m_remoteAddress)
+        , m_compatibility(param.m_compatibility)
+      { }
+
+      PCaselessString  & m_registrarAddress; // For backward compatibility
       CompatibilityModes m_compatibility;
     };
 
     SIPRegister(
       SIPEndPoint   & endpoint,
       OpalTransport & transport,
-      const SIPURL & proxy,
       const PString & callId,
       unsigned cseq,
       const Params & params
@@ -1003,50 +970,113 @@ class SIPRegister : public SIPTransaction
 };
 
 
+#if PTRACING
+ostream & operator<<(ostream & strm, SIPRegister::CompatibilityModes mode);
+ostream & operator<<(ostream & strm, const SIPRegister::Params & params);
+#endif
+
+
 /////////////////////////////////////////////////////////////////////////
 
 class SIPSubscribe : public SIPTransaction
 {
     PCLASSINFO(SIPSubscribe, SIPTransaction);
   public:
-    /** Valid types for a presence event
+    /** Valid types for an event package
      */
     enum PredefinedPackages {
       MessageSummary,
       Presence,
       Dialog,
-      NumPredefinedPackages
+
+      NumPredefinedPackages,
+
+      Watcher = 0x8000,
+
+      MessageSummaryWatcher = Watcher|MessageSummary,
+      PresenceWatcher       = Watcher|Presence,
+      DialogWatcher         = Watcher|Dialog,
+
+      PackageMask = Watcher-1
     };
+    friend PredefinedPackages operator|(PredefinedPackages p1, PredefinedPackages p2) { return (PredefinedPackages)((int)p1|(int)p2); }
 
     class EventPackage : public PCaselessString
     {
-        PCLASSINFO(EventPackage, PCaselessString);
+      PCLASSINFO(EventPackage, PCaselessString);
       public:
-        EventPackage(PredefinedPackages);
-        EventPackage(const PString & str) : PCaselessString(str) { }
-        EventPackage(const char * cstr) : PCaselessString(cstr) { }
-        bool operator==(PredefinedPackages) const;
+        EventPackage(PredefinedPackages = NumPredefinedPackages);
+        explicit EventPackage(const PString & str) : PCaselessString(str) { }
+        explicit EventPackage(const char   *  str) : PCaselessString(str) { }
+
+        EventPackage & operator=(PredefinedPackages pkg);
+        EventPackage & operator=(const PString & str) { PCaselessString::operator=(str); return *this; }
+        EventPackage & operator=(const char   *  str) { PCaselessString::operator=(str); return *this; }
+
+        bool operator==(PredefinedPackages pkg) const { return Compare(EventPackage(pkg)) == EqualTo; }
         bool operator==(const PString & str) const { return Compare(str) == EqualTo; }
         bool operator==(const char * cstr) const { return InternalCompare(0, P_MAX_INDEX, cstr) == EqualTo; }
         virtual Comparison InternalCompare(PINDEX offset, PINDEX length, const char * cstr) const;
+
+        bool IsWatcher() const;
     };
 
-    struct Params {
-      Params(PredefinedPackages pkg = NumPredefinedPackages);
+    /** Information provided on the subscription status. */
+    struct SubscriptionStatus {
+      SIPSubscribeHandler * m_handler;           ///< Handler for subscription
+      PString               m_addressofRecord;   ///< Address of record for registration
+      bool                  m_wasSubscribing;    ///< Was registering or unregistering
+      bool                  m_reSubscribing;     ///< Was a registration refresh
+      SIP_PDU::StatusCodes  m_reason;            ///< Reason for status change
+      OpalProductInfo       m_productInfo;       ///< Server product info from registrar if available.
+      void                * m_userData;          ///< User data corresponding to this registration
+    };
 
-      EventPackage  m_eventPackage;
-      PString       m_agentAddress;
-      PString       m_addressOfRecord;
-      PString       m_contactAddress;
-      PString       m_authID;
-      PString       m_password;
-      PString       m_realm;
-      unsigned      m_expire;
-      unsigned      m_restoreTime;
-      PTimeInterval m_minRetryTime;
-      PTimeInterval m_maxRetryTime;
-      void          * m_userData;
-      PString       m_from;
+    struct NotifyCallbackInfo {
+      NotifyCallbackInfo(
+        SIPEndPoint & ep,
+        OpalTransport & trans,
+        SIP_PDU & notify,
+        SIP_PDU & response
+      );
+
+      bool SendResponse(
+        SIP_PDU::StatusCodes status,
+        const char * extra = NULL
+      );
+
+      SIPEndPoint   & m_endpoint;
+      OpalTransport & m_transport;
+      SIP_PDU       & m_notify;
+      SIP_PDU       & m_response;
+      bool            m_sendResponse;
+    };
+
+    struct Params : public SIPParameters
+    {
+      Params(PredefinedPackages pkg = NumPredefinedPackages)
+        : m_agentAddress(m_remoteAddress)
+        , m_eventPackage(pkg)
+        , m_eventList(false)
+      { }
+
+      Params(const Params & param)
+        : SIPParameters(param)
+        , m_agentAddress(m_remoteAddress)
+        , m_eventPackage(param.m_eventPackage)
+        , m_eventList(param.m_eventList)
+        , m_contentType(param.m_contentType)
+        , m_onSubcribeStatus(param.m_onSubcribeStatus)
+        , m_onNotify(param.m_onNotify)
+      { }
+
+      PCaselessString & m_agentAddress; // For backward compatibility
+      EventPackage      m_eventPackage;
+      bool              m_eventList;    // Enable RFC4662
+      PCaselessString   m_contentType;  // May be \n separated list of types
+
+      PNotifierTemplate<const SubscriptionStatus &> m_onSubcribeStatus;
+      PNotifierTemplate<NotifyCallbackInfo &> m_onNotify;
     };
 
     SIPSubscribe(
@@ -1058,6 +1088,11 @@ class SIPSubscribe : public SIPTransaction
 };
 
 
+#if PTRACING
+ostream & operator<<(ostream & strm, const SIPSubscribe::Params & params);
+#endif
+
+
 typedef SIPSubscribe::EventPackage SIPEventPackage;
 
 
@@ -1106,7 +1141,7 @@ class SIPPublish : public SIPTransaction
       OpalTransport & trans,
       const PString & id,
       const PString & sipIfMatch,
-      SIPSubscribe::Params & params,
+      const SIPSubscribe::Params & params,
       const PString & body
     );
 };
@@ -1120,21 +1155,8 @@ class SIPRefer : public SIPTransaction
   public:
     SIPRefer(
       SIPConnection & connection,
-      OpalTransport & transport,
-      const SIPURL & refer
-    );
-    SIPRefer(
-      SIPConnection & connection,
-      OpalTransport & transport,
-      const SIPURL & refer,
-      const SIPURL & referred_by
-    );
-  protected:
-    void Construct(
-      SIPConnection & connection,
-      OpalTransport & transport,
-      const SIPURL & refer,
-      const SIPURL & referred_by
+      const SIPURL & referTo,
+      const SIPURL & referred_by = SIPURL()
     );
 };
 
@@ -1150,7 +1172,6 @@ class SIPReferNotify : public SIPTransaction
   public:
     SIPReferNotify(
       SIPConnection & connection,
-      OpalTransport & transport,
       StatusCodes code
     );
 };
@@ -1162,17 +1183,34 @@ class SIPReferNotify : public SIPTransaction
  */
 class SIPMessage : public SIPTransaction
 {
-    PCLASSINFO(SIPMessage, SIPTransaction);
-    
+  PCLASSINFO(SIPMessage, SIPTransaction);
+  public:
+    struct Params : public SIPParameters
+    {
+      Params()
+      { 
+        m_expire = 5000;
+      }
+
+      Params(const Params & param)
+        : SIPParameters(param)
+        , m_contentType(param.m_contentType)
+        , m_id(param.m_id)
+      { }
+
+      PCaselessString m_contentType;
+      PString m_id;
+    };
+
+
   public:
     SIPMessage(
-      SIPEndPoint & ep,
-      OpalTransport & trans,
-      const SIPURL & proxy,
-      const SIPURL & to,
-      const PString & id,
-      const PString & body
+        SIPEndPoint & ep,
+        OpalTransport & trans,
+        const Params & params,
+        const PString & body
     );
+    SIPURL m_localAddress;
 };
 
 
@@ -1219,11 +1257,11 @@ class SIPPing : public SIPTransaction
 
   public:
     SIPPing(
-               SIPEndPoint & ep,
-             OpalTransport & trans,
-              const SIPURL & address,
-              const PString & body = PString::Empty()
-   );
+      SIPEndPoint & ep,
+      OpalTransport & trans,
+      const SIPURL & address,
+      const PString & body = PString::Empty()
+    );
 };
 
 
