@@ -21,15 +21,18 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22640 $
+ * $Revision: 23865 $
  * $Author: rjongbloed $
- * $Date: 2009-05-18 22:19:20 -0500 (Mon, 18 May 2009) $
+ * $Date: 2009-12-10 00:01:28 -0600 (Thu, 10 Dec 2009) $
  */
 
 #include "precompile.h"
 #include "main.h"
 
 
+// -ttttt c:\temp\testfax.tif sip:fax@10.0.1.11
+// -ttttt c:\temp\incoming.tif
+
 PCREATE_PROCESS(FaxOPAL);
 
 
@@ -56,13 +59,14 @@ void FaxOPAL::Main()
              "G-gk-id:"
              "h-help."
              "H-h323:"
-             "I-inband."
+             "I-inband-detect."
+             "i-inband-send."
+             "L-lines:"
              "m-mode:"
              "N-stun:"
              "p-password:"
              "P-proxy:"
              "r-register:"
-             "s-spandsp:"
              "S-sip:"
              "u-user:"
 #if PTRACING
@@ -83,13 +87,7 @@ void FaxOPAL::Main()
             "Available options are:\n"
             "  --help                  : print this help message.\n"
             "  -d or --directory dir   : Set default directory for fax receive\n"
-            "  -s or --spandsp exe     : Set location of spandsp_util.exe\n"
             "  -a or --audio           : Send fax as G.711 audio\n"
-            "  -m or --mode m          : T.38 synchronisation mode:\n"
-            "                            \"Wait\" wait for remote to switch,\n"
-            "                            \"Timeout\" switch after a short time\n"
-            "                            \"UserInput\" send/detect CNG/CED tones\n"
-            "                            \"InBand\" send/detect CNG/CED tones in band\n"
             "  -u or --user name       : Set local username, defaults to OS username.\n"
             "  -p or --password pwd    : Set password for authentication.\n"
 #if OPAL_SIP
@@ -102,8 +100,10 @@ void FaxOPAL::Main()
             "  -g or --gk-host host    : H.323 gatekeeper host.\n"
             "  -G or --gk-id id        : H.323 gatekeeper identifier.\n"
 #endif
+            "  -L or --lines devices   : Set Line Interface Devices.\n"
             "  -N or --stun server     : Set NAT traversal STUN server.\n"
-            "  -I or --inband          : Disable detection of in-band tones.\n"
+            "  -I or --inband-detect   : Disable detection of in-band tones.\n"
+            "  -i or --inband-send     : Disable transmission of in-band tones.\n"
 #if PTRACING
             "  -o or --output file     : file name for output of log messages\n"       
             "  -t or --trace           : degree of verbosity in error log (more times for more detail)\n"     
@@ -117,6 +117,19 @@ void FaxOPAL::Main()
 
   m_manager = new MyManager();
 
+  if (args.HasOption('u'))
+    m_manager->SetDefaultUserName(args.GetOptionString('u'));
+
+  PString prefix = args.HasOption('a') ? "fax" : "t38";
+
+  // Create audio or T.38 fax endpoint.
+  MyFaxEndPoint * fax  = new MyFaxEndPoint(*m_manager);
+  if (args.HasOption('d'))
+    fax->SetDefaultDirectory(args.GetOptionString('d'));
+
+
+  PCaselessString interfaces;
+
   if (args.HasOption('N')) {
     PSTUNClient::NatTypes nat = m_manager->SetSTUNServer(args.GetOptionString('N'));
     cout << "STUN server \"" << m_manager->GetSTUNClient()->GetServer() << "\" replies " << nat;
@@ -126,10 +139,6 @@ void FaxOPAL::Main()
     cout << endl;
   }
 
-  if (args.HasOption('u'))
-    m_manager->SetDefaultUserName(args.GetOptionString('u'));
-
-  PCaselessString interfaces;
 
 #if OPAL_SIP
   // Set up SIP
@@ -162,7 +171,11 @@ void FaxOPAL::Main()
         cerr << "Could not complete SIP registration for " << aor << endl;
         return;
       }
+
+      m_manager->AddRouteEntry(prefix + ":.*\ttel:.* = sip:<dn>" + aor.Mid(aor.Find('@')));
     }
+
+    m_manager->AddRouteEntry("sip.*:.* = " + prefix + ":" + args[0] + ";receive");
   }
 #endif // OPAL_SIP
 
@@ -177,39 +190,43 @@ void FaxOPAL::Main()
       return;
     }
 
-    if (args.HasOption('g') || args.HasOption('G'))
-      h323->UseGatekeeper(args.GetOptionString('g'), args.GetOptionString('G'));
+    if (args.HasOption('g') || args.HasOption('G')) {
+      if (!h323->UseGatekeeper(args.GetOptionString('g'), args.GetOptionString('G'))) {
+        cerr << "Could not complete gatekeeper registration" << endl;
+        return;
+      }
+
+      m_manager->AddRouteEntry(prefix + ":.*\ttel:.* = h323:<dn>");
+    }
+
+    m_manager->AddRouteEntry("h323.*:.* = " + prefix + ":" + args[0] + ";receive");
   }
 #endif // OPAL_H323
 
-  // Create audio or T.38 fax endpoint.
-  MyFaxEndPoint * fax  = new MyFaxEndPoint(*m_manager);
-  if (args.HasOption('d'))
-    fax->SetDefaultDirectory(args.GetOptionString('d'));
-  if (args.HasOption('s'))
-    fax->SetSpanDSP(args.GetOptionString('s'));
-  if (args.HasOption('m'))
-    fax->SetDefaultStringOption("Fax-Sync-Mode", args.GetOptionString('m'));
 
-  // Route SIP/H.323 calls to the Fax endpoint
-  PString prefix = args.HasOption('a') ? "fax" : "t38";
-#if OPAL_SIP
-  m_manager->AddRouteEntry("sip.*:.* = " + prefix + ":" + args[0] + ";receive");
-#endif
-#if OPAL_H323
-  m_manager->AddRouteEntry("h323.*:.* = " + prefix + ":" + args[0] + ";receive");
-#endif
+  // If we have LIDs speficied in command line, load them
+  if (args.HasOption('L')) {
+    OpalLineEndPoint * lines = new OpalLineEndPoint(*m_manager);
+    if (!lines->AddDeviceNames(args.GetOptionString('L').Lines()))
+      cerr << "Could not start Line Interface Device(s)" << endl;
+    else {
+      m_manager->AddRouteEntry("pots.*:.* = " + prefix + ":" + args[0] + ";receive");
+      m_manager->AddRouteEntry("pstn.*:.* = " + prefix + ":" + args[0] + ";receive");
+    }
+  }
 
-  // If no explicit protocol on URI, then send to SIP.
-  m_manager->AddRouteEntry(prefix+":.* = sip:<da>");
 
-  m_manager->DisableDetectInBandDTMF(args.HasOption('I'));
+  OpalConnection::StringOptions stringOptions;
+  if (args.HasOption('I'))
+    stringOptions.SetAt(OPAL_OPT_DETECT_INBAND_DTMF, "false");
+  if (args.HasOption('i'))
+    stringOptions.SetAt(OPAL_OPT_SEND_INBAND_DTMF, "false");
 
   if (args.GetCount() == 1)
     cout << "Awaiting incoming fax, saving as " << args[0] << " ... " << flush;
   else {
     PString token;
-    if (!m_manager->SetUpCall(prefix + ":" + args[0], args[1], token)) {
+    if (!m_manager->SetUpCall(prefix + ":" + args[0], args[1], token, NULL, 0, &stringOptions)) {
       cerr << "Could not start call to \"" << args[1] << '"' << endl;
       return;
     }
@@ -239,9 +256,7 @@ void MySIPEndPoint::OnRegistrationStatus(const RegistrationStatus & status)
 void MyFaxEndPoint::OnFaxCompleted(OpalFaxConnection & connection, bool failed)
 {
   OpalMediaStatistics stats;
-  OpalMediaStreamPtr stream = connection.GetMediaStream(OpalMediaType::Fax(), false);
-  if (stream != NULL)
-    stream->GetStatistics(stats);
+  connection.GetStatistics(stats);
   switch (stats.m_fax.m_result) {
     case -2 :
       cout << " failed to establish ";
