@@ -23,9 +23,9 @@
  *
  * Contributor(s): __________________________________
  *
- * $Revision: 19427 $
+ * $Revision: 24036 $
  * $Author: rjongbloed $
- * $Date: 2008-02-08 21:23:08 -0600 (Fri, 08 Feb 2008) $
+ * $Date: 2010-02-06 21:42:13 -0600 (Sat, 06 Feb 2010) $
  */
 
 #include <ptlib.h>
@@ -46,9 +46,6 @@
 
 #define new PNEW
 
-namespace PWLibStupidLinkerHacks {
-  int h235AuthLoader;
-};
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -316,7 +313,7 @@ H235Authenticator::ValidationResult
 
 ///////////////////////////////////////////////////////////////////////////////
 
-static PFactory<H235Authenticator>::Worker<H235AuthSimpleMD5> factoryH235AuthSimpleMD5("SimpleMD5");
+PFACTORY_CREATE(PFactory<H235Authenticator>, H235AuthSimpleMD5, "SimpleMD5", false);
 
 static const char OID_MD5[] = "1.2.840.113549.2.5";
 
@@ -337,16 +334,6 @@ const char * H235AuthSimpleMD5::GetName() const
 }
 
 
-static PWCharArray GetUCS2plusNULL(const PString & str)
-{
-  PWCharArray ucs2 = str.AsUCS2();
-  PINDEX len = ucs2.GetSize();
-  if (len > 0 && ucs2[len-1] != 0)
-    ucs2.SetSize(len+1);
-  return ucs2;
-}
-
-
 H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken()
 {
   if (!IsActive())
@@ -364,13 +351,13 @@ H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken()
   clearToken.m_tokenOID = "0.0";
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_generalID);
-  clearToken.m_generalID = GetUCS2plusNULL(localId);
+  clearToken.m_generalID.SetValueRaw(localId.AsUCS2()); // Use SetValueRaw to make sure trailing NULL is included
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_password);
-  clearToken.m_password = GetUCS2plusNULL(password);
+  clearToken.m_password.SetValueRaw(password.AsUCS2());
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_timeStamp);
-  clearToken.m_timeStamp = (int)time(NULL);
+  clearToken.m_timeStamp = (int)PTime().GetTimeInSeconds();
 
   // Encode it into PER
   PPER_Stream strm;
@@ -424,10 +411,10 @@ H235Authenticator::ValidationResult H235AuthSimpleMD5::ValidateCryptoToken(
   clearToken.m_tokenOID = "0.0";
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_generalID);
-  clearToken.m_generalID = GetUCS2plusNULL(alias);
+  clearToken.m_generalID.SetValueRaw(alias.AsUCS2()); // Use SetValueRaw to make sure trailing NULL is included
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_password);
-  clearToken.m_password = GetUCS2plusNULL(password);
+  clearToken.m_password.SetValueRaw(password.AsUCS2());
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_timeStamp);
   clearToken.m_timeStamp = cryptoEPPwdHash.m_timeStamp;
@@ -486,7 +473,7 @@ PBoolean H235AuthSimpleMD5::IsSecuredPDU(unsigned rasPDU, PBoolean received) con
 
 ///////////////////////////////////////////////////////////////////////////////
 
-static PFactory<H235Authenticator>::Worker<H235AuthCAT> factoryH235AuthCAT("SimpleCAT");
+PFACTORY_CREATE(PFactory<H235Authenticator>, H235AuthCAT, "SimpleCAT", false);
 
 static const char OID_CAT[] = "1.2.840.113548.10.1.2.1";
 
@@ -523,10 +510,10 @@ H235_ClearToken * H235AuthCAT::CreateClearToken()
   clearToken->m_tokenOID = OID_CAT;
 
   clearToken->IncludeOptionalField(H235_ClearToken::e_generalID);
-  clearToken->m_generalID = GetUCS2plusNULL(localId);
+  clearToken->m_generalID.SetValueRaw(localId.AsUCS2()); // Use SetValueRaw to make sure trailing NULL is included
 
   clearToken->IncludeOptionalField(H235_ClearToken::e_timeStamp);
-  clearToken->m_timeStamp = (int)time(NULL);
+  clearToken->m_timeStamp = (int)PTime().GetTimeInSeconds();
   PUInt32b timeStamp = (DWORD)clearToken->m_timeStamp;
 
   clearToken->IncludeOptionalField(H235_ClearToken::e_random);
