@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23566 $
+ * $Revision: 24506 $
  * $Author: rjongbloed $
- * $Date: 2009-10-01 01:06:00 -0500 (Thu, 01 Oct 2009) $
+ * $Date: 2010-06-17 21:40:22 -0500 (Thu, 17 Jun 2010) $
  */
 
 #include <ptlib.h>
@@ -47,6 +47,7 @@
 #include <h323/h323pdu.h>
 #include <h323/transaddr.h>
 #include <t38/h323t38.h>
+#include <codec/opalplugin.h>
 
 
 #define DEFINE_G711_CAPABILITY(cls, code, capName) \
@@ -1472,6 +1473,13 @@ PObject * H323H239VideoCapability::Clone() const
 }
 
 
+void H323H239VideoCapability::PrintOn(ostream & strm) const
+{
+  strm << GetMediaFormat() << '+';
+  H323ExtendedVideoCapability::PrintOn(strm);
+}
+
+
 PString H323H239VideoCapability::GetFormatName() const
 {
   static class H239VideoMediaFormat : public OpalMediaFormat { 
@@ -1479,7 +1487,8 @@ PString H323H239VideoCapability::GetFormatName() const
       H239VideoMediaFormat() 
         : OpalMediaFormat("H.239-Video", H239MediaType, RTP_DataFrame::MaxPayloadType, NULL, false, 0, 0, 0, 0)
       {
-        OpalMediaOption * option = new OpalMediaOptionUnsigned(OpalVideoFormat::ContentRoleMaskOption(), true, OpalMediaOption::AndMerge, 1, 1, 3);
+        OpalMediaOption * option = new OpalMediaOptionUnsigned(OpalVideoFormat::ContentRoleMaskOption(),
+                                                               true, OpalMediaOption::IntersectionMerge, 1, 1, 3);
 
         OpalMediaOption::H245GenericInfo genericInfo;
         genericInfo.ordinal = 1;
@@ -1530,7 +1539,7 @@ PObject * H323H239ControlCapability::Clone() const
 
 PString H323H239ControlCapability::GetFormatName() const
 {
-  static const OpalMediaFormat name("H.239-Control", H239MediaType, RTP_DataFrame::IllegalPayloadType, NULL, false, 0, 0, 0, 0);
+  static const OpalMediaFormat name("H.239-Control", H239MediaType, RTP_DataFrame::MaxPayloadType, NULL, false, 0, 0, 0, 0);
   return name;
 }
 
@@ -1984,6 +1993,7 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
    *  e_h263VideoCapability.
    */
   mediaPacketizations += "RFC2190";  // Always supported
+  mediaPacketizations += OpalPluginCodec_Identifer_H264_Aligned; // Always supported
   const H245_MultiplexCapability * muxCap = NULL;
   if (pdu.HasOptionalField(H245_TerminalCapabilitySet::e_multiplexCapability)) {
     muxCap = &pdu.m_multiplexCapability;
@@ -2023,38 +2033,40 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
         H323Capability * capability = allCapabilities.FindCapability(pdu.m_capabilityTable[i].m_capability);
         if (capability != NULL) {
           H323Capability * copy = (H323Capability *)capability->Clone();
-          copy->SetCapabilityNumber(pdu.m_capabilityTable[i].m_capabilityTableEntryNumber);
-          if (mediaPacketizations.GetSize() != 0) { // also update the mediaPacketizations option
-            OpalMediaFormat & mediaFormat = copy->GetWritableMediaFormat();
-            PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
-                                          mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
-            if (!packetizationString.IsEmpty()) {
-              // remove packetizations not signaled by the remote party
-              PStringArray packetizations = packetizationString.Tokenise(",");
-              for (PINDEX j = packetizations.GetSize(); j > 0; j--) {
-                if (!mediaPacketizations.Contains(packetizations[j-1])) {
-                  packetizations.RemoveAt(j-1);
-                }
-              }
+          if (!copy->OnReceivedPDU(pdu.m_capabilityTable[i].m_capability))
+            delete copy;
+          else {
+            copy->SetCapabilityNumber(pdu.m_capabilityTable[i].m_capabilityTableEntryNumber);
+            table.Append(copy);
 
-              // construct the new packetization string
-              packetizationString.MakeEmpty();
-              if (packetizations.GetSize() == 0) { // should not happen actually
-                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), PString::Empty());
-                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  PString::Empty());
-              } else {
-                packetizationString = packetizations[0];
-                for (PINDEX j = 1; j < packetizations.GetSize(); j++)
-                  packetizationString += "," + packetizations[j];
-                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), packetizationString);
-                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  packetizations[0]);
+            if (mediaPacketizations.GetSize() != 0) { // also update the mediaPacketizations option
+              OpalMediaFormat & mediaFormat = copy->GetWritableMediaFormat();
+              PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                                            mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
+              if (!packetizationString.IsEmpty()) {
+                // remove packetizations not signaled by the remote party
+                PStringArray packetizations = packetizationString.Tokenise(",");
+                for (PINDEX j = packetizations.GetSize(); j > 0; j--) {
+                  if (!mediaPacketizations.Contains(packetizations[j-1]))
+                    packetizations.RemoveAt(j-1);
+                }
+
+                // construct the new packetization string
+                packetizationString.MakeEmpty();
+                if (packetizations.GetSize() == 0) { // should not happen actually
+                  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), PString::Empty());
+                  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  PString::Empty());
+                }
+                else {
+                  packetizationString = packetizations[0];
+                  for (PINDEX j = 1; j < packetizations.GetSize(); j++)
+                    packetizationString += "," + packetizations[j];
+                  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), packetizationString);
+                  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  packetizations[0]);
+                }
               }
             }
           }
-          if (copy->OnReceivedPDU(pdu.m_capabilityTable[i].m_capability))
-            table.Append(copy);
-          else
-            delete copy;
         }
       }
     }
