@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
+ * $Revision: 23831 $
  * $Author: rjongbloed $
- * $Date: 2008-10-13 10:24:41 +1100 (Mon, 13 Oct 2008) $
+ * $Date: 2009-12-03 04:21:07 -0600 (Thu, 03 Dec 2009) $
  */
 
 #include <ptlib.h>
@@ -49,10 +49,12 @@
 
 #if OPAL_HAS_SIPIM
 
+const char SIP_IM[] = "sip-im";
+
 ////////////////////////////////////////////////////////////////////////////
 
 OpalSIPIMMediaType::OpalSIPIMMediaType()
-  : OpalIMMediaType("sip-im", "message|sip", 6)
+  : OpalIMMediaType(SIP_IM, "message|sip")
 {
 }
 
@@ -112,13 +114,15 @@ SDPMediaDescription * OpalSIPIMMediaType::CreateSDPMediaDescription(const OpalTr
 ///////////////////////////////////////////////////////////////////////////////////////////
 
 SDPSIPIMMediaDescription::SDPSIPIMMediaDescription(const OpalTransportAddress & address)
-  : SDPMediaDescription(address)
+  : SDPMediaDescription(address, SIP_IM)
 {
   SetDirection(SDPMediaDescription::SendRecv);
 }
 
 SDPSIPIMMediaDescription::SDPSIPIMMediaDescription(const OpalTransportAddress & address, const OpalTransportAddress & _transportAddr, const PString & _fromURL)
-  : SDPMediaDescription(address), transportAddress(_transportAddr), fromURL(_fromURL)
+  : SDPMediaDescription(address, SIP_IM)
+  , transportAddress(_transportAddr)
+  , fromURL(_fromURL)
 {
   SetDirection(SDPMediaDescription::SendRecv);
 }
@@ -172,7 +176,7 @@ OpalMediaFormatList SDPSIPIMMediaDescription::GetMediaFormats() const
 
 void SDPSIPIMMediaDescription::AddMediaFormat(const OpalMediaFormat & mediaFormat)
 {
-  if (!mediaFormat.IsTransportable() || !mediaFormat.IsValidForProtocol("sip") || mediaFormat.GetMediaType() != "sip-im") {
+  if (!mediaFormat.IsTransportable() || !mediaFormat.IsValidForProtocol("sip") || mediaFormat.GetMediaType() != SIP_IM) {
     PTRACE(4, "SIPIM\tSDP not including " << mediaFormat << " as it is not a valid SIPIM format");
     return;
   }
@@ -199,7 +203,7 @@ OpalMediaSession * OpalSIPIMMediaType::CreateMediaSession(OpalConnection & conn,
 ////////////////////////////////////////////////////////////////////////////////////////////
 
 OpalSIPIMMediaSession::OpalSIPIMMediaSession(OpalConnection & _conn, unsigned _sessionId)
-: OpalMediaSession(_conn, "sip-im", _sessionId)
+: OpalMediaSession(_conn, SIP_IM, _sessionId)
 {
   transportAddress = connection.GetTransport().GetLocalAddress();
   localURL         = connection.GetLocalPartyURL();
@@ -233,103 +237,14 @@ OpalMediaStream * OpalSIPIMMediaSession::CreateMediaStream(const OpalMediaFormat
                                                                          PBoolean isSource)
 {
   PTRACE(2, "SIPIM\tCreated " << (isSource ? "source" : "sink") << " media stream in " << (connection.IsOriginating() ? "originator" : "receiver") << " with local " << localURL << " and remote " << remoteURL);
-  return new OpalSIPIMMediaStream(connection, mediaFormat, sessionID, isSource, *this);
+  return new OpalIMMediaStream(connection, mediaFormat, sessionID, isSource);
 }
 
+
 void OpalSIPIMMediaSession::SetRemoteMediaAddress(const OpalTransportAddress &, const OpalMediaFormatList &)
 {
 }
 
-bool OpalSIPIMMediaSession::SendMessage(const PString & /*contentType*/, const PString & body)
-{
-  SIPEndPoint * ep = dynamic_cast<SIPEndPoint *>(&connection.GetEndPoint());
-  if (ep == NULL)
-    return false;
-
-  return ep->Message(remoteURL, body, localURL, callId);
-}
-
-bool OpalSIPIMMediaSession::SendIM(const PString & /*contentType*/, const PString & body)
-{
-  return connection.SendIM(OpalSIPIM, body);
-}
-
-
-////////////////////////////////////////////////////////////////////////////////////////////
-
-OpalSIPIMMediaStream::OpalSIPIMMediaStream(
-      OpalConnection & conn,
-      const OpalMediaFormat & mediaFormat, ///<  Media format for stream
-      unsigned sessionID,                  ///<  Session number for stream
-      bool isSource,                        ///<  Is a source stream
-      OpalSIPIMMediaSession & mediaSession
-)
-  : OpalIMMediaStream(conn, mediaFormat, sessionID, isSource)
-  , m_imSession(mediaSession)
-{
-}
-
-OpalSIPIMMediaStream::~OpalSIPIMMediaStream()
-{
-}
-
-bool OpalSIPIMMediaStream::Open()
-{
-  if (!OpalIMMediaStream::Open())
-    return false;
-
-  SIPEndPoint * ep = dynamic_cast<SIPEndPoint *>(&connection.GetEndPoint());
-  if (ep == NULL) 
-    return false;
-
-  ep->GetSIPIMManager().StartSession(&m_imSession);
-
-  return true;
-}
-
-
-PBoolean OpalSIPIMMediaStream::Close()
-{
-  if (!OpalIMMediaStream::Close())
-    return false;
-
-  SIPEndPoint * ep = dynamic_cast<SIPEndPoint *>(&connection.GetEndPoint());
-  if (ep == NULL) 
-    return false;
-
-  ep->GetSIPIMManager().EndSession(&m_imSession);
-
-  return true;
-}
-
-PBoolean OpalSIPIMMediaStream::ReadData(BYTE *,PINDEX,PINDEX &)
-{
-  PAssertAlways("Cannot ReadData from OpalSIPIMMediaStream");
-  return false;
-}
-
-PBoolean OpalSIPIMMediaStream::WriteData(
-      const BYTE * data,   ///<  Data to write
-            PINDEX length,       ///<  Length of data to read.
-          PINDEX & written     ///<  Length of data actually written
-    )
-{
-  if (!IsOpen())
-    return false;
-
-  bool stat = true;
-
-  if (length != 0 && data != NULL) {
-    // T.140 data has 3 bytes at the start of the data
-    if (length > 4) {
-      PString body((const char *)data + 3, length-3);
-      stat = m_imSession.SendMessage("", body);
-    }
-    written = length;
-  }
-
-  return stat;
-}
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -338,8 +253,9 @@ OpalSIPIMManager::OpalSIPIMManager(SIPEndPoint & endpoint)
 {
 }
 
-void OpalSIPIMManager::OnReceivedMessage(const SIP_PDU & pdu)
+void OpalSIPIMManager::OnReceivedMessage(const SIP_PDU & /*pdu*/)
 {
+#if 0
   PString callId = pdu.GetMIME().GetCallID();
   if (!callId.IsEmpty()) {
     PWaitAndSignal m(m_mutex);
@@ -348,24 +264,8 @@ void OpalSIPIMManager::OnReceivedMessage(const SIP_PDU & pdu)
       r->second->SendIM(pdu.GetMIME().GetContentEncoding(), pdu.GetEntityBody());
     }
   }
+#endif
 }
 
-bool OpalSIPIMManager::StartSession(OpalSIPIMMediaSession * mediaSession)
-{ 
-  PWaitAndSignal m(m_mutex);
-  PString callId(mediaSession->GetCallID());
-  m_imSessionMap.insert(IMSessionMapType::value_type((const char *)callId, mediaSession));
-  return true;
-}
-
-bool OpalSIPIMManager::EndSession(OpalSIPIMMediaSession * mediaSession)
-{ 
-  PWaitAndSignal m(m_mutex);
-  PString callId(mediaSession->GetCallID());
-  IMSessionMapType::iterator r = m_imSessionMap.find((const char *)callId);
-  if (r != m_imSessionMap.end())
-    m_imSessionMap.erase(r);
-  return true;
-}
 
 #endif // OPAL_HAS_SIPIM
