@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23954 $
+ * $Revision: 24547 $
  * $Author: rjongbloed $
- * $Date: 2010-01-20 17:11:47 -0600 (Wed, 20 Jan 2010) $
+ * $Date: 2010-06-30 01:36:31 -0500 (Wed, 30 Jun 2010) $
  */
 
 #include <ptlib.h>
@@ -598,7 +598,7 @@ void OpalConnection::OnConnected()
 void OpalConnection::OnEstablished()
 {
   PTRACE(3, "OpalCon\tOnEstablished " << *this);
-  StartMediaStreams();
+  ownerCall.StartMediaStreams();
   endpoint.OnEstablished(*this);
 }
 
@@ -753,7 +753,7 @@ void OpalConnection::StartMediaStreams()
   for (OpalMediaStreamPtr mediaStream = mediaStreams; mediaStream != NULL; ++mediaStream)
     mediaStream->Start();
 
-  PTRACE(3, "OpalCon\tMedia stream threads started.");
+  PTRACE(3, "OpalCon\tMedia stream threads started for " << *this);
 }
 
 
@@ -1052,6 +1052,27 @@ PBoolean OpalConnection::CreateVideoOutputDevice(const OpalMediaFormat & mediaFo
   return endpoint.CreateVideoOutputDevice(*this, mediaFormat, preview, device, autoDelete);
 }
 
+
+bool OpalConnection::SendVideoUpdatePicture(unsigned sessionID, int firstGOB, int firstMB, int numBlocks) const
+{
+  PSafeLockReadWrite safeLock(*this);
+  if (!safeLock.IsLocked())
+    return false;
+
+  OpalMediaStreamPtr stream = sessionID != 0 ? GetMediaStream(sessionID, false)
+                                             : GetMediaStream(OpalMediaType::Video(), false);
+  if (stream == NULL) {
+    PTRACE(3, "OpalCon\tNo video stream do video update picture in connection " << *this);
+    return false;
+  }
+  
+  OpalVideoUpdatePicture updatePictureCommand(firstGOB, firstMB, numBlocks);
+  stream->ExecuteCommand(updatePictureCommand);
+  PTRACE(3, "OpalCon\tUpdate video picture (I-frame) requested in video stream " << *stream);
+
+  return true;
+}
+
 #endif // OPAL_VIDEO
 
 
