@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23926 $
+ * $Revision: 24442 $
  * $Author: rjongbloed $
- * $Date: 2010-01-12 18:44:33 -0600 (Tue, 12 Jan 2010) $
+ * $Date: 2010-06-02 03:17:31 -0500 (Wed, 02 Jun 2010) $
  */
 
 #include <ptlib.h>
@@ -639,4 +639,24 @@ bool OpalEndPoint::TransmitExternalIM(OpalConnection & conn, const OpalMediaForm
 
 #endif
 
+
+bool OpalIsE164(const PString & number, bool strict)
+{
+  if (number.IsEmpty())
+    return false;
+
+  PINDEX offset;
+
+  if (strict || number[0] != '+')
+    offset = 0;
+  else {
+    if (number.GetLength() < 2)
+      return false;
+    offset = 1;
+  }
+
+  return number.FindSpan("1234567890*#", offset) == P_MAX_INDEX;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
