@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23874 $
+ * $Revision: 24132 $
  * $Author: rjongbloed $
- * $Date: 2009-12-14 03:45:30 +0000 (Mon, 14 Dec 2009) $
+ * $Date: 2010-03-19 03:49:38 -0500 (Fri, 19 Mar 2010) $
  */
 
 #include <ptlib.h>
@@ -42,16 +42,17 @@
 #include <opal/call.h>
 #include <opal/patch.h>
 #include <opal/mediastrm.h>
-
-#if OPAL_VIDEO
+#include <codec/g711codec.h>
 #include <codec/vidcodec.h>
-#endif
+#include <codec/rfc4175.h>
+#include <codec/opalpluginmgr.h>
 
 #if OPAL_HAS_H224
 #include <h224/h224.h>
 #endif
 
 #include <ptclib/random.h>
+#include <ptclib/url.h>
 
 #include "../../version.h"
 
@@ -68,12 +69,24 @@ static const char * const DefaultMediaFormatOrder[] = {
   OPAL_G728,
   OPAL_G711_ULAW_64K,
   OPAL_G711_ALAW_64K,
-  OPAL_H264,
+  OPAL_H264_MODE1,
+  OPAL_H264_MODE0,
   OPAL_MPEG4,
   OPAL_H263,
   OPAL_H261
 };
 
+// G.711 is *always* available
+// Yes, it would make more sense for this to be in g711codec.cxx, but on 
+// Linux it would not get loaded due to static initialisation optimisation
+OPAL_REGISTER_G711();
+
+// Same deal for RC4175 video
+#if OPAL_RFC4175
+OPAL_REGISTER_RFC4175();
+#endif
+
+
 #define new PNEW
 
 
@@ -153,6 +166,7 @@ OpalManager::OpalManager()
   , defaultDisplayName(defaultUserName)
   , rtpIpTypeofService(0xb8)  // New DiffServ value for Expidited Forwarding as per RFC3246
   , rtpPayloadSizeMax(576-20-16-12) // Max safe MTU size (576 bytes as per RFC879) minus IP, UDP an RTP headers
+  , rtpPacketSizeMax(2048)
   , minAudioJitterDelay(50)  // milliseconds
   , maxAudioJitterDelay(250) // milliseconds
   , mediaFormatOrder(PARRAYSIZE(DefaultMediaFormatOrder), DefaultMediaFormatOrder)
@@ -166,8 +180,6 @@ OpalManager::OpalManager()
   , zrtpEnabled(false)
 #endif
 {
-  m_recordManager = new OpalWAVRecordManager();
-
   rtpIpPorts.current = rtpIpPorts.base = 5000;
   rtpIpPorts.max = 5999;
 
@@ -221,7 +233,6 @@ OpalManager::~OpalManager()
   delete garbageCollector;
 
   delete stun;
-  delete m_recordManager;
   delete interfaceMonitor;
 
   PTRACE(4, "OpalMan\tDeleted manager.");
@@ -249,6 +260,10 @@ void OpalManager::ShutDownEndpoints()
   // Clear any pending calls, set flag so no calls can be received before endpoints removed
   InternalClearAllCalls(OpalConnection::EndedByLocalUser, true, m_clearingAllCallsCount++ == 0);
 
+  // Remove (and unsubscribe) all the presentities
+  m_presentities.RemoveAll();
+  m_presentities.DeleteObjectsToBeRemoved();
+
   // Deregister the endpoints
   endpointsMutex.StartRead();
   for (PList<OpalEndPoint>::iterator ep = endpointList.begin(); ep != endpointList.end(); ++ep)
@@ -345,19 +360,34 @@ OpalEndPoint * OpalManager::FindEndPoint(const PString & prefix)
 
 
 PBoolean OpalManager::SetUpCall(const PString & partyA,
-                            const PString & partyB,
-                            PString & token,
-                            void * userData,
-                            unsigned int options,
-                            OpalConnection::StringOptions * stringOptions)
+                                const PString & partyB,
+                                      PString & token,
+                                         void * userData,
+                                   unsigned int options,
+                OpalConnection::StringOptions * stringOptions)
+{
+  token.MakeEmpty();
+
+  PSafePtr<OpalCall> call = SetUpCall(partyA, partyB, userData, options, stringOptions);
+  if (call == NULL)
+    return false;
+
+  token = call->GetToken();
+  return true;
+}
+
+
+PSafePtr<OpalCall> OpalManager::SetUpCall(const PString & partyA,
+                                          const PString & partyB,
+                                                   void * userData,
+                                             unsigned int options,
+                          OpalConnection::StringOptions * stringOptions)
 {
   PTRACE(3, "OpalMan\tSet up call from " << partyA << " to " << partyB);
 
   OpalCall * call = CreateCall(userData);
   if (call == NULL)
-    return false;
-
-  token = call->GetToken();
+    return NULL;
 
   call->SetPartyB(partyB);
 
@@ -365,18 +395,20 @@ PBoolean OpalManager::SetUpCall(const PString & partyA,
   // go through the routing engine via OnIncomingConnection. If we were the
   // B-Party then SetUpConnection() gets called in the context of the A-party
   // thread.
-  if (MakeConnection(*call, partyA, userData, options, stringOptions) && call->GetConnection(0)->SetUpConnection()) {
-    PTRACE(3, "OpalMan\tSetUpCall succeeded, call=" << *call);
-    return true;
+  PSafePtr<OpalConnection> connection = MakeConnection(*call, partyA, userData, options, stringOptions);
+  if (connection != NULL && connection->SetUpConnection()) {
+    PTRACE(4, "OpalMan\tSetUpCall succeeded, call=" << *call);
+    return call;
   }
 
-  PSafePtr<OpalConnection> connection = call->GetConnection(0);
-  OpalConnection::CallEndReason endReason = connection != NULL ? connection->GetCallEndReason() : OpalConnection::NumCallEndReasons;
-  call->Clear(endReason != OpalConnection::NumCallEndReasons ? endReason : OpalConnection::EndedByTemporaryFailure);
+  PTRACE_IF(2, connection == NULL, "OpalMan\tCould not create connection for \"" << partyA << '"');
 
-  token.MakeEmpty();
+  OpalConnection::CallEndReason endReason = endReason = call->GetCallEndReason();
+  if (endReason == OpalConnection::NumCallEndReasons)
+    endReason = OpalConnection::EndedByTemporaryFailure;
+  call->Clear(endReason);
 
-  return false;
+  return NULL;
 }
 
 
@@ -482,14 +514,9 @@ OpalCall * OpalManager::InternalCreateCall()
 }
 
 
-OpalCall * OpalManager::CreateCall()
-{
-  return new OpalCall(*this);
-}
-
 OpalCall * OpalManager::CreateCall(void * /*userData*/)
 {
-  return CreateCall();
+  return new OpalCall(*this);
 }
 
 
@@ -499,21 +526,21 @@ void OpalManager::DestroyCall(OpalCall * call)
 }
 
 
-PString OpalManager::GetNextCallToken()
+PString OpalManager::GetNextToken(char prefix)
 {
-  return psprintf("%c%08x%u", PRandom::Number('a', 'z'), PRandom::Number(), ++lastCallTokenID);
+  return psprintf("%c%08x%u", prefix, PRandom::Number(), ++lastCallTokenID);
 }
 
-PBoolean OpalManager::MakeConnection(OpalCall & call,
-                                     const PString & remoteParty,
-                                     void * userData,
-                                     unsigned int options,
-                                     OpalConnection::StringOptions * stringOptions)
+PSafePtr<OpalConnection> OpalManager::MakeConnection(OpalCall & call,
+                                                const PString & remoteParty,
+                                                         void * userData,
+                                                   unsigned int options,
+                                OpalConnection::StringOptions * stringOptions)
 {
   PTRACE(3, "OpalMan\tSet up connection to \"" << remoteParty << '"');
 
   if (remoteParty.IsEmpty())
-    return false;
+    return NULL;
 
   PCaselessString epname = remoteParty.Left(remoteParty.Find(':'));
 
@@ -527,26 +554,13 @@ PBoolean OpalManager::MakeConnection(OpalCall & call,
   else
     ep = FindEndPoint(epname);
 
-  if (ep != NULL) {
-    if (ep->MakeConnection(call, remoteParty, userData, options, stringOptions))
-      return true;
-    PTRACE(1, "OpalMan\tCould not use endpoint for protocol \"" << epname << '"');
-  }
-  else {
-    PTRACE(1, "OpalMan\tCould not find endpoint to handle protocol \"" << epname << '"');
-  }
-  return false;
+  if (ep != NULL)
+    return ep->MakeConnection(call, remoteParty, userData, options, stringOptions);
+
+  PTRACE(1, "OpalMan\tCould not find endpoint to handle protocol \"" << epname << '"');
+  return NULL;
 }
 
-PBoolean OpalManager::OnIncomingConnection(OpalConnection & /*connection*/)
-{
-  return true;
-}
-
-PBoolean OpalManager::OnIncomingConnection(OpalConnection & /*connection*/, unsigned /*options*/)
-{
-  return true;
-}
 
 PBoolean OpalManager::OnIncomingConnection(OpalConnection & connection, unsigned options, OpalConnection::StringOptions * stringOptions)
 {
@@ -554,12 +568,6 @@ PBoolean OpalManager::OnIncomingConnection(OpalConnection & connection, unsigned
 
   connection.OnApplyStringOptions();
 
-  if (!OnIncomingConnection(connection))
-    return false;
-
-  if (!OnIncomingConnection(connection, options))
-    return false;
-
   // See if we already have a B-Party in the call. If not, make one.
   if (connection.GetOtherPartyConnection() != NULL)
     return true;
@@ -602,7 +610,7 @@ bool OpalManager::OnRouteConnection(PStringSet & routesTried,
     if (route.IsEmpty()) {
       // Check for if B-Party is an explicit address
       if (FindEndPoint(b_party.Left(b_party.Find(':'))) != NULL)
-        return MakeConnection(call, b_party, NULL, options, stringOptions);
+        return MakeConnection(call, b_party, NULL, options, stringOptions) != NULL;
 
       PTRACE(3, "OpalMan\tCould not route a=\"" << a_party << "\", b=\"" << b_party << ", call=" << call);
       return false;
@@ -614,7 +622,7 @@ bool OpalManager::OnRouteConnection(PStringSet & routesTried,
     routesTried += route;
 
     // See if this route can be connected
-    if (MakeConnection(call, route, NULL, options, stringOptions))
+    if (MakeConnection(call, route, NULL, options, stringOptions) != NULL)
       return true;
 
     // Recursively call with translated route
@@ -631,6 +639,7 @@ void OpalManager::OnProceeding(OpalConnection & connection)
   connection.GetCall().OnProceeding(connection);
 }
 
+
 void OpalManager::OnAlerting(OpalConnection & connection)
 {
   PTRACE(3, "OpalMan\tOnAlerting " << connection);
@@ -638,15 +647,14 @@ void OpalManager::OnAlerting(OpalConnection & connection)
   connection.GetCall().OnAlerting(connection);
 }
 
-OpalConnection::AnswerCallResponse
-       OpalManager::OnAnswerCall(OpalConnection & connection,
-                                  const PString & caller)
-{
-  PTRACE(3, "OpalMan\tOnAnswerCall " << connection);
 
+OpalConnection::AnswerCallResponse OpalManager::OnAnswerCall(OpalConnection & connection,
+                                                             const PString & caller)
+{
   return connection.GetCall().OnAnswerCall(connection, caller);
 }
 
+
 void OpalManager::OnConnected(OpalConnection & connection)
 {
   PTRACE(3, "OpalMan\tOnConnected " << connection);
@@ -693,11 +701,64 @@ PBoolean OpalManager::OnForwarded(OpalConnection & PTRACE_PARAM(connection),
 }
 
 
-void OpalManager::AdjustMediaFormats(const OpalConnection & /*connection*/,
+OpalMediaFormatList OpalManager::GetCommonMediaFormats(bool transportable, bool pcmAudio) const
+{
+  OpalMediaFormatList formats;
+
+  if (transportable) {
+    OpalMediaFormatList allFormats = OpalMediaFormat::GetAllRegisteredMediaFormats();
+    for (OpalMediaFormatList::iterator iter = allFormats.begin(); iter != allFormats.end(); ++iter) {
+      if (iter->IsTransportable())
+        formats += *iter;
+    }
+  }
+
+  if (pcmAudio) {
+    // Sound cards can only do 16 bit PCM, but at various sample rates
+    // The following will be in order of preference, so lets do wideband first
+    formats += OpalPCM16_48KHZ;
+    formats += OpalPCM16_32KHZ;
+    formats += OpalPCM16_16KHZ;
+    formats += OpalPCM16;
+    formats += OpalRFC2833;
+#if OPAL_T38_CAPABILITY
+    formats += OpalCiscoNSE;
+#endif
+  }
+
+#if OPAL_VIDEO
+  if (!videoInputDevice.deviceName.IsEmpty())
+    formats += OpalYUV420P;
+#endif
+
+#if OPAL_HAS_MSRP
+  formats += OpalMSRP;
+#endif
+
+#if OPAL_HAS_SIPIM
+  formats += OpalSIPIM;
+#endif
+
+#if OPAL_HAS_RFC4103
+  formats += OpalT140;
+#endif
+
+#if OPAL_HAS_H224
+  formats += OpalH224AnnexQ;
+  formats += OpalH224Tunnelled;
+#endif
+
+  return formats;
+}
+
+
+void OpalManager::AdjustMediaFormats(bool local,
+                                     const OpalConnection & /*connection*/,
                                      OpalMediaFormatList & mediaFormats) const
 {
   mediaFormats.Remove(mediaFormatMask);
-  mediaFormats.Reorder(mediaFormatOrder);
+  if (local)
+    mediaFormats.Reorder(mediaFormatOrder);
 }
 
 
@@ -720,30 +781,117 @@ PBoolean OpalManager::OnOpenMediaStream(OpalConnection & PTRACE_PARAM(connection
 }
 
 
+RTP_UDP * OpalManager::CreateRTPSession (const RTP_Session::Params & params)
+{
+  return new RTP_UDP(params);
+}
+
+
 void OpalManager::OnRTPStatistics(const OpalConnection & connection, const RTP_Session & session)
 {
   connection.GetCall().OnRTPStatistics(connection, session);
 }
 
 
+bool OpalManager::OnLocalRTP(OpalConnection & PTRACE_PARAM(connection1),
+                             OpalConnection & PTRACE_PARAM(connection2),
+                             unsigned         PTRACE_PARAM(sessionID),
+                             bool             PTRACE_PARAM(started)) const
+{
+  PTRACE(3, "OpalMan\tOnLocalRTP(" << connection1 << ',' << connection2 << ',' << sessionID << ',' << started);
+  return false;
+}
+
+
+static bool BypassOne(OpalMediaStreamPtr source,
+                      OpalMediaStreamPtr sink,
+                      bool bypass)
+{
+  OpalMediaPatch * sourcePatch = source != NULL ? source->GetPatch() : NULL;
+  OpalMediaPatch * sinkPatch = sink != NULL ? sink->GetPatch() : NULL;
+  if (sourcePatch == NULL || sinkPatch == NULL) {
+    PTRACE(2, "OpalMan\tSetMediaBypass could not complete as one patch does not exist");
+    return false;
+  }
+
+  if (source->GetMediaFormat() != sink->GetMediaFormat()) {
+    PTRACE(3, "OpalMan\tSetMediaBypass could not complete as different formats: "
+           << source->GetMediaFormat() << "!=" << sink->GetMediaFormat());
+    return false;
+  }
+
+  // Note SetBypassPatch() will do PTRACE() on status.
+  return sourcePatch->SetBypassPatch(bypass ? sinkPatch : NULL);
+}
+
+
+bool OpalManager::SetMediaBypass(OpalConnection & connection1,
+                                 OpalConnection & connection2,
+                                 bool bypass,
+                                 unsigned sessionID)
+{
+  bool gotOne = false;
+
+  if (sessionID != 0) {
+    // Do not use || as McCarthy will not execute the second bypass
+    if (BypassOne(connection1.GetMediaStream(sessionID, true), connection2.GetMediaStream(sessionID, false), bypass))
+      gotOne = true;
+    if (BypassOne(connection2.GetMediaStream(sessionID, true), connection1.GetMediaStream(sessionID, false), bypass))
+      gotOne = true;
+  }
+  else {
+    OpalMediaStreamPtr stream;
+    while ((stream = connection1.GetMediaStream(OpalMediaType(), true , stream)) != NULL) {
+      if (BypassOne(stream, connection2.GetMediaStream(stream->GetSessionID(), false), bypass))
+        gotOne = true;
+    }
+    while ((stream = connection2.GetMediaStream(OpalMediaType(), true, stream)) != NULL) {
+      if (BypassOne(stream, connection1.GetMediaStream(stream->GetSessionID(), false), bypass))
+        gotOne = true;
+    }
+  }
+
+  return gotOne;
+}
+
+
+bool OpalManager::SetMediaBypass(const PString & token1,
+                                 const PString & token2,
+                                 bool bypass,
+                                 unsigned sessionID,
+                                 bool network)
+{
+  PSafePtr<OpalCall> call1 = FindCallWithLock(token1);
+  PSafePtr<OpalCall> call2 = FindCallWithLock(token2);
+
+  if (call1 == NULL || call2 == NULL) {
+    PTRACE(2, "OpalMan\tSetMediaBypass could not complete as one call does not exist");
+    return false;
+  }
+
+  PSafePtr<OpalConnection> connection1 = call1->GetConnection(0, PSafeReadOnly);
+  while (connection1 != NULL && connection1->IsNetworkConnection() == network)
+    ++connection1;
+
+  PSafePtr<OpalConnection> connection2 = call2->GetConnection(0, PSafeReadOnly);
+  while (connection2 != NULL && connection2->IsNetworkConnection() == network)
+    ++connection2;
+
+  if (connection1 == NULL || connection2 == NULL) {
+    PTRACE(2, "OpalMan\tSetMediaBypass could not complete as network connection not present in calls");
+    return false;
+  }
+
+  return OpalManager::SetMediaBypass(*connection1, *connection2, sessionID, bypass);
+}
+
+
 void OpalManager::OnClosedMediaStream(const OpalMediaStream & /*channel*/)
 {
 }
 
 #if OPAL_VIDEO
 
-void OpalManager::AddVideoMediaFormats(OpalMediaFormatList & mediaFormats,
-                                       const OpalConnection * /*connection*/) const
-{
-  if (videoInputDevice.deviceName.IsEmpty())
-      return;
-
-  mediaFormats += OpalYUV420P;
-  mediaFormats += OpalRGB32;
-  mediaFormats += OpalRGB24;
-}
-
-
 PBoolean OpalManager::CreateVideoInputDevice(const OpalConnection & /*connection*/,
                                          const OpalMediaFormat & mediaFormat,
                                          PVideoInputDevice * & device,
@@ -751,11 +899,7 @@ PBoolean OpalManager::CreateVideoInputDevice(const OpalConnection & /*connection
 {
   // Make copy so we can adjust the size
   PVideoDevice::OpenArgs args = videoInputDevice;
-  args.width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption(), PVideoFrameInfo::QCIFWidth);
-  args.height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption(), PVideoFrameInfo::QCIFHeight);
-  unsigned maxRate = mediaFormat.GetClockRate()/mediaFormat.GetFrameTime();
-  if (args.rate > maxRate)
-    args.rate = maxRate;
+  mediaFormat.AdjustVideoArgs(args);
 
   autoDelete = true;
   device = PVideoInputDevice::CreateOpenedDevice(args, false);
@@ -779,9 +923,7 @@ PBoolean OpalManager::CreateVideoOutputDevice(const OpalConnection & connection,
 
   // Make copy so we can adjust the size
   PVideoDevice::OpenArgs args = preview ? videoPreviewDevice : videoOutputDevice;
-  args.width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption(), PVideoFrameInfo::QCIFWidth);
-  args.height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption(), PVideoFrameInfo::QCIFHeight);
-  args.rate = mediaFormat.GetClockRate()/mediaFormat.GetFrameTime();
+  mediaFormat.AdjustVideoArgs(args);
 
   PINDEX start = args.deviceName.Find("TITLE=\"");
   if (start != P_MAX_INDEX) {
@@ -814,9 +956,13 @@ void OpalManager::DestroyMediaPatch(OpalMediaPatch * patch)
 }
 
 
-PBoolean OpalManager::OnStartMediaPatch(const OpalMediaPatch & /*patch*/)
+void OpalManager::OnStartMediaPatch(OpalConnection & /*connection*/, OpalMediaPatch & /*patch*/)
+{
+}
+
+
+void OpalManager::OnStopMediaPatch(OpalConnection & /*connection*/, OpalMediaPatch & /*patch*/)
 {
-  return true;
 }
 
 
@@ -937,9 +1083,9 @@ PBoolean OpalManager::AddRouteEntry(const PString & spec)
   }
 
   PTRACE(4, "OpalMan\tAdded route \"" << *entry << '"');
-  routeTableMutex.Wait();
-  routeTable.Append(entry);
-  routeTableMutex.Signal();
+  m_routeMutex.Wait();
+  m_routeTable.Append(entry);
+  m_routeMutex.Signal();
   return true;
 }
 
@@ -948,15 +1094,15 @@ PBoolean OpalManager::SetRouteTable(const PStringArray & specs)
 {
   PBoolean ok = false;
 
-  routeTableMutex.Wait();
-  routeTable.RemoveAll();
+  m_routeMutex.Wait();
+  m_routeTable.RemoveAll();
 
   for (PINDEX i = 0; i < specs.GetSize(); i++) {
     if (AddRouteEntry(specs[i].Trim()))
       ok = true;
   }
 
-  routeTableMutex.Signal();
+  m_routeMutex.Signal();
 
   return ok;
 }
@@ -964,10 +1110,10 @@ PBoolean OpalManager::SetRouteTable(const PStringArray & specs)
 
 void OpalManager::SetRouteTable(const RouteTable & table)
 {
-  routeTableMutex.Wait();
-  routeTable = table;
-  routeTable.MakeUnique();
-  routeTableMutex.Signal();
+  m_routeMutex.Wait();
+  m_routeTable = table;
+  m_routeTable.MakeUnique();
+  m_routeMutex.Signal();
 }
 
 
@@ -987,9 +1133,9 @@ static void ReplaceNDU(PString & destination, const PString & subst)
 
 PString OpalManager::ApplyRouteTable(const PString & a_party, const PString & b_party, PINDEX & routeIndex)
 {
-  PWaitAndSignal mutex(routeTableMutex);
+  PWaitAndSignal mutex(m_routeMutex);
 
-  if (routeTable.IsEmpty())
+  if (m_routeTable.IsEmpty())
     return routeIndex++ == 0 ? b_party : PString::Empty();
 
   PString search = a_party + '\t' + b_party;
@@ -1017,10 +1163,11 @@ PString OpalManager::ApplyRouteTable(const PString & a_party, const PString & b_
    */
 
   PString destination;
-  while (routeIndex < routeTable.GetSize()) {
-    RouteEntry & entry = routeTable[routeIndex++];
+  while (routeIndex < m_routeTable.GetSize()) {
+    RouteEntry & entry = m_routeTable[routeIndex++];
     PINDEX pos;
     if (entry.regex.Execute(search, pos)) {
+      PTRACE(4, "OpalMan\tMatched regex \"" << entry.pattern << '"');
       if (entry.destination.NumCompare("label:") != EqualTo) {
         destination = entry.destination;
         break;
@@ -1030,6 +1177,9 @@ PString OpalManager::ApplyRouteTable(const PString & a_party, const PString & b_
       search = entry.destination;
       routeIndex = 0;
     }
+    else {
+      PTRACE(4, "OpalMan\tDid not match regex \"" << entry.pattern << '"');
+    }
   }
 
   // No route found
@@ -1071,16 +1221,22 @@ PString OpalManager::ApplyRouteTable(const PString & a_party, const PString & b_
   while ((pos = digits.FindOneOf("+-.()")) != P_MAX_INDEX)
     digits.Delete(pos, 1);
 
+  PString user = b_party(colon, at-1);
+
   destination.Replace("<da>", b_party, true);
-  destination.Replace("<db>",  b_party.Mid(colon), true);
+  destination.Replace("<db>", user, true);
 
   if (at != P_MAX_INDEX) {
-    destination.Replace("<du>", b_party(colon, at-1), true);
+    destination.Replace("<du>", user, true);
     ReplaceNDU(destination, b_party.Mid(at));
   }
+  else if (PIPSocket::IsLocalHost(user.Left(user.Find(':')))) {
+    destination.Replace("<du>", "", true);
+    ReplaceNDU(destination, user);
+  }
   else {
-    destination.Replace("<du>", digits, true);
-    ReplaceNDU(destination, b_party.Mid(nonDigitPos));
+    destination.Replace("<du>", user, true);
+    ReplaceNDU(destination, "");
   }
 
   destination.Replace("<dn>", digits, true);
@@ -1263,7 +1419,11 @@ PBoolean OpalManager::TranslateIPAddress(PIPSocket::Address & localAddress,
     return true;
   }
 
-  if (stun != NULL && stun->GetNatType() != PSTUNClient::BlockedNat)
+  PIPSocket::Address stunInterface;
+  if (stun != NULL &&
+      stun->GetNatType() != PSTUNClient::BlockedNat &&
+      stun->GetInterfaceAddress(stunInterface) &&
+      stunInterface == localAddress)
     return stun->GetExternalAddress(localAddress); // Translate it!
 
   return false; // Have nothing to translate it to
@@ -1551,13 +1711,17 @@ void OpalManager::OnNewConnection(OpalConnection & /*conn*/)
 {
 }
 
-bool OpalManager::StartRecording(const PString & callToken, const PFilePath & fn, bool mono)
+#if OPAL_HAS_MIXER
+
+bool OpalManager::StartRecording(const PString & callToken,
+                                 const PFilePath & fn,
+                                 const OpalRecordManager::Options & options)
 {
   PSafePtr<OpalCall> call = activeCalls.FindWithLock(callToken, PSafeReadWrite);
   if (call == NULL)
     return false;
 
-  return call->StartRecording(fn, mono);
+  return call->StartRecording(fn, options);
 }
 
 
@@ -1578,32 +1742,118 @@ bool OpalManager::StopRecording(const PString & callToken)
   return true;
 }
 
+#endif
 
-void OpalManager::AddIMMediaFormats(OpalMediaFormatList & mediaFormats, const OpalConnection * /*connection*/) const
-{
-#if OPAL_HAS_MSRP
-  mediaFormats += OpalMSRP;
-#endif
-#if OPAL_HAS_SIPIM
-  mediaFormats += OpalSIPIM;
-#endif
-#if OPAL_HAS_RFC4103
-  mediaFormats += OpalT140;
-#endif
-#if OPAL_HAS_H224
-  mediaFormats += OpalH224AnnexQ;
-  mediaFormats += OpalH224Tunnelled;
-#endif
-}
 
-void OpalManager::OnApplyStringOptions(
-  OpalConnection & conn,
-  OpalConnection::StringOptions & stringOptions
-)
+void OpalManager::OnApplyStringOptions(OpalConnection & conn, OpalConnection::StringOptions & stringOptions)
 {
   conn.ApplyStringOptions(stringOptions);
 }
 
+
+PSafePtr<OpalPresentity> OpalManager::AddPresentity(const PString & presentity)
+{
+  OpalPresentity * newPresentity = OpalPresentity::Create(*this, presentity);
+  if (newPresentity == NULL)
+    return NULL;
+
+  m_presentities.SetAt(presentity, newPresentity);
+  return PSafePtr<OpalPresentity>(newPresentity, PSafeReadWrite);
+}
+
+
+PSafePtr<OpalPresentity> OpalManager::GetPresentity(const PString & presentity, PSafetyMode mode)
+{
+  return m_presentities.FindWithLock(presentity, mode);
+}
+
+
+PStringList OpalManager::GetPresentities() const
+{
+  PStringList presentities;
+
+  for (PSafePtr<OpalPresentity> presentity(m_presentities, PSafeReference); presentity != NULL; ++presentity)
+    presentities += presentity->GetAOR().AsString();
+
+  return presentities;
+}
+
+
+bool OpalManager::RemovePresentity(const PString & presentity)
+{
+  return m_presentities.RemoveAt(presentity);
+}
+
+
+PBoolean OpalManager::Message(const PString & to, const PString & body)
+{
+  PURL url(to);
+
+  for (PList<OpalEndPoint>::iterator it = endpointList.begin(); it != endpointList.end(); ++it) {
+    if (url.GetScheme() *= it->GetPrefixName())
+      return it->Message(to, body);
+  }
+
+  return false;
+}
+
+
+PBoolean OpalManager::Message(const PURL & to, const PString & type, const PString & body, PURL & from, PString & conversationId)
+{
+  for (PList<OpalEndPoint>::iterator it = endpointList.begin(); it != endpointList.end(); ++it) {
+    if (to.GetScheme() *= it->GetPrefixName())
+      return it->Message(to, type, body, from, conversationId);
+  }
+
+  return false;
+}
+
+void OpalManager::OnMessageReceived(
+  const PURL & /*from*/, 
+  const PString & /*fromName*/,
+  const PURL & /*to*/, 
+  const PString & /*type*/,
+  const PString & /*body*/,
+  const PString & /*conversationId*/
+)
+{
+}
+
+#if OPAL_HAS_IM
+
+bool OpalManager::TransmitExternalIM(OpalConnection & conn, const OpalMediaFormat & /*format*/, RTP_IMFrame & frame)
+{
+  PURL remotePartyURL, localPartyURL;
+  PString remotePartyName;
+
+  // get information about sender
+  PSafePtr<OpalConnection> otherParty = conn.GetOtherPartyConnectionAs<OpalConnection>();
+  if (otherParty != NULL) {
+    remotePartyURL  = otherParty->GetRemotePartyCallbackURL();
+    remotePartyName = otherParty->GetRemotePartyName();
+    localPartyURL   = otherParty->GetLocalPartyURL();
+  }
+
+  // get ID
+  PString id = conn.GetToken(); //format.GetOptionString("Path", GetIdentifier());
+
+  // pass information up the chain
+  T140String t140;
+  PString str;
+  frame.GetContent(t140);
+  t140.AsString(str);
+  OnMessageReceived(remotePartyURL, 
+                    remotePartyName, 
+                    localPartyURL, 
+                    frame.GetContentType(),
+                    str,
+                    id);
+
+  return true;
+}
+
+#endif
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalManager::InterfaceMonitor::InterfaceMonitor(OpalManager & manager)
@@ -1634,159 +1884,3 @@ bool OpalManager::GetZRTPEnabled() const
 
 
 /////////////////////////////////////////////////////////////////////////////
-
-OpalWAVRecordManager::Mixer_T::Mixer_T()
-  :	OpalAudioMixer(true)
-  ,	m_mono(false)
-  ,	m_started(false)
-{
-}
-
-bool OpalWAVRecordManager::Mixer_T::Open(const PFilePath & fn, bool mono)
-{
-  PWaitAndSignal mut(mutex);
-
-  if (m_started) {
-    PTRACE(2, "OPAL\tCannot open mixer after it has started.");
-    return false;
-  }
-
-  m_file.SetFormat(OpalWAVFile::fmt_PCM);
-  if (!m_file.Open(fn, PFile::ReadWrite, PFile::Create|PFile::Truncate)) {
-    PTRACE(2, "OPAL\tCould not open file \"" << fn << "\": " << m_file.GetErrorText());
-    return false;
-  }
-
-  m_mono = mono;
-  if (!mono)
-    m_file.SetChannels(2);
-
-  m_started = true;
-  PTRACE(4, "OPAL\t" << (mono ? "Mono" : "Stereo") << " mixer opened for file \"" << fn << '"');
-  return true;
-}
-
-bool OpalWAVRecordManager::Mixer_T::Close()
-{
-  RemoveAllStreams();
-
-  PWaitAndSignal mut(mutex);
-  m_started = false;
-  m_file.Close();
-
-  return true;
-}
-
-PBoolean OpalWAVRecordManager::Mixer_T::OnWriteAudio(const MixerFrame & mixerFrame)
-{
-  if (!m_file.IsOpen())
-    return false;
-
-  OpalAudioMixerStream::StreamFrame frame;
-  if (m_mono)
-    mixerFrame.GetMixedFrame(frame);
-  else
-    mixerFrame.GetStereoFrame(frame);
-  m_file.Write(frame.GetPointerAndLock(), frame.GetSize());
-  frame.Unlock();
-
-  return true;
-}
-
-
-//////////////////////////////////////////////////////////////////////////////
-
-OpalWAVRecordManager::OpalWAVRecordManager()
-{
-}
-
-
-OpalWAVRecordManager::~OpalWAVRecordManager()
-{
-  for (MixerMap_T::iterator mixer = m_mixers.begin(); mixer != m_mixers.end(); ++mixer)
-    delete mixer->second;
-}
-
-
-bool OpalWAVRecordManager::Open(const PString & callToken, const PFilePath & fn, bool mono)
-{
-  PWaitAndSignal mutex(m_mutex);
-
-  if (callToken.IsEmpty())
-    return false;
-
-  if (m_mixers.find(callToken) != m_mixers.end()) {
-    PTRACE(2, "OPAL\tAttempting to record for call " << callToken << " when already recording.");
-    return false;
-  }
-
-  Mixer_T * mixer = new Mixer_T;
-  if (mixer->Open(fn, mono)) {
-    m_mixers[callToken] = mixer;
-    PTRACE(4, "OPAL\tOpened recorder on call " << callToken);
-    return true;
-  }
-
-  delete mixer;
-  return false;
-}
-
-
-bool OpalWAVRecordManager::IsOpen(const PString & callToken) const
-{
-  PWaitAndSignal mutex(m_mutex);
-
-  MixerMap_T::const_iterator mixer = m_mixers.find(callToken);
-  if (mixer == m_mixers.end())
-    return false;
-
-  return mixer->second->IsOpen();
-}
-
-
-bool OpalWAVRecordManager::CloseStream(const PString & callToken, const std::string & streamId)
-{
-  PWaitAndSignal mutex(m_mutex);
-
-  MixerMap_T::iterator mixer = m_mixers.find(callToken);
-  if (mixer == m_mixers.end()) {
-    PTRACE(2, "OPAL\tAttempting to close stream on call " << callToken << " when not recording.");
-    return false;
-  }
-
-  mixer->second->RemoveStream(streamId);
-  PTRACE(4, "OPAL\tClosed stream " << streamId << " on call " << callToken);
-  return true;
-}
-
-
-bool OpalWAVRecordManager::Close(const PString & callToken)
-{
-  PWaitAndSignal mutex(m_mutex);
-
-  MixerMap_T::iterator mixer = m_mixers.find(callToken);
-  if (mixer == m_mixers.end()) {
-    PTRACE(2, "OPAL\tAttempting to close recording on call " << callToken << " when not recording.");
-    return false;
-  }
-
-  PTRACE(4, "OPAL\tClosed recorder on call " << callToken);
-  mixer->second->Close();
-  delete mixer->second;
-  m_mixers.erase(mixer);
-  return true;
-}
-
-
-bool OpalWAVRecordManager::WriteAudio(const PString & callToken, const std::string & strm, const RTP_DataFrame & rtp)
-{ 
-  PWaitAndSignal mutex(m_mutex);
-
-  MixerMap_T::iterator mixer = m_mixers.find(callToken);
-  if (mixer == m_mixers.end()) {
-    PTRACE(4, "OPAL\tAttempt to write to call " << callToken << " when not recording.");
-    return false;
-  }
-
-  return mixer->second->Write(strm, rtp);
-}
