@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 23028 $
+ * $Revision: 24362 $
  * $Author: rjongbloed $
- * $Date: 2009-07-02 10:39:41 +0000 (Thu, 02 Jul 2009) $
+ * $Date: 2010-05-20 03:24:51 -0500 (Thu, 20 May 2010) $
  */
 
 #include <ptlib.h>
@@ -49,7 +49,7 @@
 #include <rtp/rtp.h>
 #include <opal/transports.h>
 #include <opal/rtpconn.h>
-#include <opal/endpoint.h>
+#include <opal/rtpep.h>
 #include <opal/call.h>
 
 #define MAX_PAYLOAD_TYPE_MISMATCHES 10
@@ -73,6 +73,9 @@ OpalMediaStream::OpalMediaStream(OpalConnection & conn, const OpalMediaFormat &
   , marker(true)
   , mismatchedPayloadTypes(0)
   , mediaPatch(NULL)
+  , m_payloadType(mediaFormat.GetPayloadType())
+  , m_frameTime(mediaFormat.GetFrameTime())
+  , m_frameSize(mediaFormat.GetFrameSize())
 {
   connection.SafeReference();
   PTRACE(5, "Media\tCreated " << (isSource ? "Source" : "Sink") << ' ' << this);
@@ -114,13 +117,13 @@ PBoolean OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaForm
   if (mediaPatch != NULL && !fromPatch)
     return mediaPatch->UpdateMediaFormat(newMediaFormat);
 
-  if (mediaFormat != newMediaFormat)
-    return mediaFormat.Merge(newMediaFormat);
-
-  mediaFormat = newMediaFormat;
+  if (!mediaFormat.Update(newMediaFormat))
+    return false;
 
   PTRACE(4, "Media\tMedia format updated on " << *this);
-
+  m_payloadType = mediaFormat.GetPayloadType();
+  m_frameTime = mediaFormat.GetFrameTime();
+  m_frameSize = mediaFormat.GetFrameSize();
   return true;
 }
 
@@ -232,24 +235,24 @@ PBoolean OpalMediaStream::WritePackets(RTP_DataFrameList & packets)
 }
 
 
-inline static unsigned CalculateTimestamp(PINDEX size, const OpalMediaFormat & mediaFormat)
+void OpalMediaStream::IncrementTimestamp(PINDEX size)
 {
-  unsigned frameTime = mediaFormat.GetFrameTime();
-  PINDEX frameSize = mediaFormat.GetFrameSize();
-
-  if (frameSize == 0)
-    return frameTime;
-
-  unsigned frames = (size + frameSize - 1) / frameSize;
-  return frames*frameTime;
+  timestamp += m_frameTime * (m_frameSize != 0 ? ((size + m_frameSize - 1) / m_frameSize) : 1);
 }
 
 
 PBoolean OpalMediaStream::ReadPacket(RTP_DataFrame & packet)
 {
+  if (!isOpen)
+    return false;
+
   unsigned oldTimestamp = timestamp;
 
-  PAssert(defaultDataSize <= (packet.GetSize() - packet.GetHeaderSize()), "default data size too large for packet");
+  if (defaultDataSize < (packet.GetSize() - RTP_DataFrame::MinHeaderSize)) {
+    stringstream str;
+    str << "Media stream buffer " << (packet.GetSize() - RTP_DataFrame::MinHeaderSize) << " too small for media packet " << defaultDataSize;
+    PAssertAlways(str.str().c_str());
+  }
 
   PINDEX lastReadCount;
   if (!ReadData(packet.GetPayloadPtr(), defaultDataSize, lastReadCount))
@@ -258,9 +261,9 @@ PBoolean OpalMediaStream::ReadPacket(RTP_DataFrame & packet)
   // If the ReadData() function did not change the timestamp then use the default
   // method or fixed frame times and sizes.
   if (oldTimestamp == timestamp)
-    timestamp += CalculateTimestamp(lastReadCount, mediaFormat);
+    IncrementTimestamp(lastReadCount);
 
-  packet.SetPayloadType(mediaFormat.GetPayloadType());
+  packet.SetPayloadType(m_payloadType);
   packet.SetPayloadSize(lastReadCount);
   packet.SetTimestamp(oldTimestamp); // Beginning of frame
   packet.SetMarker(marker);
@@ -275,16 +278,19 @@ PBoolean OpalMediaStream::ReadPacket(RTP_DataFrame & packet)
 
 PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
 {
+  if (!isOpen)
+    return false;
+
   if (paused)
     packet.SetPayloadSize(0);
 
   timestamp = packet.GetTimestamp();
 
   int size = packet.GetPayloadSize();
-  if (size > 0 && mediaFormat.IsTransportable()) {
-    if (packet.GetPayloadType() == mediaFormat.GetPayloadType()) {
+  if (size > 0 && m_payloadType < RTP_DataFrame::MaxPayloadType) {
+    if (packet.GetPayloadType() == m_payloadType) {
       PTRACE_IF(2, mismatchedPayloadTypes > 0,
-                "H323RTP\tPayload type matched again " << mediaFormat.GetPayloadType());
+                "H323RTP\tPayload type matched again " << m_payloadType);
       mismatchedPayloadTypes = 0;
     }
     else {
@@ -292,7 +298,7 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
       if (mismatchedPayloadTypes < MAX_PAYLOAD_TYPE_MISMATCHES) {
         PTRACE(2, "Media\tRTP data with mismatched payload type,"
                   " is " << packet.GetPayloadType() << 
-                  " expected " << mediaFormat.GetPayloadType() <<
+                  " expected " << m_payloadType <<
                   ", ignoring packet.");
         size = 0;
       }
@@ -300,14 +306,14 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
         PTRACE_IF(2, mismatchedPayloadTypes == MAX_PAYLOAD_TYPE_MISMATCHES,
                   "Media\tRTP data with consecutive mismatched payload types,"
                   " is " << packet.GetPayloadType() << 
-                  " expected " << mediaFormat.GetPayloadType() <<
+                  " expected " << m_payloadType <<
                   ", ignoring payload type from now on.");
       }
     }
   }
 
   if (size == 0) {
-    timestamp += CalculateTimestamp(1, mediaFormat);
+    IncrementTimestamp(1);
     packet.SetTimestamp(timestamp);
     PINDEX dummy;
     return WriteData(NULL, 0, dummy);
@@ -328,7 +334,7 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
     // If the Write() function did not change the timestamp then use the default
     // method of fixed frame times and sizes.
     if (oldTimestamp == timestamp)
-      timestamp += CalculateTimestamp(size, mediaFormat);
+      IncrementTimestamp(size);
 
     size -= written;
     ptr += written;
@@ -375,7 +381,7 @@ PBoolean OpalMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDEX &
   written = length;
   RTP_DataFrame packet(length);
   memcpy(packet.GetPayloadPtr(), buffer, length);
-  packet.SetPayloadType(mediaFormat.GetPayloadType());
+  packet.SetPayloadType(m_payloadType);
   packet.SetTimestamp(timestamp);
   packet.SetMarker(marker);
   return WritePacket(packet);
@@ -384,23 +390,22 @@ PBoolean OpalMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDEX &
 
 PBoolean OpalMediaStream::PushPacket(RTP_DataFrame & packet)
 {
-  if(mediaPatch == NULL) {
-    return false;
+  OpalMediaPatch * patch = NULL;
+  if (LockReadOnly()) {
+    patch = mediaPatch;
+    UnlockReadOnly();
   }
-	
-  return mediaPatch->PushFrame(packet);
+
+  // OpalMediaPatch::PushFrame() might block, do outside of mutex
+  return patch != NULL && patch->PushFrame(packet);
 }
 
 
-PBoolean OpalMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameSize)
+PBoolean OpalMediaStream::SetDataSize(PINDEX dataSize, PINDEX /*frameTime*/)
 {
   if (dataSize <= 0)
     return false;
 
-  // Quantise data size
-  if (frameSize > 1)
-    dataSize = (dataSize+frameSize-1)/frameSize * frameSize;
-
   PTRACE_IF(4, defaultDataSize != dataSize, "Media\tSet data size from " << defaultDataSize << " to " << dataSize);
   defaultDataSize = dataSize;
   return true;
@@ -433,11 +438,38 @@ void OpalMediaStream::SetPaused(bool p)
 
 PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
 {
+#if PTRACING
+  if (PTrace::CanTrace(4) && (patch != NULL || mediaPatch != NULL)) {
+    ostream & trace = PTrace::Begin(4, __FILE__, __LINE__);
+    if (patch == NULL)
+      trace << "Removing patch " << *mediaPatch;
+    else if (mediaPatch == NULL)
+      trace << "Adding patch " << *patch;
+    else
+      trace << "Overwriting patch " << *mediaPatch << " with " << *patch;
+    trace << " on stream " << *this << PTrace::End;
+  }
+#endif
+
   PSafeLockReadWrite safeLock(*this);
   if (!safeLock.IsLocked())
     return false;
 
+  if (mediaPatch == NULL) {
+    mediaPatch = patch;
+    return true;
+  }
+
+  OpalMediaPatch * oldPatch = mediaPatch;
   mediaPatch = patch;
+
+  if (IsSink())
+    oldPatch->RemoveSink(this);
+  else {
+    oldPatch->Close();
+    connection.GetEndPoint().GetManager().DestroyMediaPatch(oldPatch);
+  }
+
   return true;
 }
 
@@ -445,10 +477,7 @@ PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
 void OpalMediaStream::AddFilter(const PNotifier & Filter, const OpalMediaFormat & Stage)
 {
   PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
-    return;
-
-  if (mediaPatch != NULL)
+  if (safeLock.IsLocked() && mediaPatch != NULL)
     mediaPatch->AddFilter(Filter, Stage);
 }
 
@@ -456,12 +485,7 @@ void OpalMediaStream::AddFilter(const PNotifier & Filter, const OpalMediaFormat
 PBoolean OpalMediaStream::RemoveFilter(const PNotifier & Filter, const OpalMediaFormat & Stage)
 {
   PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
-
-  if (mediaPatch != NULL) return mediaPatch->RemoveFilter(Filter, Stage);
-
-  return false;
+  return safeLock.IsLocked() && mediaPatch != NULL && mediaPatch->RemoveFilter(Filter, Stage);
 }
 
 
@@ -469,10 +493,7 @@ PBoolean OpalMediaStream::RemoveFilter(const PNotifier & Filter, const OpalMedia
 void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
 {
   PSafeLockReadOnly safeLock(*this);
-  if (!safeLock.IsLocked())
-    return;
-
-  if (mediaPatch != NULL && !fromPatch)
+  if (safeLock.IsLocked() && mediaPatch != NULL && !fromPatch)
     mediaPatch->GetStatistics(statistics, IsSink());
 }
 #endif
@@ -484,14 +505,14 @@ void OpalMediaStream::RemovePatch(OpalMediaPatch * /*patch*/ )
 }
 
 
-void OpalMediaStream::OnPatchStart() 
+void OpalMediaStream::OnStartMediaPatch() 
 { 
-  connection.OnMediaPatchStart(sessionID, isSource);
+  connection.OnStartMediaPatch(*mediaPatch);
 }
 
-void OpalMediaStream::OnPatchStop() 
+void OpalMediaStream::OnStopMediaPatch(OpalMediaPatch & patch)
 { 
-  connection.OnMediaPatchStop(sessionID, isSource);
+  connection.OnStopMediaPatch(patch);
 }
 
 
@@ -534,6 +555,21 @@ OpalNullMediaStream::OpalNullMediaStream(OpalConnection & conn,
   : OpalMediaStream(conn, mediaFormat, sessionID, isSource)
   , OpalMediaStreamPacing(mediaFormat)
   , m_isSynchronous(isSyncronous)
+  , m_requiresPatchThread(isSyncronous)
+{
+}
+
+
+OpalNullMediaStream::OpalNullMediaStream(OpalConnection & conn,
+                                         const OpalMediaFormat & mediaFormat,
+                                         unsigned sessionID,
+                                         bool isSource,
+                                         bool usePacingDelay,
+                                         bool requiresPatchThread)
+  : OpalMediaStream(conn, mediaFormat, sessionID, isSource)
+  , OpalMediaStreamPacing(mediaFormat)
+  , m_isSynchronous(usePacingDelay)
+  , m_requiresPatchThread(requiresPatchThread)
 {
 }
 
@@ -546,7 +582,8 @@ PBoolean OpalNullMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & leng
   memset(buffer, 0, size);
   length = size;
 
-  Pace(true, size, marker);
+  if (m_isSynchronous)
+    Pace(true, size, marker);
   return true;
 }
 
@@ -558,14 +595,15 @@ PBoolean OpalNullMediaStream::WriteData(const BYTE * /*buffer*/, PINDEX length,
 
   written = length != 0 ? length : defaultDataSize;
 
-  Pace(false, written, marker);
+  if (m_isSynchronous)
+    Pace(false, written, marker);
   return true;
 }
 
 
 PBoolean OpalNullMediaStream::RequiresPatchThread() const
 {
-  return m_isSynchronous;
+  return m_requiresPatchThread;
 }
 
 
@@ -595,7 +633,7 @@ OpalRTPMediaStream::OpalRTPMediaStream(OpalRTPConnection & conn,
      practical UDP packet size. This means we have a buffer that can accept
      whatever the RTP sender throws at us. For sink, we set it to the
      maximum size based on MTU (or other criteria). */
-  defaultDataSize = isSource ? 2048 : conn.GetMaxRtpPayloadSize();
+  defaultDataSize = isSource ? conn.GetEndPoint().GetManager().GetMaxRtpPacketSize() : conn.GetMaxRtpPayloadSize();
 }
 
 
@@ -631,6 +669,27 @@ PBoolean OpalRTPMediaStream::Close()
 }
 
 
+void OpalRTPMediaStream::SetPaused(bool pause)
+{
+  if (paused == pause)
+    return;
+
+  OpalMediaStream::SetPaused(pause);
+
+  // If coming out of pause, reopen the RTP session, even though it is probably
+  // still open, to make sure any pending error/statistic conditions are reset.
+  if (!paused)
+    rtpSession.Reopen(IsSource());
+
+  if (IsSource()) {
+    if (pause)
+      rtpSession.SetJitterBufferSize(0, 0);
+    else
+      EnableJitterBuffer();
+  }
+}
+
+
 PBoolean OpalRTPMediaStream::ReadPacket(RTP_DataFrame & packet)
 {
   if (IsSink()) {
@@ -638,8 +697,12 @@ PBoolean OpalRTPMediaStream::ReadPacket(RTP_DataFrame & packet)
     return false;
   }
 
-  if (!rtpSession.ReadBufferedData(packet))
-    return false;
+  if (paused)
+    packet.SetPayloadSize(0);
+  else {
+    if (!rtpSession.ReadBufferedData(packet))
+      return false;
+  }
 
   timestamp = packet.GetTimestamp();
   return true;
@@ -648,9 +711,6 @@ PBoolean OpalRTPMediaStream::ReadPacket(RTP_DataFrame & packet)
 
 PBoolean OpalRTPMediaStream::WritePacket(RTP_DataFrame & packet)
 {
-  if (paused)
-    packet.SetPayloadSize(0);
-  
   if (IsSource()) {
     PTRACE(1, "Media\tTried to write to source media stream");
     return false;
@@ -658,14 +718,15 @@ PBoolean OpalRTPMediaStream::WritePacket(RTP_DataFrame & packet)
 
   timestamp = packet.GetTimestamp();
 
-  if (packet.GetPayloadSize() == 0)
+  if (paused || packet.GetPayloadSize() == 0)
     return true;
 
+  packet.SetPayloadType(m_payloadType);
   return rtpSession.WriteData(packet);
 }
 
 
-PBoolean OpalRTPMediaStream::SetDataSize(PINDEX PTRACE_PARAM(dataSize), PINDEX /*frameSize*/)
+PBoolean OpalRTPMediaStream::SetDataSize(PINDEX PTRACE_PARAM(dataSize), PINDEX /*frameTime*/)
 {
   PTRACE(3, "Media\tRTP data size cannot be changed to " << dataSize << ", fixed at " << defaultDataSize);
   return true;
@@ -674,16 +735,41 @@ PBoolean OpalRTPMediaStream::SetDataSize(PINDEX PTRACE_PARAM(dataSize), PINDEX /
 
 PBoolean OpalRTPMediaStream::IsSynchronous() const
 {
-  return IsSource() && !mediaFormat.NeedsJitterBuffer();
+  return false;
+}
+
+
+PBoolean OpalRTPMediaStream::RequiresPatchThread() const
+{
+  return !dynamic_cast<OpalRTPEndPoint &>(connection.GetEndPoint()).CheckForLocalRTP(*this);
 }
 
 
 void OpalRTPMediaStream::EnableJitterBuffer() const
 {
-  if (mediaFormat.NeedsJitterBuffer())
-    rtpSession.SetJitterBufferSize(minAudioJitterDelay*mediaFormat.GetTimeUnits(),
-                                   maxAudioJitterDelay*mediaFormat.GetTimeUnits(),
-				                       mediaFormat.GetTimeUnits());
+  unsigned minJitter, maxJitter;
+  if (mediaFormat.NeedsJitterBuffer()) {
+    minJitter = minAudioJitterDelay*mediaFormat.GetTimeUnits();
+    maxJitter = maxAudioJitterDelay*mediaFormat.GetTimeUnits();
+  }
+  else
+    minJitter = maxJitter = 0;
+
+  rtpSession.SetJitterBufferSize(minJitter, maxJitter,
+                                 mediaFormat.GetTimeUnits(),
+                                 connection.GetEndPoint().GetManager().GetMaxRtpPacketSize());
+}
+
+
+PBoolean OpalRTPMediaStream::SetPatch(OpalMediaPatch * patch)
+{
+  if (!isOpen || IsSink())
+    return OpalMediaStream::SetPatch(patch);
+
+  rtpSession.Close(true);
+  bool ok = OpalMediaStream::SetPatch(patch);
+  rtpSession.Reopen(true);
+  return ok;
 }
 
 
@@ -739,13 +825,21 @@ PBoolean OpalRawMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & lengt
   if (!IsOpen() || m_channel == NULL)
     return false;
 
-  if (!m_channel->Read(buffer, size))
-    return false;
+  if (buffer == NULL || size == 0)
+    return m_channel->Read(buffer, size);
 
-  length = m_channel->GetLastReadCount();
+  while (size > 0) {
+    if (!m_channel->Read(buffer, size))
+      return false;
+
+    PINDEX lastReadCount = m_channel->GetLastReadCount();
+    CollectAverage(buffer, lastReadCount);
+
+    buffer += lastReadCount;
+    length += lastReadCount;
+    size -= lastReadCount;
+  }
 
-  if (buffer != NULL)
-    CollectAverage(buffer, length);
 
   return true;
 }
@@ -813,7 +907,7 @@ bool OpalRawMediaStream::SetChannel(PChannel * chan, bool autoDelete)
   m_channel = chan;
   m_autoDelete = autoDelete;
 
-  SetDataSize(GetDataSize(), sizeof(short));
+  SetDataSize(GetDataSize(), 1);
 
   m_channelMutex.Signal();
 
@@ -929,11 +1023,13 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
                                            unsigned sessionID,
                                            PBoolean isSource,
                                            PINDEX buffers,
+                                           unsigned bufferTime,
                                            PSoundChannel * channel,
                                            PBoolean autoDel)
   : OpalRawMediaStream(conn, mediaFormat, sessionID, isSource, channel, autoDel)
+  , m_soundChannelBuffers(buffers)
+  , m_soundChannelBufferTime(bufferTime)
 {
-  soundChannelBuffers = buffers;
 }
 
 
@@ -942,6 +1038,7 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
                                            unsigned sessionID,
                                            PBoolean isSource,
                                            PINDEX buffers,
+                                           unsigned bufferTime,
                                            const PString & deviceName)
   : OpalRawMediaStream(conn, mediaFormat, sessionID, isSource,
                        PSoundChannel::CreateOpenedChannel(PString::Empty(),
@@ -950,27 +1047,38 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
                                                                    : PSoundChannel::Player,
                                                           1, mediaFormat.GetClockRate(), 16),
                        true)
+  , m_soundChannelBuffers(buffers)
+  , m_soundChannelBufferTime(bufferTime)
 {
-  soundChannelBuffers = buffers;
 }
 
 
-PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameSize)
+PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 {
   /* For efficiency reasons we will not accept a packet size that is too small.
      We move it up to the next even multiple, which has a danger of the remote not
      sending an even number of our multiplier. */
-  const unsigned MinTimeMilliseconds = 10;
-  PINDEX minSize = mediaFormat.GetClockRate()/(1000/MinTimeMilliseconds)*sizeof(short);
-  if (frameSize < minSize) {
-    PTRACE(1, "Media\tClamping audio stream frame size from " << frameSize << " to minimum " << minSize);
-    frameSize = minSize;
-  }
+  const unsigned BufferTimeMilliseconds = 10;
+  PINDEX bufferSize = BufferTimeMilliseconds*mediaFormat.GetClockRate()/1000*sizeof(short);
+
+  // Calculate sound buffers from global system settings
+  PINDEX soundChannelBuffers = (m_soundChannelBufferTime+BufferTimeMilliseconds-1)/BufferTimeMilliseconds;
+  if (soundChannelBuffers < m_soundChannelBuffers)
+    soundChannelBuffers = m_soundChannelBuffers;
+
+  // Quantise dataSize up to multiple of the frame time
+  PINDEX frameSize = frameTime*sizeof(short);
+  dataSize = (dataSize+frameSize-1)/frameSize * frameSize;
+
+  // Calculte buffers needed for the data we require and increase buffers if required
+  PINDEX dataBuffersNeeded = (dataSize+bufferSize-1)/bufferSize;
+  if (soundChannelBuffers < dataBuffersNeeded)
+    soundChannelBuffers = dataBuffersNeeded;
 
   PTRACE(3, "Media\tAudio " << (IsSource() ? "source" : "sink") << " data size set to "
-         << frameSize << " bytes and " << soundChannelBuffers << " buffers.");
-  return OpalMediaStream::SetDataSize(dataSize, frameSize) &&
-         ((PSoundChannel *)m_channel)->SetBuffers(frameSize, soundChannelBuffers);
+         << dataSize << ", buffer size set to " << bufferSize << " and " << soundChannelBuffers << " buffers.");
+  return OpalMediaStream::SetDataSize(dataSize, frameTime) &&
+         ((PSoundChannel *)m_channel)->SetBuffers(bufferSize, soundChannelBuffers);
 }
 
 
@@ -1011,7 +1119,7 @@ OpalVideoMediaStream::~OpalVideoMediaStream()
 }
 
 
-PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameSize)
+PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 {
   if (inputDevice != NULL) {
     PINDEX minDataSize = inputDevice->GetMaxFrameBytes();
@@ -1024,7 +1132,7 @@ PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameSize)
       dataSize = minDataSize;
   }
 
-  return OpalMediaStream::SetDataSize(sizeof(PluginCodec_Video_FrameHeader) + dataSize, frameSize);
+  return OpalMediaStream::SetDataSize(sizeof(PluginCodec_Video_FrameHeader) + dataSize, frameTime);
 }
 
 
@@ -1117,15 +1225,16 @@ PBoolean OpalVideoMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & lengt
     return false;
 
   PTimeInterval currentGrabTime = PTimer::Tick();
-  timestamp += ((lastGrabTime - currentGrabTime)*1000/OpalMediaFormat::VideoClockRate).GetInterval();
+  timestamp += (int)((currentGrabTime - lastGrabTime).GetMilliSeconds()*OpalMediaFormat::VideoClockRate/1000);
   lastGrabTime = currentGrabTime;
 
-  marker = true;
-  length = bytesReturned + sizeof(PluginCodec_Video_FrameHeader);
-
-  if ((flags & PluginCodec_ReturnCoderRequestIFrame) != 0) {
+  if ((flags & PluginCodec_ReturnCoderRequestIFrame) != 0)
     ExecuteCommand(OpalVideoUpdatePicture());
-  }
+
+  marker = true;
+  length = bytesReturned;
+  if (length > 0)
+    length += sizeof(PluginCodec_Video_FrameHeader);
 
   if (outputDevice == NULL)
     return true;
@@ -1200,11 +1309,16 @@ OpalUDPMediaStream::OpalUDPMediaStream(OpalConnection & conn,
 {
 }
 
-
-PBoolean OpalUDPMediaStream::ReadPacket(RTP_DataFrame & Packet)
+OpalUDPMediaStream::~OpalUDPMediaStream()
 {
-  Packet.SetPayloadType(mediaFormat.GetPayloadType());
-  Packet.SetPayloadSize(0);
+  Close();
+}
+
+
+PBoolean OpalUDPMediaStream::ReadPacket(RTP_DataFrame & packet)
+{
+  packet.SetPayloadType(m_payloadType);
+  packet.SetPayloadSize(0);
 
   if (IsSink()) {
     PTRACE(1, "Media\tTried to read from sink media stream");
@@ -1219,8 +1333,8 @@ PBoolean OpalUDPMediaStream::ReadPacket(RTP_DataFrame & Packet)
   }
 
   if (rawData.GetSize() > 0) {
-    Packet.SetPayloadSize(rawData.GetSize());
-    memcpy(Packet.GetPayloadPtr(), rawData.GetPointer(), rawData.GetSize());
+    packet.SetPayloadSize(rawData.GetSize());
+    memcpy(packet.GetPayloadPtr(), rawData.GetPointer(), rawData.GetSize());
   }
 
   return true;
