@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22640 $
+ * $Revision: 24365 $
  * $Author: rjongbloed $
- * $Date: 2009-05-18 22:19:20 -0500 (Mon, 18 May 2009) $
+ * $Date: 2010-05-21 01:02:25 -0500 (Fri, 21 May 2010) $
  */
 
 #include <ptlib.h>
@@ -91,21 +91,56 @@ OpalRTPConnection::~OpalRTPConnection()
 }
 
 
+unsigned OpalRTPConnection::GetNextSessionID(const OpalMediaType & mediaType, bool isSource)
+{
+  unsigned nextSessionId = m_rtpSessions.GetNextSessionID();
+
+  if (GetMediaStream(mediaType, isSource) != NULL)
+    return nextSessionId;
+
+  OpalMediaStreamPtr mediaStream = GetMediaStream(mediaType, !isSource);
+  if (mediaStream != NULL)
+    return mediaStream->GetSessionID();
+
+  unsigned defaultSessionId = mediaType.GetDefinition()->GetDefaultSessionId();
+  if (defaultSessionId >= nextSessionId ||
+      GetMediaStream(defaultSessionId,  isSource) != NULL ||
+      GetMediaStream(defaultSessionId, !isSource) != NULL)
+    return nextSessionId;
+
+  return defaultSessionId;
+}
+
+
 RTP_Session * OpalRTPConnection::GetSession(unsigned sessionID) const
 {
   return m_rtpSessions.GetSession(sessionID);
 }
 
+
 OpalMediaSession * OpalRTPConnection::GetMediaSession(unsigned sessionID) const
 {
   return m_rtpSessions.GetMediaSession(sessionID);
 }
 
+
+RTP_UDP * OpalRTPConnection::FindSessionByLocalPort(WORD port) const
+{
+  for (list<OpalRTPSessionManager *>::const_iterator it = m_allSessions.begin(); it != m_allSessions.end(); ++it) {
+    RTP_UDP * rtp = (*it)->FindSessionByLocalPort(port);
+    if (rtp != NULL)
+      return rtp;
+  }
+
+  return NULL;
+}
+
+
 RTP_Session * OpalRTPConnection::UseSession(const OpalTransport & transport, unsigned sessionID, const OpalMediaType & mediaType, RTP_QOS * rtpqos)
 {
   RTP_Session * rtpSession = m_rtpSessions.GetSession(sessionID);
   if (rtpSession == NULL) {
-    rtpSession = CreateSession(transport, sessionID, rtpqos);
+    rtpSession = CreateSession(transport, sessionID, mediaType, rtpqos);
     m_rtpSessions.AddSession(rtpSession, mediaType);
   }
 
@@ -115,33 +150,37 @@ RTP_Session * OpalRTPConnection::UseSession(const OpalTransport & transport, uns
 
 RTP_Session * OpalRTPConnection::CreateSession(const OpalTransport & transport,
                                                             unsigned sessionID,
+                                               const OpalMediaType & mediaType,
                                                            RTP_QOS * rtpqos)
 {
   // We only support RTP over UDP at this point in time ...
   if (!transport.IsCompatibleTransport("ip$127.0.0.1")) 
     return NULL;
 
-  PIPSocket::Address localAddress;
- 
-  transport.GetLocalAddress().GetIpAddress(localAddress);
+  // create an RTP session
+  RTP_UDP * rtpSession = CreateRTPSession(sessionID, mediaType, remoteIsNAT);
+  if (rtpSession == NULL) 
+    return NULL;
 
-  OpalManager & manager = GetEndPoint().GetManager();
+  PIPSocket::Address localAddress; 
+  transport.GetLocalAddress(false).GetIpAddress(localAddress);
 
   PIPSocket::Address remoteAddress;
   transport.GetRemoteAddress().GetIpAddress(remoteAddress);
-  PNatMethod * natMethod = manager.GetNatMethod(remoteAddress);
 
-  // create an RTP session
-  RTP_UDP * rtpSession = CreateRTPSession(sessionID, remoteIsNAT);
-  if (rtpSession == NULL) 
-    return NULL;
+  OpalManager & manager = GetEndPoint().GetManager();
+  PNatMethod * natMethod = manager.GetNatMethod(remoteAddress);
 
   WORD firstPort = manager.GetRtpIpPortPair();
   WORD nextPort = firstPort;
   while (!rtpSession->Open(localAddress, nextPort, nextPort, manager.GetRtpIpTypeofService(), natMethod, rtpqos)) {
     nextPort = manager.GetRtpIpPortPair();
     if (nextPort == firstPort) {
-      PTRACE(1, "RTPCon\tNo ports available for RTP session " << sessionID << " for " << *this);
+      PTRACE(1, "RTPCon\tNo ports available for RTP session " << sessionID << ","
+                " base=" << manager.GetRtpIpPortBase() << ","
+                " max=" << manager.GetRtpIpPortMax() << ","
+                " bind=" << localAddress << ","
+                " for " << *this);
       delete rtpSession;
       return NULL;
     }
@@ -156,9 +195,8 @@ RTP_Session * OpalRTPConnection::CreateSession(const OpalTransport & transport,
 }
 
 
-RTP_UDP * OpalRTPConnection::CreateRTPSession(unsigned sessionID, bool remoteIsNAT)
+RTP_UDP * OpalRTPConnection::CreateRTPSession(unsigned sessionID, const OpalMediaType & mediaType, bool remoteIsNAT)
 {
-  OpalMediaType mediaType = OpalMediaTypeDefinition::GetMediaTypeForSessionId(sessionID);
   OpalMediaTypeDefinition * def = mediaType.GetDefinition();
   if (def == NULL) {
     PTRACE(1, "RTPCon\tNo definition for media type " << mediaType);
@@ -187,6 +225,29 @@ RTP_UDP * OpalRTPConnection::CreateRTPSession(unsigned sessionID, bool remoteIsN
 }
 
 
+bool OpalRTPConnection::ChangeSessionID(unsigned fromSessionID, unsigned toSessionID)
+{
+  PTRACE(3, "RTPCon\tChanging session ID " << fromSessionID << " to " << toSessionID);
+  if (!m_rtpSessions.ChangeSessionID(fromSessionID, toSessionID))
+    return false;
+
+  for (OpalMediaStreamPtr stream(mediaStreams, PSafeReference); stream != NULL; ++stream) {
+    if (stream->GetSessionID() == fromSessionID) {
+      stream->SetSessionID(toSessionID);
+      OpalMediaPatch * patch = stream->GetPatch();
+      if (patch != NULL) {
+        patch->GetSource().SetSessionID(toSessionID);
+        OpalMediaStreamPtr otherStream;
+        for (PINDEX i = 0; (otherStream = patch->GetSink(i)) != NULL; ++i)
+          otherStream->SetSessionID(toSessionID);
+      }
+    }
+  }
+
+  return true;
+}
+
+
 void OpalRTPConnection::ReleaseSession(unsigned sessionID, PBoolean clearAll/* = PFalse */)
 {
 #ifdef HAS_LIBZRTP
@@ -270,10 +331,7 @@ PBoolean OpalRTPConnection::SendUserInputTone(char tone, unsigned duration)
     return true;
 
   PTRACE(2, "RTPCon\tCould not send tone '" << tone << "' via RFC2833.");
-
-  //Probably need a PCM generator in here
-
-  return true;
+  return OpalConnection::SendUserInputTone(tone, duration);
 }
 
 
@@ -317,28 +375,48 @@ OpalMediaStream * OpalRTPConnection::CreateMediaStream(const OpalMediaFormat & m
       return mediaStream;
   }
 
+  if (mediaFormat.GetMediaType().GetDefinition()->UsesRTP()) {
+    if (UseSession(GetTransport(), sessionID, mediaFormat.GetMediaType(), NULL) == NULL) {
+      PTRACE(1, "RTPCon\tCreateMediaStream could not find/create session " << sessionID);
+      return NULL;
+    }
+  }
+
   OpalMediaSession * mediaSession = GetMediaSession(sessionID);
   if (mediaSession == NULL) {
-    PTRACE(1, "RTPCon\tCreateMediaStream could not find session " << sessionID);
+    PTRACE(1, "RTPCon\tUnable to create media stream for session " << sessionID);
     return NULL;
   }
 
-  return mediaSession->CreateMediaStream(mediaFormat, sessionID, isSource);
+  return PAssertNULL(mediaSession)->CreateMediaStream(mediaFormat, sessionID, isSource);
 }
 
+
+void OpalRTPConnection::AdjustMediaFormats(bool local,
+                          OpalMediaFormatList & mediaFormats,
+                               OpalConnection * otherConnection) const
+{
+  OpalConnection::AdjustMediaFormats(local, mediaFormats, otherConnection);
+
+  if (otherConnection != NULL && otherConnection != this)
+    return;
+
+  OpalMediaFormatList::iterator fmt = mediaFormats.begin();
+  while (fmt != mediaFormats.end()) {
+    if (fmt->IsTransportable())
+      ++fmt;
+    else
+      mediaFormats -= *fmt++;
+  }
+}
+
+
 void OpalRTPConnection::OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & patch)
 {
   OpalConnection::OnPatchMediaStream(isSource, patch);
-  if (patch.GetSource().GetMediaFormat().GetMediaType() == OpalMediaType::Audio()) {
-    AttachRFC2833HandlerToPatch(isSource, patch);
-#if OPAL_PTLIB_DTMF
-    if (detectInBandDTMF && isSource) {
-      patch.AddFilter(PCREATE_NOTIFIER(OnUserInputInBandDTMF), OPAL_PCM16);
-    }
-#endif
-  }
 
-  patch.SetCommandNotifier(PCREATE_NOTIFIER(OnMediaCommand), !isSource);
+  if (patch.GetSource().GetMediaFormat().GetMediaType() == OpalMediaType::Audio())
+    AttachRFC2833HandlerToPatch(isSource, patch);
 }
 
 void OpalRTPConnection::OnUserInputInlineRFC2833(OpalRFC2833Info & info, INT type)
@@ -387,9 +465,12 @@ OpalMediaSession::OpalMediaSession(const OpalMediaSession & _obj)
 
 /////////////////////////////////////////////////////////////////////////////
 
-OpalRTPMediaSession::OpalRTPMediaSession(OpalConnection & conn, const OpalMediaType & mediaType, unsigned sessionId)
+OpalRTPMediaSession::OpalRTPMediaSession(OpalConnection & conn,
+                                         const OpalMediaType & mediaType,
+                                         unsigned sessionId,
+                                         RTP_Session * rtp)
   : OpalMediaSession(conn, mediaType, sessionId)
-  , rtpSession(NULL)
+  , rtpSession(rtp)
 {
 }
 
@@ -440,15 +521,25 @@ OpalMediaStream * OpalRTPMediaSession::CreateMediaStream(const OpalMediaFormat &
 
 /////////////////////////////////////////////////////////////////////////////
 
-OpalRTPSessionManager::OpalRTPSessionManager(OpalConnection & conn)
-  : connection(conn)
+OpalRTPSessionManager::OpalRTPSessionManager(OpalRTPConnection & connection)
+  : m_connection(connection)
 {
+  m_connection.m_allSessions.push_back(this);
+}
+
+
+OpalRTPSessionManager::OpalRTPSessionManager(const OpalRTPSessionManager & other)
+  : m_connection(other.m_connection)
+  , sessions(other.sessions)
+{
+  m_connection.m_allSessions.push_back(this);
 }
 
 
 OpalRTPSessionManager::~OpalRTPSessionManager()
 {
-  PWaitAndSignal m(m_mutex);
+  PWaitAndSignal mutex(m_mutex);
+
   if (sessions.IsUnique()) {
     while (sessions.GetSize() > 0) {
       PINDEX id = sessions.GetKeyAt(0);
@@ -457,45 +548,62 @@ OpalRTPSessionManager::~OpalRTPSessionManager()
       sessions.RemoveAt(id);
     }
   }
+
+  m_connection.m_allSessions.remove(this);
+}
+
+
+unsigned OpalRTPSessionManager::GetNextSessionID()
+{
+  unsigned maxSessionID = 0;
+
+  for (PINDEX i = 0; i < sessions.GetSize(); ++i) {
+    unsigned sessionID = sessions.GetDataAt(i).sessionId;
+    if (maxSessionID < sessionID)
+      maxSessionID = sessionID;
+  }
+
+  return maxSessionID+1;
 }
 
 
 void OpalRTPSessionManager::AddSession(RTP_Session * rtpSession, const OpalMediaType & mediaType)
 {
+  if (rtpSession == NULL)
+    return;
+
   PWaitAndSignal m(m_mutex);
   
-  if (rtpSession != NULL) {
-    OpalMediaSession * session = sessions.GetAt(rtpSession->GetSessionID());
-    OpalRTPMediaSession * s;
-    if (session == NULL) {
-      s = new OpalRTPMediaSession(connection, mediaType, 0);
+  unsigned sessionID = rtpSession->GetSessionID();
+  OpalMediaSession * session = sessions.GetAt(sessionID);
+  if (session == NULL) {
+    sessions.Insert(POrdinalKey(sessionID), new OpalRTPMediaSession(m_connection, mediaType, sessionID, rtpSession));
+    PTRACE(3, "RTP\tCreating new session " << *rtpSession);
+  }
+  else {
+    OpalRTPMediaSession * s = dynamic_cast<OpalRTPMediaSession *>(session);
+    if (PAssert(s != NULL,             "RTP session type does not match") &&
+        PAssert(s->rtpSession == NULL, "Cannot add already existing session"))
       s->rtpSession = rtpSession;
-      sessions.Insert(POrdinalKey(rtpSession->GetSessionID()), s);
-      PTRACE(3, "RTP\tCreating new session " << *rtpSession);
-    }
-    else
-    {
-      s = dynamic_cast<OpalRTPMediaSession *>(session);
-      PAssert(s != NULL,             "RTP session type does not match");
-      PAssert(s->rtpSession == NULL, "Cannot add already existing session");
-      s->rtpSession = rtpSession;
-    }
   }
 }
 
+
 void OpalRTPSessionManager::AddMediaSession(OpalMediaSession * mediaSession, const OpalMediaType & /*mediaType*/)
 {
   PWaitAndSignal m(m_mutex);
-  
+
   PAssert(sessions.GetAt(mediaSession->sessionId) == NULL, "Cannot add already existing session");
   sessions.Insert(POrdinalKey(mediaSession->sessionId), mediaSession);
 }
 
+
 void OpalRTPSessionManager::ReleaseSession(unsigned PTRACE_PARAM(sessionID), PBoolean /*clearAll*/)
 {
   PTRACE(3, "RTP\tReleasing session " << sessionID);
 }
 
+
 OpalMediaSession * OpalRTPSessionManager::GetMediaSession(unsigned sessionID) const
 {
   PWaitAndSignal wait(m_mutex);
@@ -528,6 +636,49 @@ RTP_Session * OpalRTPSessionManager::GetSession(unsigned sessionID) const
   return ((OpalRTPMediaSession *)session)->rtpSession;
 }
 
+
+RTP_UDP * OpalRTPSessionManager::FindSessionByLocalPort(WORD port) const
+{
+  PWaitAndSignal m(m_mutex);
+
+  for (PINDEX i = 0; i < sessions.GetSize(); ++i) {
+    const OpalMediaSession & session = sessions.GetDataAt(i);
+    if (session.IsActive() && session.IsRTP()) {
+      RTP_UDP * rtp = dynamic_cast<RTP_UDP * >(dynamic_cast<const OpalRTPMediaSession &>(session).rtpSession);
+      if (rtp != NULL && (rtp->GetLocalDataPort() == port || rtp->GetLocalControlPort() == port))
+        return rtp;
+    }
+  }
+
+  return NULL;
+}
+
+
+bool OpalRTPSessionManager::ChangeSessionID(unsigned fromSessionID, unsigned toSessionID)
+{
+  PWaitAndSignal wait(m_mutex);
+
+  if (sessions.Contains(toSessionID)) {
+    PTRACE(2, "RTP\tAttempt to renumber session " << fromSessionID << " to existing sesion ID " << toSessionID);
+    return false;
+  }
+
+  sessions.DisallowDeleteObjects();
+  OpalMediaSession * session = sessions.RemoveAt(fromSessionID);
+  sessions.AllowDeleteObjects();
+
+  if (session == NULL)
+    return false;
+
+  OpalRTPMediaSession * rtpSession = dynamic_cast<OpalRTPMediaSession *>(session);
+  if (rtpSession != NULL)
+    rtpSession->rtpSession->SetSessionID(toSessionID);
+
+  session->sessionId = toSessionID;
+  return sessions.SetAt(POrdinalKey(toSessionID), session);
+}
+
+
 bool OpalRTPSessionManager::AllSessionsFailing()
 {
   PWaitAndSignal wait(m_mutex);
