@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24280 $
+ * $Revision: 24554 $
  * $Author: rjongbloed $
- * $Date: 2010-04-25 23:18:25 -0500 (Sun, 25 Apr 2010) $
+ * $Date: 2010-07-02 05:09:12 -0500 (Fri, 02 Jul 2010) $
  */
 
 #include <ptlib.h>
@@ -103,6 +103,7 @@ void OpalTranscoder::SetInstanceID(const BYTE * /*instance*/, unsigned /*instanc
 
 RTP_DataFrame::PayloadTypes OpalTranscoder::GetPayloadType(PBoolean input) const
 {
+  PWaitAndSignal mutex(updateMutex);
   return (input ? inputMediaFormat : outputMediaFormat).GetPayloadType();
 }
 
@@ -116,6 +117,8 @@ void OpalTranscoder::GetStatistics(OpalMediaStatistics & /*statistics*/) const
 
 PBoolean OpalTranscoder::ConvertFrames(const RTP_DataFrame & input, RTP_DataFrameList & output)
 {
+  PWaitAndSignal mutex(updateMutex);
+
   // make sure there is at least one output frame available
   if (output.IsEmpty())
     output.Append(new RTP_DataFrame(0, maxOutputSize));
@@ -165,6 +168,27 @@ OpalTranscoder * OpalTranscoder::Create(const OpalMediaFormat & srcFormat,
 }
 
 
+static bool GetBaseFormat(const OpalMediaFormatList & masterFormats,
+                          const OpalMediaFormat & capability,
+                          OpalMediaFormat & format,
+                          const char * PTRACE_PARAM(direction) = NULL)
+{
+  OpalMediaFormatList::const_iterator masterFormat = masterFormats.FindFormat(capability);
+  if (masterFormat == masterFormats.end()) {
+    format = capability;
+    PTRACE(5, "Opal\tInitial " << direction << " format from capabilities:\n" << setw(-1) << format);
+    return true;
+  }
+
+  format = *masterFormat;
+  PTRACE(5, "Opal\tInitial " << direction << " format from master:\n" << setw(-1) << format
+                           << "Merging with capability\n" << setw(-1) << capability);
+
+  OpalMediaFormat tmpCapability = capability;
+  return tmpCapability.Merge(format) && format.Update(tmpCapability);
+}
+
+
 static bool MergeFormats(const OpalMediaFormatList & masterFormats,
                          const OpalMediaFormat & srcCapability,
                          const OpalMediaFormat & dstCapability,
@@ -229,36 +253,11 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
          dstFormat = YUV420P[QCIF]          <-- No change, srcFormat is superset
    */
 
-  OpalMediaFormatList::const_iterator masterFormat = masterFormats.FindFormat(srcCapability);
-  if (masterFormat == masterFormats.end()) {
-    srcFormat = srcCapability;
-    PTRACE(5, "Opal\tInitial source format from capabilities:\n" << setw(-1) << srcFormat);
-  }
-  else {
-    srcFormat = *masterFormat;
-    PTRACE(5, "Opal\tInitial source format from master:\n" << setw(-1) << srcFormat
-                        << "Merging with capability\n" << setw(-1) << srcCapability);
-    if (!srcFormat.Merge(srcCapability))
-      return false;
-  }
+  if (!GetBaseFormat(masterFormats, srcCapability, srcFormat, PTRACE_PARAM("source")))
+    return false;
 
-  masterFormat = masterFormats.FindFormat(dstCapability);
-  if (masterFormat == masterFormats.end()) {
-    dstFormat = dstCapability;
-    PTRACE(5, "Opal\tInitial destination format from capabilities:\n" << setw(-1) << dstFormat);
-  }
-  else {
-    dstFormat = *masterFormat;
-    PTRACE(5, "Opal\tInitial destination format from master:\n" << setw(-1) << dstFormat
-                             << "Merging with capability\n" << setw(-1) << dstCapability);
-
-    OpalMediaFormat tmpCapability = dstCapability;
-    if (!tmpCapability.Merge(dstFormat))
-      return false;
-
-    if (!dstFormat.Update(tmpCapability))
-      return false;
-  }
+  if (!GetBaseFormat(masterFormats, dstCapability, dstFormat, PTRACE_PARAM("destination")))
+    return false;
 
   if (!srcFormat.Merge(dstFormat))
     return false;
@@ -457,6 +456,8 @@ PINDEX OpalFramedTranscoder::GetOptimalDataFrameSize(PBoolean input) const
 
 PBoolean OpalFramedTranscoder::Convert(const RTP_DataFrame & input, RTP_DataFrame & output)
 {
+  // Note updateMutex should already be locked at this point.
+
   if (inputIsRTP || outputIsRTP) {
 
     const BYTE * inputPtr;
