@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 23925 $
+ * $Author: rjongbloed $
+ * $Date: 2010-01-12 01:06:45 -0600 (Tue, 12 Jan 2010) $
  */
 
 #include <ptlib.h>
@@ -60,26 +60,124 @@ jitter buffer target */
 
 
 
-#if PTRACING && !defined(NO_ANALYSER)
+#if !PTRACING && !defined(NO_ANALYSER)
+#define NO_ANALYSER 1
+#endif
 
-class RTP_JitterBufferAnalyser : public PObject
-{
-    PCLASSINFO(RTP_JitterBufferAnalyser, PObject);
-  public:
-    RTP_JitterBufferAnalyser();
-    void In(DWORD time, unsigned depth, const char * extra);
-    void Out(DWORD time, unsigned depth, const char * extra);
-    void PrintOn(ostream & strm) const;
+#ifdef NO_ANALYSER
+  #define ANALYSE(inout, time, extra)
+#else
+  #define ANALYSE(inout, time, extra) analyser->inout(time, jitterBuffer.GetSize(), extra)
 
-    struct Info {
-      Info() { }
-      DWORD         time;
-      PTimeInterval tick;
-      int           depth;
-      const char *  extra;
-    } in[1000], out[1000];
-    PINDEX inPos, outPos;
-};
+  class RTP_JitterBufferAnalyser : public PObject
+  {
+      PCLASSINFO(RTP_JitterBufferAnalyser, PObject);
+
+    protected:
+      struct Info {
+        Info() { }
+        DWORD         time;
+        PTimeInterval tick;
+        int           depth;
+        const char *  extra;
+      } in[1000], out[1000];
+      PINDEX inPos, outPos;
+
+    public:
+      RTP_JitterBufferAnalyser()
+      {
+        inPos = outPos = 1;
+        in[0].time = out[0].time = 0;
+        in[0].tick = out[0].tick = PTimer::Tick();
+        in[0].depth = out[0].depth = 0;
+      }
+
+      void In(DWORD time, unsigned depth, const char * extra)
+      {
+        if (inPos < PARRAYSIZE(in)) {
+          in[inPos].tick = PTimer::Tick();
+          in[inPos].time = time;
+          in[inPos].depth = depth;
+          in[inPos++].extra = extra;
+        }
+      }
+
+      void Out(DWORD time, unsigned depth, const char * extra)
+      {
+        if (outPos < PARRAYSIZE(out)) {
+          out[outPos].tick = PTimer::Tick();
+          if (time == 0 && outPos > 0)
+            out[outPos].time = out[outPos-1].time;
+          else
+            out[outPos].time = time;
+          out[outPos].depth = depth;
+          out[outPos++].extra = extra;
+        }
+      }
+
+      void PrintOn(ostream & strm) const
+      {
+        strm << "Input samples: " << inPos << " Output samples: " << outPos << "\n"
+                "Dir\tRTPTime\tInDiff\tOutDiff\tInMode\tOutMode\t"
+                "InDepth\tOutDep\tInTick\tInDelay\tOutTick\tOutDel\tIODelay\n";
+        PINDEX ix = 1;
+        PINDEX ox = 1;
+        while (ix < inPos || ox < outPos) {
+          while (ix < inPos && (ox >= outPos || in[ix].time < out[ox].time)) {
+            strm << "In\t"
+                 << in[ix].time << '\t'
+                 << (in[ix].time - in[ix-1].time) << "\t"
+                    "\t"
+                 << in[ix].extra << "\t"
+                    "\t"
+                 << in[ix].depth << "\t"
+                    "\t"
+                 << (in[ix].tick - in[0].tick) << '\t'
+                 << (in[ix].tick - in[ix-1].tick) << "\t"
+                    "\t"
+                    "\t"
+                    "\n";
+            ix++;
+          }
+
+          while (ox < outPos && (ix >= inPos || out[ox].time < in[ix].time)) {
+            strm << "Out\t"
+                 << out[ox].time << "\t"
+                    "\t"
+                 << (out[ox].time - out[ox-1].time) << "\t"
+                    "\t"
+                 << out[ox].extra << "\t"
+                    "\t"
+                 << out[ox].depth << "\t"
+                    "\t"
+                    "\t"
+                 << (out[ox].tick - out[0].tick) << '\t'
+                 << (out[ox].tick - out[ox-1].tick) << "\t"
+                    "\n";
+            ox++;
+          }
+
+          while (ix < inPos && ox < outPos && in[ix].time == out[ox].time) {
+            strm << "I/O\t"
+                 << in[ix].time << '\t'
+                 << (in[ix].time - in[ix-1].time) << '\t'
+                 << (out[ox].time - out[ox-1].time) << '\t'
+                 << in[ix].extra << '\t'
+                 << out[ox].extra << '\t'
+                 << in[ix].depth << '\t'
+                 << out[ox].depth << '\t'
+                 << (in[ix].tick - in[0].tick) << '\t'
+                 << (in[ix].tick - in[ix-1].tick) << '\t'
+                 << (out[ox].tick - out[0].tick) << '\t'
+                 << (out[ox].tick - out[ox-1].tick) << '\t'
+                 << (out[ox].tick - in[ix].tick)
+                 << '\n';
+            ox++;
+            ix++;
+          }
+        }
+      }
+  };
 
 #endif
 
@@ -88,173 +186,116 @@ class RTP_JitterBufferAnalyser : public PObject
 
 /////////////////////////////////////////////////////////////////////////////
 
-OpalJitterBuffer::OpalJitterBuffer(unsigned _minJitterTime,
-                                   unsigned _maxJitterTime,
-                                   unsigned _time,
-                                   PINDEX stackSize)
-  : jitterThread(NULL), jitterStackSize(stackSize)
-{
-  timeUnits        = _time;
-
-  // Nothing in the buffer so far
-  maxConsecutiveMarkerBits     = 10;
-  doJitterReductionImmediately = false;
-
-  lastWriteTimestamp    = 0;
-  lastWriteTick         = 0;
-  jitterCalc            = 0;
-  jitterCalcPacketCount = 0;
-
-  currentFrame  = NULL;
-  bufferSize   = 0;
-
-  Start(_minJitterTime, _maxJitterTime);
-
-  PTRACE(4, "RTP\tOpal jitter buffer created:" << *this << " obj=" << this);
-
-#if PTRACING && !defined(NO_ANALYSER)
-  analyser = new RTP_JitterBufferAnalyser;
+OpalJitterBuffer::OpalJitterBuffer(unsigned minJitter,
+                                   unsigned maxJitter,
+                                   unsigned units,
+                                     PINDEX packetSize)
+  : timeUnits(units)
+  , maxConsecutiveMarkerBits(10)
+  , lastWriteTimestamp(0)
+  , jitterCalc(0)
+  , jitterCalcPacketCount(0)
+  , m_resetJitterBufferNow(false)
+  , currentFrame(NULL)
+#ifdef NO_ANALYSER
+  , analyser(NULL)
 #else
-  analyser = NULL;
+  , analyser(new RTP_JitterBufferAnalyser)
 #endif
-}
-
-
-void OpalJitterBuffer::Start(unsigned _minJitterTime, unsigned _maxJitterTime)
 {
-  PWaitAndSignal m(bufferMutex);
+  SetDelay(minJitter, maxJitter, packetSize);
 
-  // retreive all of the frames in use
-  if (currentFrame != NULL) {
-    PAssertNULL(currentFrame);
-    freeFrames.push_back(currentFrame);
-    currentFrame = NULL;
-  }
-  while (jitterBuffer.size() > 0) {
-    PAssertNULL(GetNewest(false));
-    freeFrames.push_back(GetNewest(true));
-  }
-
-  // Calculate number of frames to allocate, we make the assumption that the
-  // smallest packet we can possibly get is 5ms long.
-  bufferSize = _maxJitterTime/(5*timeUnits)+1;
-
-  // must be able to handle at least the max number of overruns we accept
-  bufferSize = PMAX(MAX_BUFFER_OVERRUNS, bufferSize);
-
-  // resize the free queue for the new buffer size
-  freeFrames.resize(bufferSize);
-
-  minJitterTime     = _minJitterTime;
-  maxJitterTime     = _maxJitterTime;
-  currentJitterTime = _minJitterTime;
-  targetJitterTime  = _minJitterTime;
-
-  preBuffering  = true;
-  firstReadData = true;
-
-  packetsTooLate                  = 0;
-  bufferOverruns                  = 0;
-  consecutiveBufferOverruns       = 0;
-  consecutiveMarkerBits           = 0;
-  consecutiveEarlyPacketStartTime = 0;
+  PTRACE(4, "RTP\tJitter buffer created:" << *this);
 }
 
 
 OpalJitterBuffer::~OpalJitterBuffer()
 {
-  shuttingDown = true;
-
-  if (jitterThread != NULL) {
-    PTRACE(3, "RTP\tRemoving jitter buffer " << this << ' ' << jitterThread->GetThreadName());
-    PAssert(jitterThread->WaitForTermination(10000), "Jitter buffer thread did not terminate");
-    delete jitterThread;
-    jitterThread = NULL;
-  } 
-
   // Free up all the memory allocated (jitterBuffer and freeFrames will self delete)
   delete currentFrame;
   currentFrame = NULL;
 
-#if PTRACING && !defined(NO_ANALYSER)
+#ifndef NO_ANALYSER
   PTRACE(5, "RTP\tJitter buffer analysis: size=" << bufferSize
          << " time=" << currentJitterTime << '\n' << *analyser);
   delete analyser;
 #endif
+
+  PTRACE(4, "RTP\tJitter buffer destroyed:" << *this);
 }
 
+
 void OpalJitterBuffer::PrintOn(ostream & strm) const
 {
-  strm << " size=" << bufferSize 
-	     << " delay=" << (minJitterTime/timeUnits) << '-' << (maxJitterTime/timeUnits) << " ms  /" 
-       << currentJitterTime << " (" << (currentJitterTime/timeUnits) << "ms)";
+  strm << this
+       << " size=" << bufferSize
+       << " delay=" << (minJitterTime/timeUnits) << '-'
+                    << (currentJitterTime/timeUnits) << '-'
+                    << (maxJitterTime/timeUnits) << "ms";
 }
 
-void OpalJitterBuffer::SetDelay(unsigned _minJitterDelay, unsigned _maxJitterDelay)
-{
-  PWaitAndSignal m(bufferMutex);
-  Start(_minJitterDelay, _maxJitterDelay);
-  PTRACE(3, "RTP\tJitter buffer restarted:" << *this);
-}
 
-void OpalJitterBuffer::Resume(PHandleAggregator * /*aggregator */)
-{
-  PWaitAndSignal m(bufferMutex);
-  if (jitterThread != NULL) {
-    if (shuttingDown) {
-      jitterThread->WaitForTermination();
-      delete jitterThread;
-      jitterThread= NULL;
-    }
-  }
-  shuttingDown = false;
-  jitterThread = PThread::Create(PCREATE_NOTIFIER(JitterThreadMain), "RTP Jitter");
-  jitterThread->Resume();
-}
-
-void OpalJitterBuffer::JitterThreadMain(PThread &, INT)
-{
-  OpalJitterBuffer::Entry * currentReadFrame;
-  PBoolean markerWarning;
-
-  PTRACE(4, "RTP\tJitter RTP receive thread started: " << this);
-
-  if (Init(currentReadFrame, markerWarning)) {
-
-    while (!shuttingDown) {
-      if (!PreRead(currentReadFrame, markerWarning))
-        break;
-
-      if (!OnRead(currentReadFrame, markerWarning, true))
-        break;
-    }
-
-    DeInit(currentReadFrame, markerWarning);
-  }
-
-  PTRACE(4, "RTP\tJitter RTP receive thread finished: " << this);
-}
-
-PBoolean OpalJitterBuffer::Init(Entry * & /*currentReadFrame*/, PBoolean & markerWarning)
+void OpalJitterBuffer::SetDelay(unsigned minJitterDelay, unsigned maxJitterDelay, PINDEX packetSize)
 {
   bufferMutex.Wait();
+
+  minJitterTime     = minJitterDelay;
+  maxJitterTime     = maxJitterDelay;
+  currentJitterTime = minJitterDelay;
+  targetJitterTime  = minJitterDelay;
+
+  preBuffering  = true;
+  firstReadData = true;
   markerWarning = false;
-  return true;
+
+  packetsTooLate                  = 0;
+  bufferOverruns                  = 0;
+  consecutiveBufferOverruns       = 0;
+  consecutiveMarkerBits           = 0;
+
+  // retreive all of the frames in use
+  if (currentFrame != NULL) {
+    PAssertNULL(currentFrame);
+    freeFrames.Append(currentFrame);
+    currentFrame = NULL;
+  }
+
+  while (jitterBuffer.GetSize() > 0) {
+    Entry * entry = GetNewest(true);
+    freeFrames.Append(PAssertNULL(entry));
+  }
+
+  // Calculate number of frames to allocate, we make the assumption that the
+  // smallest packet we can possibly get is 5ms long.
+  bufferSize = maxJitterDelay/(5*timeUnits)+1;
+
+  // must be able to handle at least the max number of overruns we accept
+  bufferSize = PMAX(MAX_BUFFER_OVERRUNS, bufferSize);
+
+  // resize the free queue for the new buffer size
+  while (freeFrames.GetSize() < bufferSize)
+    freeFrames.Append(new Entry(packetSize));
+
+  while (freeFrames.GetSize() > bufferSize)
+    delete freeFrames.RemoveAt(0);
+
+  PTRACE(3, "RTP\tDelays set for jitter buffer " << *this);
+
+  bufferMutex.Signal();
 }
 
-void OpalJitterBuffer::DeInit(Entry * & /*currentReadFrame*/, PBoolean & /*markerWarning*/)
-{
-}
 
-PBoolean OpalJitterBuffer::PreRead(OpalJitterBuffer::Entry * & currentReadFrame, PBoolean & /*markerWarning*/)
+OpalJitterBuffer::Entry * OpalJitterBuffer::GetAvailableEntry()
 {
+  OpalJitterBuffer::Entry * availableEntry;
+
   // Get the next free frame available for use for reading from the RTP
   // transport. Place it into a parking spot.
-  if (freeFrames.size() > 0) {
+  if (freeFrames.GetSize() > 0) {
 
     // Take the next free frame and make it the current for reading
-    currentReadFrame = freeFrames.front();
-    freeFrames.pop_front();
+    availableEntry = &freeFrames.front();
+    freeFrames.RemoveAt(0);
     PTRACE_IF(2, consecutiveBufferOverruns > 1,
                 "RTP\tJitter buffer full, threw away "
                 << consecutiveBufferOverruns << " oldest frames");
@@ -262,47 +303,50 @@ PBoolean OpalJitterBuffer::PreRead(OpalJitterBuffer::Entry * & currentReadFrame,
   }
   else {
     // We have a full jitter buffer, need a new frame so take the oldest one
-    PAssert(jitterBuffer.size() > 0, "Cannot find free frame in jitter buffer");
-    currentReadFrame = GetOldest(true);
+    PAssert(jitterBuffer.GetSize() > 0, "Cannot find free frame in jitter buffer");
+    availableEntry = GetOldest(true);
 
     bufferOverruns++;
     consecutiveBufferOverruns++;
     if (consecutiveBufferOverruns > MAX_BUFFER_OVERRUNS) {
       PTRACE(2, "RTP\tJitter buffer continuously full, throwing away entire buffer.");
-      while (jitterBuffer.size() > 0) 
-        freeFrames.push_front(GetOldest(true));
+      while (jitterBuffer.GetSize() > 0) 
+        freeFrames.InsertAt(0, GetOldest(true));
       preBuffering = true;
     }
     else {
-      PTRACE_IF(2, (consecutiveBufferOverruns == 1) && (currentReadFrame != NULL),
+      PTRACE_IF(2, (consecutiveBufferOverruns == 1) && (availableEntry != NULL),
                 "RTP\tJitter buffer full, throwing away oldest frame ("
-                << currentReadFrame->GetTimestamp() << ')');
+                << availableEntry->GetTimestamp() << ')');
     }
   }
 
-  bufferMutex.Signal();
+  return availableEntry;
+}
 
+
+PBoolean OpalJitterBuffer::WriteData(const RTP_DataFrame & frame)
+{
+  PWaitAndSignal mutex(bufferMutex);
+
+  OpalJitterBuffer::Entry * availableEntry = GetAvailableEntry();
+  if (availableEntry == NULL)
+    return false;
+
+  *dynamic_cast<RTP_DataFrame *>(availableEntry) = frame;
+  InternalWriteData(availableEntry);
   return true;
 }
 
-PBoolean OpalJitterBuffer::OnRead(OpalJitterBuffer::Entry * & currentReadFrame, PBoolean & markerWarning, PBoolean loop)
-{
-  do {
-    // Keep reading from the RTP transport frames
-    if (!OnReadPacket(*currentReadFrame, loop)) {
-      if (currentReadFrame != NULL)
-        delete currentReadFrame;  // Destructor won't delete this one, so do it here.
-      shuttingDown = true; // Flag to stop the reading side thread
-      PTRACE(3, "RTP\tJitter RTP receive thread ended");
-      return false;
-    }
-  } while (currentReadFrame->GetSize() == 0);
 
-  currentReadFrame->tick = PTimer::Tick();
+void OpalJitterBuffer::InternalWriteData(OpalJitterBuffer::Entry * availableEntry)
+{
+  PAssertNULL(availableEntry);
+  availableEntry->tick = PTimer::Tick();
 
   if (consecutiveMarkerBits < maxConsecutiveMarkerBits) {
-    if (currentReadFrame != NULL && currentReadFrame->GetMarker()) {
-      PTRACE(3, "RTP\tReceived start of talk burst: " << currentReadFrame->GetTimestamp());
+    if (availableEntry != NULL && availableEntry->GetMarker()) {
+      PTRACE(3, "RTP\tReceived start of talk burst: " << availableEntry->GetTimestamp());
       //preBuffering = true;
       consecutiveMarkerBits++;
     }
@@ -310,47 +354,29 @@ PBoolean OpalJitterBuffer::OnRead(OpalJitterBuffer::Entry * & currentReadFrame,
       consecutiveMarkerBits = 0;
   }
   else {
-    if (currentReadFrame != NULL && currentReadFrame->GetMarker())
-      currentReadFrame->SetMarker(false);
+    if (availableEntry != NULL && availableEntry->GetMarker())
+      availableEntry->SetMarker(false);
     if (!markerWarning && consecutiveMarkerBits == maxConsecutiveMarkerBits) {
       markerWarning = true;
       PTRACE(2, "RTP\tEvery packet has Marker bit, ignoring them from this client!");
     }
   }
-    
-#if PTRACING && !defined(NO_ANALYSER)
-  analyser->In(currentReadFrame->GetTimestamp(), jitterBuffer.size(), preBuffering ? "PreBuf" : "");
-#endif
-
-  // Queue the frame for playing by the thread at other end of jitter buffer
-  bufferMutex.Wait();
 
   // Have been reading a frame, put it into the queue now, at correct position
-  if (jitterBuffer.size() == 0) {
-    PAssertNULL(currentReadFrame);
-    jitterBuffer.push_front(currentReadFrame);
-  }
-  else {
-    DWORD time = currentReadFrame->GetTimestamp();
+  DWORD time = availableEntry->GetTimestamp();
 
-    // add the entry to the jitter buffer
-    FrameQueue::iterator r;
-    bool inserted = false;
-    for (r = jitterBuffer.begin(); r != jitterBuffer.end(); ++r) {
-      if (time < (*r)->GetTimestamp() || ((time == (*r)->GetTimestamp()) && (currentReadFrame->GetSequenceNumber() < (*r)->GetSequenceNumber()))) {
-        PAssertNULL(currentReadFrame);
-        jitterBuffer.insert(r, currentReadFrame);
-        inserted = true;
-        break;
-      }
-    }
-    if (!inserted) {
-      PAssertNULL(currentReadFrame);
-      jitterBuffer.push_back(currentReadFrame);
+  // add the entry to the jitter buffer
+  for (FrameQueue::iterator r = jitterBuffer.begin(); r != jitterBuffer.end(); ++r) {
+    if (time < r->GetTimestamp() || ((time == r->GetTimestamp()) && (availableEntry->GetSequenceNumber() < r->GetSequenceNumber()))) {
+      PAssertNULL(availableEntry);
+      jitterBuffer.Insert(*r, availableEntry);
+      ANALYSE(In, time, "OutOfOrder");
+      return;
     }
   }
 
-  return true;
+  jitterBuffer.Append(availableEntry);
+  ANALYSE(In, time, preBuffering ? "PreBuf" : "");
 }
 
 
@@ -361,17 +387,12 @@ PBoolean OpalJitterBuffer::ReadData(RTP_DataFrame & frame)
 
   PWaitAndSignal mutex(bufferMutex);
 
-  if (shuttingDown) {
-    PTRACE(2, "Jitter\tBuffer shutting down.");
-    return false;
-  }
-
   /*Free the frame just written to codec, putting it back into
     the free list and clearing the parking spot for it.
    */
   if (currentFrame != NULL) {
     PAssertNULL(currentFrame);
-    freeFrames.push_back(currentFrame);
+    freeFrames.Append(currentFrame);
     currentFrame = NULL;
   }
 
@@ -380,16 +401,14 @@ PBoolean OpalJitterBuffer::ReadData(RTP_DataFrame & frame)
     special member so can unlock the mutex while the writer thread has its
     way with the buffer.
    */
-  if (jitterBuffer.size() == 0) {
+  if (jitterBuffer.GetSize() == 0) {
     /*No data to play! We ran the buffer down to empty, restart buffer by
       setting flag that will fill it again before returning any data.
      */
     preBuffering = true;
     currentJitterTime = targetJitterTime;
 
-#if PTRACING && !defined(NO_ANALYSER)
-    analyser->Out(0, jitterBuffer.size(), "Empty");
-#endif
+    ANALYSE(Out, 0, "Empty");
     return true;
   }
 
@@ -424,9 +443,7 @@ PBoolean OpalJitterBuffer::ReadData(RTP_DataFrame & frame)
 
     // If oldest frame has not been in the buffer long enough, don't return anything yet
     if ((PTimer::Tick() - GetOldest(false)->tick).GetInterval() * timeUnits < currentJitterTime/2) {
-#if PTRACING && !defined(NO_ANALYSER)
-      analyser->Out(oldestTimestamp, jitterBuffer.size(), "PreBuf");
-#endif
+      ANALYSE(Out, oldestTimestamp, "PreBuf");
       return true;
     }
 
@@ -446,18 +463,13 @@ PBoolean OpalJitterBuffer::ReadData(RTP_DataFrame & frame)
   
   if (shortSilence) {
     // It is not yet time for something in the buffer
-#if PTRACING && !defined(NO_ANALYSER)
-    analyser->Out(oldestTimestamp, jitterBuffer.size(), "Wait");
-#endif
+    ANALYSE(Out, oldestTimestamp, "Wait");
     lastWriteTimestamp = 0;
     lastWriteTick = 0;
     return true;
   }
 
   // Detach oldest packet from the list, put into parking space
-#if PTRACING && !defined(NO_ANALYSER)
-  analyser->Out(oldestTimestamp, jitterBuffer.size(), requestedTimestamp >= oldestTimestamp ? "" : "Late");
-#endif
   currentFrame = GetOldest(true);
 
   // Calculate the jitter contribution of this frame
@@ -516,7 +528,7 @@ PBoolean OpalJitterBuffer::ReadData(RTP_DataFrame & frame)
   lastWriteTimestamp = currentFrame->GetTimestamp();
   lastWriteTick      = currentFrame->tick;
 
-  if (jitterBuffer.size() != 0) {
+  if (jitterBuffer.GetSize() != 0) {
 
     // If exceeded current jitter buffer time delay:
     DWORD currentJitterLen = newestTimestamp - currentFrame->GetTimestamp();
@@ -532,13 +544,14 @@ PBoolean OpalJitterBuffer::ReadData(RTP_DataFrame & frame)
       if (firstReadData) {
         PTRACE(4, "RTP\tJitter buffer length exceed was prior to first write. Not increasing buffer size");
         while ((newestTimestamp - currentFrame->GetTimestamp()) > currentJitterTime) {
-          PAssertNULL(currentFrame);
-          freeFrames.push_back(currentFrame);
+          ANALYSE(Out, currentFrame->GetTimestamp(), "Overrun");
+          freeFrames.Append(currentFrame);
           currentFrame = GetOldest(true);
         }
-        
+
         firstReadData = false;
         frame = *currentFrame;
+        ANALYSE(Out, currentFrame->GetTimestamp(), "");
         return true;
       }
 
@@ -555,10 +568,10 @@ PBoolean OpalJitterBuffer::ReadData(RTP_DataFrame & frame)
         currentJitterTime = maxJitterTime;
       
         // Throw away the oldest frame and move everything up
-        PAssertNULL(currentFrame);
-        freeFrames.push_back(currentFrame);
-        currentFrame = jitterBuffer.front();
-        jitterBuffer.pop_front();
+        ANALYSE(Out, currentFrame->GetTimestamp(), "Late");
+        freeFrames.Append(currentFrame);
+        currentFrame = &jitterBuffer.front();
+        jitterBuffer.RemoveAt(0);
       }
 
       // Now change the jitter time to cope with the new size
@@ -587,12 +600,12 @@ PBoolean OpalJitterBuffer::ReadData(RTP_DataFrame & frame)
 
   /* If using immediate jitter reduction (rather than waiting for silence opportunities)
   then trash oldest frames as necessary to reduce the size of the jitter buffer */
-  if (targetJitterTime < currentJitterTime && doJitterReductionImmediately && jitterBuffer.size() != 0) {
+  if (targetJitterTime < currentJitterTime && m_resetJitterBufferNow && jitterBuffer.GetSize() != 0) {
     while ((GetNewest(false)->GetTimestamp() - currentFrame->GetTimestamp()) > targetJitterTime){
 
       // Throw away the newest entries
       PAssertNULL(GetNewest(false));
-      freeFrames.push_back(GetNewest(true));
+      freeFrames.Append(GetNewest(true));
 
       // Reset jitter calculation baseline
       lastWriteTimestamp = 0;
@@ -602,27 +615,152 @@ PBoolean OpalJitterBuffer::ReadData(RTP_DataFrame & frame)
     currentJitterTime = targetJitterTime;
     PTRACE(3, "RTP\tJitter buffer size decreased to "
         << currentJitterTime << " (" << (currentJitterTime/timeUnits) << "ms)");
+    m_resetJitterBufferNow = false;
   }
 
   firstReadData = false;
   frame = *currentFrame;
+  ANALYSE(Out, currentFrame->GetTimestamp(), "");
   return true;
 }
 
 
+OpalJitterBuffer::Entry * OpalJitterBuffer::GetNewest(bool pop)
+{
+  Entry * e = &jitterBuffer.back();
+  if (pop)
+    jitterBuffer.RemoveAt(jitterBuffer.GetSize()-1);
+  return e;
+}
+
+
+OpalJitterBuffer::Entry * OpalJitterBuffer::GetOldest(bool pop)
+{
+  Entry * e = &jitterBuffer.front();
+  if (pop)
+    jitterBuffer.RemoveAt(0);
+  return e;
+}
+
+
+/////////////////////////////////////////////////////////////////////////////
+
+OpalJitterBufferThread::OpalJitterBufferThread(unsigned minJitterDelay,
+                                               unsigned maxJitterDelay,
+                                               unsigned timeUnits,
+                                                 PINDEX packetSize)
+  : OpalJitterBuffer(minJitterDelay, maxJitterDelay, timeUnits, packetSize)
+  , m_jitterThread(NULL)
+  , m_running(true)
+{
+}
+
+
+OpalJitterBufferThread::~OpalJitterBufferThread()
+{
+  WaitForThreadTermination(); // Failsafe, should be called from derived class dtor
+}
+
+
+void OpalJitterBufferThread::StartThread()
+{
+  bufferMutex.Wait();
+
+  if (m_jitterThread == NULL) {
+    m_jitterThread = PThread::Create(PCREATE_NOTIFIER(JitterThreadMain), "RTP Jitter");
+    m_jitterThread->Resume();
+  }
+
+  bufferMutex.Signal();
+}
+
+
+void OpalJitterBufferThread::WaitForThreadTermination()
+{
+  m_running = false;
+
+  bufferMutex.Wait();
+  PThread * jitterThread = m_jitterThread;
+  m_jitterThread = NULL;
+  bufferMutex.Signal();
+
+  if (jitterThread != NULL) {
+    PTRACE(3, "RTP\tWaiting for thread " << jitterThread->GetThreadName() << " on jitter buffer " << *this);
+    PAssert(jitterThread->WaitForTermination(10000), "Jitter buffer thread did not terminate");
+    delete jitterThread;
+  } 
+}
+
+
+PBoolean OpalJitterBufferThread::ReadData(RTP_DataFrame & frame)
+{
+  if (m_running)
+    return OpalJitterBuffer::ReadData(frame);
+
+  PTRACE(3, "Jitter\tShutting down " << *this);
+  return false;
+}
+
+
+void OpalJitterBufferThread::JitterThreadMain(PThread &, INT)
+{
+  PTRACE(4, "RTP\tJitter receive thread started: " << *this);
+
+  bufferMutex.Wait();
+
+  while (m_running) {
+    OpalJitterBuffer::Entry * availableEntry = GetAvailableEntry();
+
+    do {
+      bufferMutex.Signal();
+
+      // Keep reading from the RTP transport frames
+      if (!OnReadPacket(*availableEntry, true)) {
+        delete availableEntry;  // Destructor won't delete this one, so do it here.
+        m_running = false; // Flag to stop the reading side thread
+        goto exitThread;
+      }
+    } while (availableEntry->GetSize() == 0);
+
+    bufferMutex.Wait();
+    InternalWriteData(availableEntry);
+  }
+
+  bufferMutex.Signal();
+
+exitThread:
+  PTRACE(4, "RTP\tJitter receive thread finished: " << *this);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 RTP_JitterBuffer::RTP_JitterBuffer(RTP_Session & sess,
                                         unsigned minJitterDelay,
                                         unsigned maxJitterDelay,
                                         unsigned time,
-                                          PINDEX stackSize)
-  : OpalJitterBuffer(minJitterDelay, maxJitterDelay, time, stackSize),
+                                          PINDEX packetSize)
+  : OpalJitterBufferThread(minJitterDelay, maxJitterDelay, time, packetSize),
     session(sess)
 {
-    PTRACE(6, "RTP_JitterBuffer\tConstructor" << *this);
+  StartThread();
 }
 
+
+RTP_JitterBuffer::~RTP_JitterBuffer()
+{
+  PTRACE(4, "RTP\tDestroying jitter buffer " << *this);
+
+  m_running = false;
+  bool reopen = session.Close(true);
+
+  WaitForThreadTermination();
+
+  if (reopen)
+    session.Reopen(true);
+}
+
+
 PBoolean RTP_JitterBuffer::OnReadPacket(RTP_DataFrame & frame, PBoolean loop)
 {
   PBoolean success = session.ReadData(frame, loop);
@@ -630,110 +768,5 @@ PBoolean RTP_JitterBuffer::OnReadPacket(RTP_DataFrame & frame, PBoolean loop)
   return success;
 }
 
-/////////////////////////////////////////////////////////////////////////////
-
-
-#if PTRACING && !defined(NO_ANALYSER)
-
-RTP_JitterBufferAnalyser::RTP_JitterBufferAnalyser()
-{
-  inPos = outPos = 1;
-  in[0].time = out[0].time = 0;
-  in[0].tick = out[0].tick = PTimer::Tick();
-  in[0].depth = out[0].depth = 0;
-}
-
-
-void RTP_JitterBufferAnalyser::In(DWORD time, unsigned depth, const char * extra)
-{
-  if (inPos < PARRAYSIZE(in)) {
-    in[inPos].tick = PTimer::Tick();
-    in[inPos].time = time;
-    in[inPos].depth = depth;
-    in[inPos++].extra = extra;
-  }
-}
-
-
-void RTP_JitterBufferAnalyser::Out(DWORD time, unsigned depth, const char * extra)
-{
-  if (outPos < PARRAYSIZE(out)) {
-    out[outPos].tick = PTimer::Tick();
-    if (time == 0 && outPos > 0)
-      out[outPos].time = out[outPos-1].time;
-    else
-      out[outPos].time = time;
-    out[outPos].depth = depth;
-    out[outPos++].extra = extra;
-  }
-}
-
-
-void RTP_JitterBufferAnalyser::PrintOn(ostream & strm) const
-{
-  strm << "Input samples: " << inPos << " Output samples: " << outPos << "\n"
-          "Dir\tRTPTime\tInDiff\tOutDiff\tInMode\tOutMode\t"
-          "InDepth\tOutDep\tInTick\tInDelay\tOutTick\tOutDel\tIODelay\n";
-  PINDEX ix = 1;
-  PINDEX ox = 1;
-  while (ix < inPos || ox < outPos) {
-    while (ix < inPos && (ox >= outPos || in[ix].time < out[ox].time)) {
-      strm << "In\t"
-           << in[ix].time << '\t'
-           << (in[ix].time - in[ix-1].time) << "\t"
-              "\t"
-           << in[ix].extra << "\t"
-              "\t"
-           << in[ix].depth << "\t"
-              "\t"
-           << (in[ix].tick - in[0].tick) << '\t'
-           << (in[ix].tick - in[ix-1].tick) << "\t"
-              "\t"
-              "\t"
-              "\n";
-      ix++;
-    }
-
-    while (ox < outPos && (ix >= inPos || out[ox].time < in[ix].time)) {
-      strm << "Out\t"
-           << out[ox].time << "\t"
-              "\t"
-           << (out[ox].time - out[ox-1].time) << "\t"
-              "\t"
-           << out[ox].extra << "\t"
-              "\t"
-           << out[ox].depth << "\t"
-              "\t"
-              "\t"
-           << (out[ox].tick - out[0].tick) << '\t'
-           << (out[ox].tick - out[ox-1].tick) << "\t"
-              "\n";
-      ox++;
-    }
-
-    while (ix < inPos && ox < outPos && in[ix].time == out[ox].time) {
-      strm << "I/O\t"
-           << in[ix].time << '\t'
-           << (in[ix].time - in[ix-1].time) << '\t'
-           << (out[ox].time - out[ox-1].time) << '\t'
-           << in[ix].extra << '\t'
-           << out[ox].extra << '\t'
-           << in[ix].depth << '\t'
-           << out[ox].depth << '\t'
-           << (in[ix].tick - in[0].tick) << '\t'
-           << (in[ix].tick - in[ix-1].tick) << '\t'
-           << (out[ox].tick - out[0].tick) << '\t'
-           << (out[ox].tick - out[ox-1].tick) << '\t'
-           << (out[ox].tick - in[ix].tick)
-           << '\n';
-      ox++;
-      ix++;
-    }
-  }
-}
-
-
-#endif
-
 
 /////////////////////////////////////////////////////////////////////////////
