@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 22029 $
- * $Author: csoutheren $
- * $Date: 2009-02-10 05:25:00 +0000 (Tue, 10 Feb 2009) $
+ * $Revision: 24092 $
+ * $Author: rjongbloed $
+ * $Date: 2010-03-04 00:22:29 -0600 (Thu, 04 Mar 2010) $
  */
 
 /*
@@ -72,7 +72,7 @@ SDPMediaDescription * OpalFaxMediaType::CreateSDPMediaDescription(const OpalTran
 /////////////////////////////////////////////////////////////////////////////
 
 SDPFaxMediaDescription::SDPFaxMediaDescription(const OpalTransportAddress & address)
-  : SDPMediaDescription(address)
+  : SDPMediaDescription(address, OpalMediaType::Fax())
 {
   t38Attributes.SetAt("T38FaxRateManagement", "transferredTCF");
   t38Attributes.SetAt("T38FaxVersion",        "0");
@@ -92,7 +92,15 @@ PString SDPFaxMediaDescription::GetSDPMediaType() const
 
 SDPMediaFormat * SDPFaxMediaDescription::CreateSDPMediaFormat(const PString & portString)
 {
-  return new SDPMediaFormat(*this, RTP_DataFrame::DynamicBase, portString);
+  const OpalMediaFormat mediaFormat(RTP_DataFrame::DynamicBase, 0, portString, "sip");
+  if (mediaFormat.IsEmpty()) {
+    PTRACE(2, "SDPFax\tCould not find media format for " << portString);
+    return NULL;
+  }
+
+  PTRACE(3, "SDPFax\tUsing RTP payload " << mediaFormat.GetPayloadType() << " for " << portString);
+
+  return new SDPMediaFormat(*this, mediaFormat);
 }
 
 
